<?php

/*
 * définititon des actions à distance sur les postes
 *
 * * @Projet LCS / SambaEdu
 *
 * @Auteurs Equipe Sambaedu
 *
 * @Version
 *
 * @Note: AD :
 * - netbootinitialization => id
 * - netbootmachinefilepath => token
 *
 * les actions en cours sont enregistrées dans un tableau caché par apcu.
 *
 * action : array(id_xx => array( 'type' , machines => array('token', 'action'), 'statut', 'progress' )))
 *
 * @Licence Distribue sous la licence GPL
 *
 */

/**
 * Corrige si besoin les comptes utilisés pour les actions automatisées
 *
 * @param unknown $config
 */
function check_system_accounts($config)
{

    // utilisateur se4install pour les install auto
    if (empty($config['se4install_name'])) {
        set_config('se4install_name', "se4install", "clients");
    }

    if (empty($config['se4install_passwd'])) {
        set_config('se4install_passwd', create_random_password(8, false), "clients");
    }
    if (! userexist($config, $config['se4install_name']))
        create_user($config, $config['se4install_name'], "install", "auto", $config['se4install_passwd'], "", "", "");
    if (count(search_ad($config, $config['se4install_name'], "memberof", "Domain Admins")) == 0)
        groupaddmember($config, $config['se4install_name'], "Domain Admins");
    if (! user_valid_passwd($config, $config['se4install_name'], $config['se4install_passwd']))
        usersetpassword($config, $config['se4install_name'], $config['se4install_passwd']);
    // suppression adminse qui ne doit pas être au domaine
    if (! empty($config['adminse_name']) && userexist($config, $config['adminse_name']) && ! in_array($config['adminse_name'], [
        "admin",
        "administrator",
        $config['se4install_name']
    ]))
        userdel($config, $config['adminse_name']);
}

/**
 * récupére les infos des disques du modèle, envoyé par lsblkid
 * par defaut toutes les partitions sont clonées
 * le statut contient l'avancement du clonage (0-100), -1 en cas d'erreur
 *
 * @param
 *            $config
 * @param $partition :
 *            sortie de lsblkid
 * @return array[]|number
 */
function get_disks($config, $partitions)
{
    $partitions = explode("\n", $partitions);
    $disks = array();
    $index = - 1;
    foreach ($partitions as $ligne) {
        $match = array();
        if (preg_match("@^(/dev/sd[a-z]|/dev/nvme[0-9]n[0-9])\s*$@", $ligne, $match)) {
            $index ++;
            $disks[] = array(
                'device' => $match[1],
                'clone' => true
            );
            $indexp = 0;
        } elseif (preg_match("@^(/dev/sd[a-z][0-9]+|/dev/nvme[0-9]n[0-9]p[0-9]+)\s+(.*)\s*$@", $ligne, $match)) {
            $disks[$index]['partitions'][$indexp ++] = array(
                'device' => $match[1],
                'fs' => $match[2],
                'clone' => true
            );
            /*
             * $disks[$match[1]][$match[2]]['statut'] = 0; //pour info
             * $disks[$match[1]][$match[2]]['start'] = 0; //pour info
             * $disks[$match[1]][$match[2]]['size'] = 0; //pour info
             * $disks[$match[1]][$match[2]]['name'] = ""; //pour info
             * $disks[$match[1]][$match[2]]['type'] = ""; //pour info
             * $disks[$match[1]][$match[2]]['uuid'] = ""; //pour info
             *
             */
        }
    }
    return $disks;
}

function set_disk_size(&$disks, $sfdisk)
{
    $partitions = explode("\n", $sfdisk);
    foreach ($partitions as $ligne) {
        $match = array();
        if (preg_match("@^(/dev/sd[a-z][0-9]|n0p[0-9]+|/dev/nvme[0-9]n[0-9]p[0-9]+)\s+.*size=\s+([0-9]+).*$@", $ligne, $match)) {
            foreach ($disks as $index => $disk) {
                foreach ($disk['partitions'] as $indexp => $partition) {
                    if ($partition['device'] == $match[1]) {
                        $disks[$index]['partitions'][$indexp]['size'] = (int) $match[2] / 2;
                        break;
                    }
                }
            }
        }
    }
}

/**
 * enregistre l'avanceemnt du clonage à partir des infos envoyées par partclone
 *
 * @param string $id
 * @param array $progress
 */
function set_progress($id, $progress)
{
    $match = array();
    if (preg_match("/([0-9.]+)%/", $progress, $match)) {
        $actions = fetch_action();
        if ($actions[$id]['type'] == "clonage") {
            foreach ($actions[$id] as $membre) {
                if (! empty($membre['role'])) {
                    if ($membre['role'] == 'modele') {
                        $etape = $membre['ret'];
                    }
                }
            }
            $disks = $actions[$id]['disks'];
            $total = get_total_size($disks);
            $devs = get_clone_parts($disks);
            $fait = round(get_total_size($disks, $etape) + $devs[$etape]['size'] * ($match[1] / 100 - 1), 1);
        } else {
            $total = 100;
            $fait = $match[1];
        }
        $actions[$id]['progress'] = array(
            'total' => $total,
            'fait' => $fait,
            'avancement' => round($fait / $total * 100, 1)
        );
        apcu_store('actions', $actions);
    }
}

function get_clone_disks($disks)
{
    $list = array();
    foreach ($disks as $disk) {
        if ($disk['clone'])
            $list[] = $disk['device'];
    }
    return $list;
}

function get_clone_parts($disks)
{
    $list = array();
    foreach ($disks as $disk) {
        if ($disk['clone'])
            foreach ($disk['partitions'] as $part)
                if ($part['clone'])
                    $list[] = array(
                        'device' => $part['device'],
                        'fs' => $part['fs'],
                        'size' => $part['size']
                    );
    }
    return $list;
}

function get_total_size($disks, $index = "")
{
    $size = $i = 0;
    foreach ($disks as $disk) {
        if ($disk['clone'])
            foreach ($disk['partitions'] as $part)
                if ($part['clone']) {
                    $size = $size + $part['size'];
                    if (! empty($index) && ++ $i == $index)
                        return $size;
                }
    }
    return $size;
}

/**
 * incremente le port udp utilisé par le clonage, de façon à etre unique et pair
 *
 * @param
 *            string
 * @return number
 */
function get_clone_port($id)
{
    $port = 9000;
    $ports = array();
    $actions = fetch_action();
    if (! empty($actions[$id]['port'])) {
        $port = $actions[$id]['port'];
    } else {
        foreach ($actions as $action) {
            if (! empty($action['port']))
                $ports[] = $action['port'];
        }
        while (in_array($port, $ports)) {
            $port = $port + 2;
        }
    }
    $actions[$id]['port'] = $port;
    apcu_store('actions', $actions);
    return $port;
}

function get_action_member($config, $id, $role)
{
    $list = array();
    foreach (fetch_action()[$id] as $token => $membre) {
        if (! empty($membre['role'])) {
            if ($membre['role'] == $role) {
                $list[] = search_machine($config, $token)['cn'];
            }
        }
    }
    return $list;
}

function set_statut($id, $statut = "")
{
    $actions = fetch_action();
    $script = $actions[$id]['statut'] ?? "inconnu";
    foreach ($actions[$id] as $membre) {
        if (! empty($membre['role'])) {
            if ($membre['role'] == 'modele') {
                $script = $membre['script'];
            }
        }
    }
    if (($actions[$id]['type'] != "clonage" || $script == "emit-partitions") && ! empty($statut)) {
        $script = $statut;
    }
    $actions[$id]['statut'] = $script;
    apcu_store('actions', $actions);
    return $script;
}

function wait_clone($id)
{
    $pret = true;
    $nb = $p = 0;
    foreach (fetch_action()[$id] as $membre) {
        if (! empty($membre['role'])) {
            if ($membre['role'] == 'modele') {
                $etape = $membre['ret'] + 1;
            }
        }
    }
    foreach (fetch_action()[$id] as $membre) {
        if (! empty($membre['role'])) {
            if ($membre['role'] == 'clone') {
                $nb ++;
                $mp = (($membre['script'] == "receive-partitions") && ($membre['ret'] == $etape));
                if ($mp)
                    $p ++;
                $pret = ($pret && $mp);
            }
        }
    }
    // il faut qu'au moins un clone ait été défini, sinon on attend
    $pret = $pret && ($nb > 0);
    return array(
        'pret' => $pret,
        'clones' => "(" . $p . "/" . $nb . ")"
    );
}

function toggle_action_keep($id)
{
    $actions = fetch_action();
    if (isset($actions[$id]['keep'])) {
        if ($actions[$id]['keep'])
            $actions[$id]['keep'] = false;
        else
            $actions[$id]['keep'] = true;
    } else {
        $actions[$id]['keep'] = true;
    }
    apcu_store('actions', $actions);
}

function refresh_page($form)
{
    // <!--meta http-equiv='Refresh' CONTENT='30;URL=$page'/--!>
    $stylecss = "/elements/style_sheets/sambaedu.css";
    $html = "    <meta http-equiv='Content-Type' content='text/html; charset=UTF-8' />
<link type='text/css' rel='stylesheet' href='$stylecss' />
<body>
<div id='decompte' style='float: right; border: 1px solid black;'></div> 
<script type='text/javascript'>
    cpt=30;
    compte_a_rebours='y';
    
    
    /**
     * Decompte le temps 
     * @language Javascript
     * @Parametres
     * @return le decompte qui s'affiche
     */
    
    
    function decompte(cpt){
        if(compte_a_rebours=='y'){
            document.getElementById('decompte').innerHTML=cpt;
            if(cpt>0){
                cpt--;
            }
            else{
                    document." . $form . ".submit()
           }
            
            setTimeout(\"decompte(\"+cpt+\")\",1000);
    	}
	   else{
		  document.getElementById('decompte').style.display='none';
       }
    }            
    decompte(cpt);
</script>\n
</body>\n";

    return $html;
}

function form_actions_statut($type = "")
{
    $html = "<TABLE border=1 width=\"80%\">\n";
    $html .= "<th colspan=5 class=menuheader style=\"height: 30\">Liste des actions</th>\n";

    $html .= "<tr><td class='menuheader' align=\"center\">Identifiant</td>";
    $html .= "<td class='menuheader' align=\"center\">Type</td>";
    $html .= "<td class='menuheader' align=\"center\">Avancement</td>";
    $html .= "<td class='menuheader' align=\"center\">Statut</td>";
    $html .= "<td class='menuheader' align=\"center\">Selectionner</td>";
    $html .= "</tr>\n";
    $actions = fetch_action();

    foreach ($actions as $id => $action) {
        if (($id != "default") && (empty($type) || $type == $action['type'])) {
            $statut = set_statut($id);
            $progress = $action['progress']['avancement'] ?? "";
            $html .= "<tr><td>" . $id . "</td>";
            $html .= "<td>" . $action['type'] . "</td>";
            // $html .= "<td><label for=\"" . $id . "\">fait : " . $action['progress']['avancement'] . "%</label>";
            // $html .= "<progress id=\"" . $id . "\" max=\"100\" value=" . $action['progress']['avancement'] . "> " . $action['progress']['avancement'] . "% </progress></td>";
            $html .= "<td><progress  max=\"100\" value=" . $progress . "> " . $progress . "% </progress>" . $progress . " %</td>";
            $html .= "<td>" . $statut . "</td>";
            $html .= "<td align=\"center\"><input type=\"checkbox\" name=\"id\" value=\"" . $id . "\"></td>";
            $html .= "</tr>\n";
        }
    }
    $html .= "</TABLE>";
    // $html .= form_action_type($type);
    return $html;
}

function form_action_type($type = "voir")
{
    $types = array(
        "voir" => "Voir l'action",
        "supprimer" => "Supprimer l'action",
        "clonage" => "Clonage en direct",
        "install" => "installation automatique des postes",
        "image" => "sauvegarde et clonage à partir d'une image",
        "renomme" => "renommage d'un poste",
        "post" => "post-intgration ou migration se3",
        "wpkg" => "Forcer wpkg en mode graphique",
        "reboot" => "Allumer ou rebooter"
    );
    $html = "Choix du type d'actions ";
    $html .= "<SELECT NAME=\"type\" \">";
    $html .= "<option value=\"\">Rafraichir</option>";
    foreach ($types as $t => $txt) {
        $html .= "<option value=\"$t\"";
        if ($type == $t) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$txt</option>";
    }
    $html .= "</SELECT>";
    $html .= " <input type=\"submit\" value=\"" . gettext("Valider") . "\">\n";
    return $html;
}

// @TODO differencier le choix en fonction de l'étape en cours
function form_action_clonage($action = "")
{
    $actions = array(
        'submit' => "Valider/Actualiser",
        'start' => "Démarrer",
        'prepare' => "Préparer le modèle",
        'keep' => "Conserver le modèle pour un clonage ultérieur",
        'reset' => "Redémarrer le clonage (sans refaire le modèle)",
        'new' => "Nouveau clonage ou modèle ",
        'add' => "Ajouter des clones",
        'remove' => "Supprimer des clones",
        'disk' => "Choisir les partitions à cloner",
        'del' => "supprimer totalement"
    );
    $html = "<SELECT NAME=\"action\" \">";
    $html .= "<option value=\"\">Choisir l'action...</option>";
    foreach ($actions as $a => $txt) {
        $html .= "<option value=\"$a\"";
        if ($action == $a) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$txt</option>";
    }
    $html .= "</SELECT>";
    $html .= " <input type=\"submit\" value=\"" . gettext("Valider") . "\">\n";
    return $html;
}

function form_choix_install($action = "")
{
    $actions = array(
        'wimboot10' => "Installation auto Windows 10",
        'deb_mate' => "Installation auto Debian Mate",
        'deb_gnome' => "Installation auto Debian Gnome",
        'deb_cinnamon' => "Installation auto Debian Cinnamon",
        'deb_kde' => "Installation auto Debian KDE",
        'deb_base' => "Installation auto Debian terminal",
    );
    $html = "Choisir l'OS à installer\n";
    $html .= "<SELECT NAME=\"action\" onchange=\"submit()\">";
    $html .= "<option value=\"\">Installation...</option>";
    foreach ($actions as $a => $txt) {
        $html .= "<option value=\"$a\"";
        if ($action == $a) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$txt</option>";
    }
    $html .= "</SELECT>";
    return $html;
}

function form_choix_role($role = "")
{
    $html = "choisir le rôle de la machine pour le clonage\n";
    $html .= "<SELECT NAME=\"role\" onchange=\"submit()\">";
    $html .= "<option value=\"\">Role...</option>";
    $roles = array(
        "modele",
        "clone"
    );
    foreach ($roles as $r) {
        $html .= "<option value=\"$r\"";
        if ($role == $r) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$r</option>";
    }
    $html .= "</SELECT>";
    return $html;
}

function form_choix_reservation($action)
{
    $html = "Action dhcp :\n";
    $html .= "<SELECT NAME=\"action\" onchange=\"submit()\">";
    $html .= "<option value=\"\">choisir...</option>";
    $roles = array(
        "reservation",
        "dynamique"
    );
    foreach ($roles as $r) {
        $html .= "<option value=\"$r\"";
        if ($action == $r) {
            $html .= " selected>";
        } else {
            $html .= ">";
        }
        $html .= "$r</option>";
    }
    $html .= "</SELECT>";
    return $html;
}

function form_new_name($name)
{
    $html = "choisir le nouveau nom pour la machine $name\n";
    $html .= " <INPUT TYPE=\"text\" NAME=\"modele\" VALUE=\"" . $name . "\" SIZE=\"15\" onchange=submit()>";
    $html .= "<input type=\"hidden\" name=\"name\" value=\"" . $name . "\">\n";
    return $html;
}

function form_action_membres($config, $id)
{
    $action = fetch_action()[$id];
    $progress = $action['progress']['avancement'] ?? "";

    $html = "<TABLE border=1 width=\"80%\">\n";
    $html .= "<th colspan=4 class=menuheader style=\"height: 30\">Liste des actions</th>\n";

    $html .= "<tr><td class='menuheader' align=\"center\">Identifiant</td>";
    $html .= "<td class='menuheader' align=\"center\">Type</td>";
    $html .= "<td class='menuheader' align=\"center\">Avancement</td>";
    $html .= "<td class='menuheader' align=\"center\">Statut</td>";
    $html .= "</tr>\n";
    $statut = $action['statut'];
    $html .= "<tr><td>" . $id . "</td>";
    $html .= "<td>" . $action['type'] . "</td>";
    $html .= "<td><progress  max=\"100\" value=" . $progress . "> " . $progress . "% </progress></td>";
    $html .= "<td>" . $statut . "</td>";
    $html .= "</tr>\n";
    $html .= "</TABLE>";

    $html .= "<TABLE border=1 width=\"80%\">\n";
    $html .= "<th colspan=6 class=menuheader style=\"height: 30\">Liste des postes</th>\n";
    $html .= "<tr><td class='menuheader' align=\"center\">Role</td>";
    $html .= "<td class='menuheader' align=\"center\">Nom</td>";
    $html .= "<td class='menuheader' align=\"center\">Ip</td>";
    $html .= "<td class='menuheader' align=\"center\">Mac</td>";
    $html .= "<td class='menuheader' align=\"center\">statut</td>";
    $html .= "<td class='menuheader' align=\"center\">Selectionner</td>";
    $html .= "</tr>\n";

    foreach ($action as $token => $membre) {
        if (! empty($membre['role'])) {
            $machine = get_action($config, $token);
            $html .= "<tr><td>" . $membre['role'] . "</td>";
            $html .= "<td>" . $machine['cn'] . "</td>";
            $html .= "<td>" . $machine['iphostnumber'] . "</td>";
            $html .= "<td>" . $machine['networkaddress'] . "</td>";
            $html .= "<td>" . $membre['script'] . "</td>";
            if ($membre['role'] == "modele") {
                if ($action['keep'])
                    $html .= "<td align=\"center\">Conserver</td>";
                else
                    $html .= "<td></td>";
            } else {
                $html .= "<td align=\"center\"><input type=\"checkbox\" name=\"clones[]\" value=\"" . $token . "\"></td>";
            }
            $html .= "</tr>\n";
        }
    }
    $html .= "</TABLE>";
    $html .= "<br>";
    $html .= "<input type=\"hidden\" name=\"id\" value=\"" . $id . "\">\n";

    return $html;
}

function form_choix_poste($config, $action, $filtre, $choix, $role, $id)
{
    echo "Choix du poste $role";
    echo " <INPUT TYPE=\"text\" name=\"filtre\" VALUE=\"$filtre\" SIZE=\"14\">";
    echo "<input type=\"hidden\" name=\"action\" value=\"$action\">\n";
    echo "<input type=\"hidden\" name=\"role\" value=\"$role\">\n";
    echo "<input type=\"hidden\" name=\"id\" value=\"$id\">\n";
    echo "<input type=\"hidden\" name=\"choix\" value=\"1\">\n";

    echo "<br>";
    if (empty($filtre))
        $filtre = "*";
    else
        $filtre = "*" . $filtre . "*";
    if ($choix == 1) {
        $list_computer = search_ad($config, $filtre, "machine");
        if (count($list_computer) > 15)
            $size = 15;
        else
            $size = count($list_computer);

        if (count($list_computer) > 0) {
            sort($list_computer);
            echo gettext("Nom ou adresse IP de la machine : ");
            if ($role == "clone" || $role == "wpkg")
                echo "<p><select size=\"" . $size . "\" name=\"clones[]\" multiple>\n";
            elseif ($role == "install")
                echo "<p><select size=\"" . $size . "\" name=\"name\">\n";
            else
                echo "<p><select size=\"" . $size . "\" name=\"modele\">\n";
            foreach ($list_computer as $machine) {
                $ma = get_action($config, $machine['cn']);
                if (@$ma['action']['keep'] && $ma['action']['role'] == "modele") {
                    if ($role == "modele")
                        echo "<option value=\"" . $machine['cn'] . "\">" . $machine['cn'] . " (préparée)</option>\n";
                } else {
                    echo "<option value=\"" . $machine['cn'] . "\">" . $machine['cn'] . "</option>\n";
                }
            }
            echo "</select>\n";
            echo "<br>";
        }
    }
    echo " <input type=\"submit\" value=\"" . gettext("Valider") . "\">\n";
    echo "<u onmouseover=\"return escape" . gettext("('Donner le début du nom ou de l\'adresse IP de la machine que vous souhaitez trouver.')") . "\"><img name=\"action_image2\"  src=\"../elements/images/system-help.png\" alt=\"help\"></u> ";
}

function form_hidden($name, $val)
{
    return "<input type=\"hidden\" name=\"" . $name . "\" value=\"" . $val . "\">\n";
}

function form_choix_partitions($id)
{
    $action = fetch_action()[$id];

    $html .= "<SELECT NAME=\"partitions[]\" SIZE=\"5\" MULTIPLE=\"true\">";
    $html .= "<option value=\"\">Selectionner les disques et partitions...</option>";
    foreach ($action['disks'] as $disk) {
        $html .= "<option value=\"" . $disk['device'] . "\"";
        if ($disk['clone'])
            $html .= " selected";
        $html .= ">Copier le MBR de " . $disk['device'] . "</option>";
        foreach ($disk['partitions'] as $partition) {
            $html .= "<option value=\"" . $partition['device'] . "\"";
            if ($partition['clone'])
                $html .= " selected";
            $html .= "> Cloner " . $partition['device'] . " (" . $partition['fs'] . ", " . round($partition['size'] / (1024 * 1024), 1) . " G)</option>";
        }
    }
    $html .= "</SELECT>";
    $html .= "<input type=\"hidden\" name=\"id\" value=\"" . $id . "\">\n";
    return $html;
}

function form_action_id($type = "")
{
    $html = "<FORM action=\"lance_action.php\" method=\"post\">\n";
    "<SELECT NAME=\"id\" SIZE=\"20\">";
    if ($actions = fetch_action()) {
        foreach ($actions as $id => $action) {
            if (empty($type) || ($action['type'] == $type)) {
                $html .= "<option value=\"" . $id . "\"> Id : " . $id . " (" . $action['type'] . ")</option>";
            }
        }
    }
    $html .= "</SELECT>";
    $html .= "<input type=\"submit\" value=\"" . gettext("Valider") . "\">\n";
    $html .= "</FORM>\n";
    return $html;
}

function select_actions($actions, $module = "")
{
    $option = "";
    foreach ($actions as $action) {
        if (empty($module) || ($action['module'] == $module)) {
            $option .= "<option value=\"" . $action['nom'] . "\">" . $action['description'] . "</option>";
        }
    }
    return $option;
}

function fetch_action()
{
    if (! ($actions = apcu_fetch('actions'))) {
        $actions = array(
            'default' => array(
                'type' => 'default',
                'default' => array(
                    'role' => 'none',
                    'script' => "",
                    'ret' => - 1
                )
            )
        );
        apcu_add('actions', $actions);
    }
    return $actions;
}

/**
 * Récupère l'action programmée pour la machine, au boot ipxe, ou lors d'actions à distance
 * on récupère l'action et un token unique commun à une action et qui permet d'authentifier la requête
 * si l'enregistrement ad n'existe pas, on renvoie une action avec l'@mac comme id
 *
 * @param array $config
 * @param string $mac
 *            : adresse mac, ip, ou nom de la machine
 * @return array : tableau des attributes de la machine, avec les valeurs par défaut pour 'netbootinitialization' (action)
 *         et 'netbootmachinefilepath' (token)
 */
function get_action(array $config, string $mac, $token = "")
{
    $machine = search_machine($config, $mac);
    $actions = fetch_action();

    if (count($machine) > 0) {
        $machine['netbootmachinefilepath'] = $machine['netbootmachinefilepath'] ?? $token;
        $machine['netbootinitialization'] = $machine['netbootinitialization'] ?? "";
        if (isset($actions[$machine['netbootinitialization']][$machine['netbootmachinefilepath']])) {
            $machine['action'] = $actions[$machine['netbootinitialization']];
        } else {
            $machine['action'] = $actions['default'];
        }
    } else {
        if (array_key_exists($mac, $actions)) {
            $machine = array(
                'networkaddress' => $mac,
                'action' => $actions[$mac],
                'netbootmachinefilepath' => $token
            );
        }
    }
    return $machine;
}

/**
 * enregistre l'action pour une machine.
 * - si accès ipxe : token + mac doivent être fournis, eventuellement id, action, role
 * - si accès sysrescuecd : mac suffit, on est déjà enregistré en phase ipxe
 * - si accès interface : choix de tous les parametres.
 *
 * @param array $config
 * @param string $cn
 *            : nom ou ip ou mac de la machine
 * @param string $type
 *            : type d'action ipxe (voir la liste des ficihers dans ipxe/actions/ )
 * @param string $id
 *            : identifiant de l'action à effectuer
 * @param string $token
 *            jeton d'identification de la machine
 * @param string $role
 *            : role de la machine le cas échéant (modele|clone)
 * @param string $script
 *            : script à exécuter par sysrecuecd((autorun)/ipxe/windows/linux,
 * @param int $ret
 *            : etape pour les scripts en plusieurs étapes côté client
 *            
 * @return array $actions
 */
function set_action(array $config, string $cn, string $type = "", string $id = "", string $token = "", string $role = "", string $script = "", int $ret = - 255, array $disks = array())
{
    // on recupere les valeurs AD
    $machine = search_machine($config, $cn);
    // initialisation du token
    $mode_t = $mode_i = "replace";
    if (! isset($machine['netbootmachinefilepath']))
        $mode_t = "add";
    if (! isset($machine['netbootinitialization']))
        $mode_i = "add";

    if (empty($token)) {
        if (! empty($machine['netbootmachinefilepath'])) {
            $token = $machine['netbootmachinefilepath'];
        } else {
            $token = bin2hex(random_bytes(32));
        }
    }
    if ($type == "default") { // || $script == "default") {
        delete_action_members($token);
        $id = "";
    } else {

        // initialisation de l'id
        if (empty($id)) {
            if (! empty($machine['netbootinitialization'])) {
                $id = $machine['netbootinitialization'];
            } elseif (isset($machine['cn'])) {
                $id = $machine['cn'] . "@" . date_et_heure();
                // $machine['netbootinitialization'] = $id;
            } else {
                $id = $cn . "@" . date_et_heure();
            }
        } elseif ($id == "new") {
            $id = $cn . "@" . date_et_heure();
        }
    }
    $actions = fetch_action();

    // modifs ad si besoin
    if (count($machine) > 0) {
        $attrs = array();
        if (! empty($machine['netbootinitialization'])) {
            if ($id != $machine['netbootinitialization']) {
                // on enleve la machine de l'action
                unset($actions[$machine['netbootinitialization']][$machine['netbootmachinefilepath']]);
                $attrs['netbootinitialization'] = $id;
                modify_ad($config, $machine['cn'], "computer", $attrs, $mode_t);
            }
        } else {
            $attrs['netbootinitialization'] = $id;
            modify_ad($config, $machine['cn'], "computer", $attrs, $mode_t);
        }
        $attrs = array();
        if (! empty($machine['netbootmachinefilepath'])) {
            if ($machine['netbootmachinefilepath'] != $token) {

                $attrs['netbootmachinefilepath'] = $token;
                foreach ($actions as $key => $action) {
                    if (isset($action['netbootmachinefilepath'])) {
                        unset($actions[$key]['netbootmachinefilepath']);
                    }
                }
                modify_ad($config, $machine['cn'], "computer", $attrs, $mode_i);
            }
        } else {
            $attrs['netbootmachinefilepath'] = $token;
            modify_ad($config, $machine['cn'], "computer", $attrs, $mode_i);
        }
    }
    // mise a jour de actions
    if (! empty($id)) {
        $actions[$id] = $actions[$id] ?? array();
        if (! empty($type)) {
            $actions[$id]['type'] = $type;
        }
        $actions[$id][$token] = $actions[$id][$token] ?? array();
        if (count($disks) > 0) {
            $actions[$id]['disks'] = $disks;
        }
        if (! empty($role)) {
            $actions[$id][$token]['role'] = $role;
        }
        if (! empty($script)) {
            $actions[$id][$token]['script'] = $script;
        }
        if ($ret != - 255) {
            $actions[$id][$token]['ret'] = $ret;
        }
    }
    apcu_store('actions', $actions);

    return get_action($config, $cn);
}

function increment_action(array $config, string $cn)
{
    $machine = get_action($config, $cn);
    return set_action($config, $cn, "", "", "", "", "", $machine['action'][$machine['netbootmachinefilepath']]['ret'] + 1);
}

function delete_action_members(string $token)
{
    $actions = fetch_action();
    foreach ($actions as $key => $action) {
        if (isset($action[$token])) {
            unset($actions[$key][$token]);
        }
    }
    apcu_store('actions', $actions);
    return $actions;
}

function delete_action(string $id)
{
    $actions = fetch_action();
    if (isset($actions[$id])) {
        unset($actions[$id]);
    }
    apcu_store('actions', $actions);
    return $actions;
}

/**
 * verifie que l'action est authentifiée pour la machine
 *
 * @param array $config
 * @param string $cn
 *            :nom ou ip ou mac de la machine
 * @param string $token
 *            : jeton d'identification
 * @return boolean
 */
function auth_action(array $config, string $cn, string $token)
{
    $machine = get_action($config, $cn, $token);
    if (! empty($machine['netbootmachinefilepath']) && ($token != 'default'))
        return ($machine['netbootmachinefilepath'] == $token);
    else
        return false;
}

/**
 * retourne un jeton d'identification pour le boot iPXE en cas d'autentification avec un compte computer_is_admin
 *
 * @param array $config
 * @param string $login
 * @param string $password
 * @return string $token : jeton unique, "" si non autorisé
 */
function login_action(array $config, $cn, string $login, string $password)
{
    $token = "";
    if (user_valid_passwd($config, $login, $password) && have_right($config, "computer_is_admin", $login)) {
        $machine = get_action($config, $cn);
        if (! empty($machine['netbootmachinefilepath'])) {
            $token = $machine['netbootmachinefilepath'];
        } else {
            $token = bin2hex(random_bytes(32));
            if (isset($machine['cn'])) {
                $attrs = array(
                    'netbootmachinefilepath' => $token
                );
                modify_ad($config, $machine['cn'], "computer", $attrs);
            } else {
                set_action($config, $cn, "enregistrement", $cn, $token);
            }
        }
    }
    return $token;
}
?>