<?php

/**
 * manipulation des parametres de conf
 * @Projet LCS / SambaEdu

 * @Auteurs Equipe SambaEdu

 * @Note Ce fichier de fonction doit être appelé par un include dans entete.inc.php
 * @Note Ce fichier est complete a l'installation

 * @Licence Distribué sous la licence GPL
 */

/**
 *
 * file: config.php
 *
 * @Repertoire: includes/
 */

// Liste des caracteres speciaux pour les mots de passe
$char_spec = "&_#@£%§:!?*$-";

/**
 * Auth sur l'interface
 *
 * @paramètres Teste si une authentification est faite via une session php
 * @returns Si non, renvoie "", Si oui, renvoie l'uid de la personne
 */
function isauth()
{
    $login = "";
    @session_name("Sambaedu");
    @session_start([
        'cookie_lifetime' => 86400
    ]);

    /*
     * if ($_SESSION['auth'] = "ent") {
     * $config = get_config(true);
     * require_once (dirname(__FILE__) . '/../vendor/autoload.php');
     *
     * $provider = new \League\OAuth2\Client\Provider\GenericProvider([
     * 'clientId' => $config['ent_oauth2_id'], // The client ID assigned to you by the provider
     * 'clientSecret' => $config['ent_oauth2_pass'], // The client password assigned to you by the provider
     * 'redirectUri' => $config['ent_oauth2_redirect_uri'],
     * 'urlAuthorize' => $config['ent_uri'] . $config['ent_oauth2_auth'],
     * 'urlAccessToken' => $config['ent_uri'] . $config['ent_oauth2_token'],
     * 'urlResourceOwnerDetails' => $config['ent_uri'] . $config['ent_oauth2_userinfo'],
     * 'scopes' => [
     * 'userinfo'
     * ]
     * ]);
     * $existingAccessToken = $_SESSION['accesstoken'];
     *
     * if ($existingAccessToken->hasExpired()) {
     * $newAccessToken = $provider->getAccessToken('refresh_token', [
     * 'refresh_token' => $existingAccessToken->getRefreshToken()
     * ]);
     *
     * $_SESSION['accesstoken'] = $newAccessToken;
     * }
     * }
     */
    $login = $_SESSION['login'] ?? "";
    return $login;
}

function db_connect($config)
{
    return mysqli_connect('localhost', 'sambaedu', $config['sql_passwd'], 'sambaedu');
}

/**
 * met en place un verrou pour l'accès à la conf afin d'éviter les accès concurrents
 *
 * @param
 *            $mode
 * @return resource|boolean
 */
function lock_conf($mode)
{
    $lock = "/var/lock/sambaedu.lock";
    if ($fp = fopen($lock, "c")) {
        $startTime = microtime();
        do {
            $canWrite = flock($fp, $mode);
            // If lock not obtained sleep for 0 - 100 milliseconds, to avoid collision and CPU load
            if (! $canWrite)
                usleep(round(rand(0, 100) * 1000));
        } while ((! $canWrite) and ((microtime() - $startTime) < 1000));

        // file was locked so now we can store information
        if ($canWrite) {
            return $fp;
        }
    }
    return false;
}

/**
 * Lire la conf de se4 et de ses modules
 * fonction pour récupérer la conf de se4 ou des modules de façon recursive dans /etc/sambaedu/
 * possibilité de parser l'ancienne conf afin de pouvoir comparer les valeurs
 *
 * @Paramètres : "nom du module", "sambaedu" ou "all"
 * @returns array["parametre"]
 */
function get_config_se4($module = "sambaedu", $old = false, $table = false)
{
    if ($old) {
        $ext = "-old";
    } else {
        $ext = "";
    }
    $config = array();
    if ($module == "all") {
        $config = get_config_se4("sambaedu", $old);
        if ($handle = opendir('/etc/sambaedu/sambaedu.conf.d')) {
            unset($module);
            while (false !== ($entry = readdir($handle))) {
                if ($entry != "." && $entry != "..") {
                    $module = preg_replace("/\.conf" . $ext . "/", "", $entry);
                    if (isset($module)) {
                        $config = array_merge($config, get_config_se4($module, $old, $table));
                    }
                }
            }
            closedir($handle);
        }
        return ($config);
    } elseif ($module == "sambaedu") {
        $conf_file = "/etc/sambaedu/sambaedu.conf" . $ext;
        $config = parse_ini_file($conf_file);
        if ($table) {
            foreach ($config as $p => $v) {
                $t[$p] = [
                    'module' => $module,
                    'valeur' => $v
                ];
            }
            $config = $t;
        } else {
            $config['dn']['admin'] = "cn=" . $config['ldap_admin_name'] . "," . $config['admin_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['people'] = $config['people_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['groups'] = $config['groups_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['rights'] = $config['rights_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['equipements'] = $config['equipements_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['delegations'] = $config['delegations_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['trash'] = $config['trash_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['parcs'] = $config['parcs_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['computers'] = $config['computers_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['autres'] = $config['other_groups_rdn'] . "," . $config['groups_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['projets'] = $config['projets_rdn'] . "," . $config['groups_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['cours'] = $config['cours_rdn'] . "," . $config['groups_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['classes'] = $config['classes_rdn'] . "," . $config['groups_rdn'] . "," . $config['ldap_base_dn'];
            $config['dn']['equipes'] = $config['equipes_rdn'] . "," . $config['groups_rdn'] . "," . $config['ldap_base_dn'];
        }
    } else {
        $conf_file = "/etc/sambaedu/sambaedu.conf.d/$module.conf" . $ext;
        if (file_exists($conf_file)) {
            if ($lock = lock_conf(LOCK_SH)) {
                $config = parse_ini_file($conf_file);
                if ($table) {
                    foreach ($config as $p => $v) {
                        $t[$p] = [
                            'module' => $module,
                            'valeur' => $v
                        ];
                    }
                    $config = $t;
                }
                fclose($lock);
            }
        }
    }
    $config['login'] = isauth();
    return ($config);
}

function compare_config($module = "all")
{
    $conf = get_config_se4($module);
    $old_conf = get_config_se4($module, true);
    $conf_diff = array_diff_assoc($conf, $old_conf);
    $conf_reverse_diff = array_diff_assoc($old_conf, $conf);
    $diff = array_merge($conf_reverse_diff, $conf_diff);
    return $diff;
}

/*
 * fonction pour récupérer la conf dans un tableau persistant 120s
 * @Parametres : [$force = true] pour forçer la lecture
 * @returns : $config = tableau des parametres
 */
function get_config($force = false)
{
    // while (apcu_fetch('config_lock')) {
    // sleep(1);
    // }
    $success = false;
    $config = apcu_fetch('config', $success);
    if (($force) || ! $success) {
        apcu_add('config_lock', 1, 60);

        unset($config);
        $config = get_config_se4('all');

        apcu_store('config', $config, 120);
        apcu_delete('config_lock');
        if (! $config) {
            die("Erreur de lecture de la configuration se4");
        }
    }
    return ($config);
}

/*
 * fonction pour executer une action système en cas de changement d'un parametre de conf
 * exécute un sudo script shell
 * @Parametres : $param
 *
 * @Parametres : module ( defaut = "base" )
 * @returns : $ret
 */
function set_config_action($param)
{
    exec("sudo -c '/usr/share/sambaedu/scripts/config_action.sh $param'", $ret);
    return ($ret);
}

/**
 * Ecriture d'un paramètre de conf
 *
 * @description  fonction pour écrire la conf dans les fichiers de conf  :retourne le tableau $config et met à jour le cache
 * @Parametres : $param : parametre a fixer, $value : valeur à positionner, $module = "sambaedu" : module SE4 concerné
 * @returns : $config ou false
 */
function set_config($param, $value = "", $module = "sambaedu")
{
    /*
     * while (apcu_fetch('config_lock')) {
     * sleep(1);
     * }
     */
    unset($config);
    $config = get_config_se4($module);
    if (empty($value)) {
        if (isset($config[$param]))
            unset($config[$param]);
    } else {
        $config[$param] = $value;
    }
    $content = "";
    foreach ($config as $key => $value) {
        if (! (("$key" == "dn") || ("$key" == "login") || empty($key))) {
            $content .= $key . " = \"" . $value . "\"\n";
        }
    }
    // write it into file
    if ($module == "sambaedu") {
        $conf_file = "/etc/sambaedu/sambaedu.conf";
        // $conf_file_tmp = "/etc/sambaedu/sambaedu.conf.tmp";
    } else {
        $conf_file = "/etc/sambaedu/sambaedu.conf.d/$module.conf";
        // $conf_file_tmp = "/etc/sambaedu/sambaedu.conf.d/$module.conf.tmp";
    }
    // apcu_add('config_lock', 1, 300);
    // on teste l'ecriture dans le fichier temporaire
    /*
     * if (! $handle1 = fopen($conf_file_tmp, "w")) {
     * apcu_delete('config_lock');
     * die("Erreur d'ecriture de la configuration se4 : $module $param $value");
     * }
     * $res_test = fwrite($handle1, $content);
     * fclose($handle1);
     * if (! $res_test) {
     * apcu_delete('config_lock');
     * die("Erreur d'ecriture de la configuration se4 : $module $param $value");
     * } else {
     * // test ecriture reussi
     * if (! $handle = fopen($conf_file, "w")) {
     * apcu_delete('config_lock');
     * die("Erreur d'ecriture de la configuration se4 : $module $param $value");
     * }
     * $res = fwrite($handle, $content);
     * fclose($handle);
     * @unlink($conf_file_tmp);
     * apcu_delete('config_lock');
     * if (! $res)
     * die("Erreur d'ecriture de la configuration se4 : $module $param $value");
     * }
     */
    if ($lock = lock_conf(LOCK_EX)) {
        if (! $handle = fopen($conf_file, "w")) {
            fclose($lock);
            die("Erreur d'ecriture de la configuration se4 : $module $param $value");
        }
        $res = fwrite($handle, $content);
        fclose($handle);
        fclose($lock);
    } else {
        die("Erreur d'ecriture de la configuration se4 : $module $param $value");
    }
    return (get_config(true));
}

/**
 * Initilialisation d'un paramètre de conf
 *
 * @description initilise la valeur d'un parametre s'il n'existe pas dans la conf
 * @Parametres : parametre a initaliser, valeur, module ( defaut = "sambaedu" )
 * @returns : array() $config
 */
function init_param(&$config, $nom, $valeur, $module = "sambaedu")
{
    if (! isset($config[$nom])) {
        set_config($nom, $valeur, $module);
    }
}

function set_param(&$config, $nom, $valeur, $module = "sambaedu")
{
    set_config($nom, $valeur, $module);
    return $valeur;
}

/**
 * Lecture d'un paramètre de conf
 *
 * @description Lit un paramètre dans la conf
 * @parametres $config, $nom
 * @returns ""|string
 */
function get_param($config, $nom)
{
    if (isset($config[$nom])) {
        return $config[$nom];
    } else {
        return "";
    }
}

/**
 * remplace de façon récursive dans tous les fichiers de conf la valeur du parametre $param par $valeur (recherche par regexp)
 *
 * @param unknown $config
 * @param unknown $param
 * @param unknown $valeur
 */
function replace_param($param, $valeur)
{
    $config_table = get_config_se4("all", false, true);
    $old_v = $config_table[$param]['valeur'];
    foreach ($config_table as $p=>$table){
        if ($preg_match("|".$old_v. "|", $table['valeur']) == 1){
            $config = set_config($p, preg_replace("|".$old_v. "|", $valeur, $table['valeur']), $table['module']);
        }
    }
    return $config;
}

/**
 *
 * @description remplace ###_NOM_### par $param['nom'] dans un fichier en mémoire. Peut s'utiliser directement avec $config
 * @parametres array $params, array $lignes tableau du fichier à remplacer
 * @returns mixed
 */
function write_param(array $params, array &$lignes)
{
    foreach ($params as $param => $valeur) {
        if (! is_array($valeur))
            $lignes = preg_replace("/###_" . strtoupper($param) . "_###/", $valeur, $lignes);
    }
    // return $lignes;
}

// fonctions pour exécution asynchrone de scripts
/**
 *
 * @description Mise en fil d'attente de commandes programmée pour exécution asynchrone
 * @parametres $command commande à lancer
 * @returns
 */
function batch_command($command)
{
    $batch = apcu_fetch("batch_" . session_id());
    $batch .= $command . "\n";
    apcu_store("batch_" . session_id(), $batch, 500);
}
/**
 *
 * @description Lancement programmé des commandes mises en fil d'attente via batch_command
 * @parametres $queu fast / normal / slow pour un lancement dans la mn / 10mn / à 21h00
 * @returns
 */
function batch_write($queue = "normal")
{
    $batch = apcu_fetch("batch_" . session_id()) ?? "";
    if (file_exists("/tmp/admin_script_$queue.sh")) {
        $script = file_get_contents("/tmp/admin_script_$queue.sh");
    } else {
        $script = "#!/bin/bash\n";
        $script .= "rm -f /tmp/admin_script_$queue.sh\n";
    }
    $script .= $batch;
    file_put_contents("/tmp/admin_script_$queue.sh", $script);
    system("sudo chmod 755 /tmp/admin_script_$queue.sh");
    apcu_delete("batch_" . session_id());
}




// configuration du proxy

/**
 * ajout des options du proxy pour curl
 *
 * @param unknown $config,
 *            $opt options curl
 */
function curl_proxy_options($config, &$opt = [])
{
    if (! empty($config['server_proxy']) && ! empty($config['proxy_address']) && ! empty($config['proxy_port'])) {
        $opt['curl'] = [
            CURLOPT_PROXY => "http://" . $config['proxy_address'] . ":" . $config['proxy_port'],
            // CURLOPT_HTTPPROXYTUNNEL => true,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_SSL_VERIFYHOST => false
        ];
    } else {
        $opt['curl'] = [
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_SSL_VERIFYHOST => false
        ];
    }
}

function set_php_proxy($config)
{
    if (! empty($config['server_proxy']) && ! empty($config['proxy_address']) && ! empty($config['proxy_port'])) {
        $context = array(
            'http' => array(
                'proxy' => "tcp://" . $config['proxy_address'] . ":" . $config['proxy_port'],
                'request_fulluri' => true,
                'verify_peer' => false,
                'verify_peer_name' => false
            ),
            "ssl" => array(
                "verify_peer" => false,
                "verify_peer_name" => false
            )
        );
        return stream_context_set_default($context);
    }
}

function set_oauth_proxy($config, &$opt)
{
    if (! empty($config['server_proxy']) && ! empty($config['proxy_address']) && ! empty($config['proxy_port'])) {
        $opt['proxy'] = "http://" . $config['proxy_address'] . ":" . $config['proxy_port'];
        $opt['verify'] = false;
    }
}

setlocale(LC_ALL, "C");
bindtextdomain("messages", "./locale");
textdomain("messages");

// Paramètres LDAP
$config = get_config(true);
set_php_proxy($config);
?>
