<?php

/**
 * Fonctions utiles
 * @Version 02-2020
 * @Projet SambaEdu
 * Fonctions Interface Homme/Machine
 * @Auteurs Denis Bonnenfant @ Equipe Sambaedu
 * @Note: Ce fichier de fonction doit etre appele par un include
 * @Licence Distribue sous la licence GPL
 */

/**
 *
 * file: fonc_outils.inc.php
 *
 * @Repertoire: includes/
 */
require_once "logs.inc.php";

function convertWindowsTimestamp($wintime)
{
    return $wintime / 10000000 - 11644477200;
}

/**
 *
 * Fonctions qui retourne vrai si l'inventaire (ocs) est actif
 *
 * @Parametres
 * @return 0
 *
 */
function inventaire_actif()
{
    require_once "config.inc.php";
    return $inventaire;
}

/**
 *
 * Fonctions qui Ping une machine Return 1 si Ok 0 pas de ping
 *
 * @Parametres Adresse IP de la machine a pinguer
 *
 * @return 1 si le ping repond - 0 si pas de reponse
 *        
 */
/*
 * function fping($machine['iphostnumber']) { // Ping une machine Return 1 si Ok 0 pas de ping
 * return exec("ping ".$machine['iphostnumber']." -c 1 -w 1 | grep received | awk '{print $4}'");
 * }
 */

/**
 *
 * Fonctions qui retourne l'adresse IP d'une machine en fonction de son nom ou 0 si pas d'IP
 *
 * @Parametres  Nom de la machine
 *
 * @return L'adresse IP
 *        
 */
function avoir_ip($mpenc)
{ // Retourne l'adresse IP d'une machine en fonction de son nom ou 0 si pas d'IP
    $mp_curr = search_ad("(&(cn=$mpenc)(objectClass=ipHost))", "computers");
    if (isset($mp_curr[0]["ipHostNumber"])) {
        $iphost = $mp_curr[0]["ipHostNumber"];
        return $iphost;
    } else {
        return 0;
    }
}

/**
 *
 * Fonctions qui retourne le nom d'une machine en fonction de son adresse IP
 *
 * @Parametres Adresse IP  de la machine
 *
 * @return Le nom de la machine
 *        
 */
function avoir_nom($ipHost)
{ // Retourne le nom d'une machine a partir de l'adresse IP ou 0 si pas
    $mp_curr = search_ad("(&(ipHostNumber=$ipHost)(objectClass=ipHost))", "computers");
    if (isset($mp_curr[0]["cn"])) {
        $mpenc = $mp_curr[0]['cn'];
        return $mpenc;
    } else {
        return 0;
    }
}

/**
 *
 * Fonctions qui retourne l'adresse mac d'une machine en fonction de son nom
 *
 * @Parametres nom de la machnie
 *
 * @return adresse mac
 *        
 */
function avoir_mac($mpenc)
{
    require_once ("ldap.inc.php");

    $mp_curr = search_ad("(&(cn=$mpenc)(objectClass=ipHost))", "computers");
    // echo "mac:".$mp_curr[0]['macAddress']."<br>";
    if (isset($mp_curr[0]['macAddress'])) {
        $ret = $mp_curr[0]['macAddress'];
        return $ret;
    } else {
        return 0;
    }
}

/**
 * Retourne la config de l'interface serveur
 *
 * @Parametres
 *
 * @return tableau des caractéristiques reseau
 *        
 */
function ifconfig()
{
    require_once "config.php";
    require_once ("ihm.inc.php");
    $reseau['mask'] = exec("/sbin/ifconfig | grep " . $se3ip . " | awk '{print $4}' | sed 's/Masque://;s/Mask://'");
    $reseau['broadcast'] = exec("/sbin/ifconfig | grep " . $se3ip . " | awk '{print $3}' | sed 's/Bcast://'");
    $reseau['network'] = long2ip(ip2long($reseau['broadcast']) & ip2long($reseau['mask']));
    $reseau['interface'] = exec("route -n | grep " . $reseau['network'] . " | awk '{print $8}'");
    $reseau['gateway'] = exec("route -n | grep UG | awk '{print $2}'");
    return $reseau;
}

/**
 * Demarre, eteint ou reboote un poste
 *
 * @Parametres action: wol, reboot, shutdown
 *             nom : nom du poste
 *
 * @return
 *
 */
function start_poste($config, $action, $name, $force = false, $silent = false)
{
    $machine = search_machine($config, $name, true);
    $reseau = get_vlan($config, $machine['iphostnumber']);
    $ret = false;
    $out = array();
    $msg = "";
    $status = get_machine_status($config, $name);
    switch ($action) {
        case "start":
        case "wol":
            if ($status['state'] == "off") {
                $msg .= "Mise en marche de la machine  <b>$name</b> : ";
                exec("/usr/bin/wakeonlan -i " . long2ip($reseau['broadcast']) . " " . $machine['networkaddress'], $out);
                $msg .= end($out) . "<br>";
                if (isset($config['wol_broadcast'])) {
                    exec("/usr/bin/wakeonlan -i " . $config['wol_broadcast'] . " " . $machine['networkaddress'], $out);
                    $msg .= end($out) . "<br>";
                }
                log_connexion($config, "", $name, "wol", "startup");
            }
            break;

        case "reboot":
            if ($status['state'] != "off") {
                $msg .= "On reboote avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> :<br>\n";
                if ($status['state'] == "windows") {
                    // machine windows
                    if ($silent) {
                        $commande = "/usr/bin/net rpc shutdown -k -t 2 -f -r -C 'Reboot demande par le serveur sambaEdu' -U www-admin% -S " . $name . " >/dev/null 2>&1 &";
                    } else {
                        $commande = "/usr/bin/net rpc shutdown -k -t 2 -f -r -C 'Reboot demande par le serveur sambaEdu' -U www-admin% -S " . $name . " 2>&1";
                    }
                    exec($commande, $out, $ret);
                    if ($ret != 0) {
                        $msg .= "<b>Attention, arrêt impossible</b> : " . end($out) . "<br><br>";
                        $msg .= start_poste($config, "wol", $name);
                    } else {
                        $msg .= end($out);
                    }
                    $msg .= "<br><br>";
                } else {
                    // poste linux : ssh...
                    exec("/usr/bin/ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o ConnectTimeout=1 root@" . $machine['cn'] . " shutdown -r now", $out, $ret);
                    $msg .= end($out) . "<br><br>";
                    if ($ret != 0) {
                        $msg .= "On reboote avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> :<br>\n";
                        $msg .= "<b>Attention, reboot impossible</b>, on essaye de la démarrer <br><br>";
                        $msg .= start_poste($config, "wol", $name);
                    } else {
                        $msg .= "OK<br>\n";
                    }
                }
            }
            break;

        case "stop":
        case "shutdown":
            if ($status['state'] != "off" && ($force || @count($status['user']) == 0)) {
                $msg .= "On éteint avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> : <br>\n";
                if ($status['state'] == "windows") {
                    // machine windows
                    if ($silent) {
                        $commande = "/usr/bin/net rpc shutdown -k -t 30 -f -C \"Arret demande par le serveur sambaEdu\" -U www-admin% -S " . $name . " >/dev/null 2>&1 &";
                    } else {
                        $commande = "/usr/bin/net rpc shutdown -k -t 30 -f -C \"Arret demande par le serveur sambaEdu\" -U www-admin% -S " . $name . " 2>&1";
                    }
                    exec($commande, $out, $ret);
                    if ($ret != 0) {
                        $msg .= "<b>Attention, arrêt impossible</b> : " . end($out) . "<br><br>";
                    } else {
                        $msg .= end($out);
                    }
                    $msg .= "<br><br>";
                } else {
                    // poste linux : ssh...
                    exec("/usr/bin/ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o ConnectTimeout=1 root@" . $machine['cn'] . " shutdown -h now", $out, $ret);
                    $msg .= end($out) . "<br><br>";
                    if ($ret != 0) {
                        $msg .= "On &#233;teint avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> : <br>\n";
                        $msg .= "<b>Attention, arr&#234;t impossible</b>, la machine est injoignable ! <br><br>";
                    } else {
                        $msg .= "OK<br>\n";
                    }
                }
            }
            break;
    }
    return $msg;
}

/**
 * Demarre, eteint ou reboote un parc
 *
 * @Parametres action: wol, reboot, shutdown
 *             parc : nom du parc
 *
 * @return
 *
 */
function start_parc($config, $action, $parc, $force = false)
{
    $liste = list_members_parc($config, $parc, false);
    sort($liste);
    $ret = "";
    foreach ($liste as $value) {
        $ret .= start_poste($config, $action, $value, $force, true) . "<br>\n";
    }
    return $ret;
}

/**
 *
 * Fonction qui retourne si une machine a une demande de maintenance et le type
 *
 * @Parametres  Nom de la machine
 *
 * @return Le type de la demande de maintenance de la machine
 *        
 */
function testMaintenance($mpenc)
{ // Retourne si une machine a une demande de maintenance et le type
    $dbnameinvent = "ocsweb";
    include ("dbconfig.inc.php");
    $authlink_invent = @mysql_connect($_SESSION["SERVEUR_SQL"], $_SESSION["COMPTE_BASE"], $_SESSION["PSWD_BASE"]);
    @mysql_select_db($dbnameinvent) or die("Impossible de se connecter &#224; la base $dbnameinvent.");
    $query = "select * from repairs where (STATUT='2' or STATUT='0') and NAME='$mpenc'";
    $result = mysql_query($query, $authlink_invent);
    $ligne = mysql_num_rows($result);
    if ($ligne > 0) {
        while ($row = mysql_fetch_array($result)) {
            return $row["PRIORITE"];
        }
    }
}

/**
 *
 * Fonctions qui retourne la date du dernier inventaire
 *
 * @Parametres Nom de la machine
 *
 * @return Date du dernier inventaire
 *        
 */
function der_inventaire($nom_machine)
{ // retourne la date du dernier inventaire a partir de hardware
    include "dbconfig.inc.php";
    $dbnameinvent = "ocsweb";

    $authlink_invent = @mysql_connect($_SESSION["SERVEUR_SQL"], $_SESSION["COMPTE_BASE"], $_SESSION["PSWD_BASE"]);
    @mysql_select_db($dbnameinvent) or die("Impossible de se connecter &#224; la base $dbnameinvent.");

    $query = "select OSNAME,WORKGROUP,PROCESSORS,MEMORY,IPADDR,LASTDATE from hardware where NAME='$nom_machine'";
    $result = mysql_query($query, $authlink_invent);
    if ($result) {
        $ligne = mysql_num_rows($result);
        if ($ligne > 0) {
            while ($res = mysql_fetch_array($result)) {
                $retour = $res["OSNAME"] . " WG : " . $res["WORKGROUP"] . " P : " . $res["PROCESSORS"] . " Mem : " . $res["MEMORY"] . " DI : ";
                if ($res["LASTDATE"]) {
                    $retour .= date('d M Y', strtotime($res["LOGDATE"]));
                }
            }
        } else {
            $retour = 0;
        }

        return $retour;
    } else { // Pas d'inventaire a ce nom
        return 0;
    }
}

/**
 *
 * Fonctions qui retourne l'ID de $nom_machine ou 0 a partir de la table hardware
 *
 * @Parametres Nom de la machine
 *
 * @return ID de la machine pour ocs
 *        
 */

/*
 * function avoir_systemid($nom_machine) { // retourne l'ID de $nom_machine ou 0 a partir de la table hardware
 *
 * include "dbconfig.inc.php";
 * $dbnameinvent="ocsweb";
 *
 * $authlink_invent=@mysql_connect($_SESSION["SERVEUR_SQL"],$_SESSION["COMPTE_BASE"],$_SESSION["PSWD_BASE"]);
 * @mysql_select_db($dbnameinvent) or die("Impossible de se connecter &#224; la base $dbnameinvent.");
 *
 * $query="select ID from hardware where NAME='$nom_machine'";
 * $result = mysql_query($query,$authlink_invent);
 * if ($result) {
 * $ligne=mysql_num_rows($result);
 * if ($ligne > 0) {
 * while ($res = mysql_fetch_array($result)) {
 * $retour=$res["ID"];
 * }
 * } else {
 * $retour=0;
 * }
 *
 * return $retour;
 * } else { // Pas d'inventaire a ce nom
 * return 0;
 * }
 * }
 */

/**
 *
 * Fonctions qui retourne l'os de la machine
 *
 * @Parametres Nom de la machine
 *
 * @return Le type de la machine
 *        
 */
function type_os($nom_machine)
{ // retourne l'os de la machine
    include "dbconfig.inc.php";
    $dbnameinvent = "ocsweb";

    $authlink_invent = @mysql_connect($_SESSION["SERVEUR_SQL"], $_SESSION["COMPTE_BASE"], $_SESSION["PSWD_BASE"]);
    @mysql_select_db($dbnameinvent) or die("Impossible de se connecter &#224; la base $dbnameinvent.");

    $query = "select OSNAME from hardware where NAME='$nom_machine' order by LASTDATE DESC limit 0,1";
    $result = mysql_query($query, $authlink_invent);
    if ($result) {
        $ligne = mysql_num_rows($result);
        if ($ligne > 0) {
            while ($res = mysql_fetch_array($result)) {
                $retour = $res["OSNAME"];
                if (preg_match('/XP/i', $retour)) { // 6 types d'icones 98 / XP / 7 / Linux / Vista / 10
                    $retour = "XP";
                    return $retour;
                } elseif (preg_match('/2000/i', $retour)) {
                    $retour = "XP";
                    return $retour;
                } elseif (preg_match('/2003/i', $retour)) {
                    $retour = "XP";
                    return $retour;
                } elseif (preg_match('/Linux/i', $retour)) {
                    $retour = "Linux";
                    return $retour;
                } elseif (preg_match('/7/i', $retour)) {
                    $retour = "7";
                    return $retour;
                } elseif (preg_match('/vista/i', $retour)) {
                    $retour = "vista";
                    return $retour;
                } elseif (preg_match('/10/i', $retour)) {
                    $retour = "10";
                    return $retour;
                } else
                    return 0;
            }
        } else {
            return 0;
        }
    } else { // Pas d'inventaire a ce nom
        return 0;
    }
}

/**
 *
 * Fonctions qui supprime une machine d'un parc
 *
 * @Parametres  Nom du parc - Nom de la machine
 *
 * @Return:
 *
 */
function move_computer_parc($parc, $computer)
{ // Supprime une machine d'un parc
  // Suppression des machines dans le parc
    require_once "config.php";
    $cDn = "cn=" . $computer . "," . $computers_rdn . "," . $ldap_base_dn;
    $pDn = "cn=" . $parc . "," . $parcs_rdn . "," . $ldap_base_dn;
    exec("/usr/share/se3/sbin/groupDelEntry.pl \"$cDn\" \"$pDn\"");
    exec("/usr/share/se3/sbin/printers_group.pl");
}

/**
 *
 * Fonctions qui teste si cups tourne
 *
 * @Parametres
 *
 * @return 1 si cups tourne - 0 si tourne pas
 */
function test_cups()
{ // test si cups tourne
    $status_cups = exec("/usr/bin/lpstat -r");
    if ($status_cups == "scheduler is running") {
        return 1;
        // $icone_cups="enabled.png";
    } else {
        return 0;
        // $icone_cups="disabled.png";
    }
}

/**
 *
 * Fonctions qui demarre cups
 *
 * @Parametres
 *
 * @return
 */
function start_cups()
{ // demarre ou stop cups
    if (test_cups() == 0) {
        exec("sudo /etc/init.d/cupsys start");
    } else {
        exec("sudo /etc/init.d/cupsys stop");
    }
}

/**
 *
 * Fonctions qui supprime une imprimante d'un parc (sans la supprimer de l'annuaire)
 *
 * @Parametres  parc : Nom du parc - printer : Nom de l'imprimante
 *
 * @return
 *
 */
function move_printer_parc($parc, $printer)
{ // Sort une imprimante $printer du parc $parc
    if ($parc != "" && $printer != "") {
        exec("/usr/share/se3/sbin/printerDelPark.pl $printer $parc", $AllOutPutValue, $ReturnValue);
    }
}

/**
 *
 * Fonctions qui supprime une imprimante definitivement
 *
 * @Parametres	printer : Nom de l'imprimante
 *
 * @return
 *
 */
function move_printer($printer)
{ // Supprime une imprimante definitivement
    exec("/usr/share/se3/sbin/printerDel.pl $printer", $AllOutPutValue, $ReturnValue);
}

/**
 *
 * Fonctions qui stop ou start une imprimante
 *
 * @Parametres  printer : Nom de l'imprimante - status : etat de l'imprimante
 *
 * @return
 *
 */
function stop_start_printer($printer, $status)
{ // Stop ou start une imprimante
    if (isset($printer)) {
        exec("/usr/bin/$status $printer");
    }
}

/**
 * Affiche la date dans le format j/m/a
 *
 * @Parametres la date
 *
 * @return la date au format j/m/a
 */
function affiche_date($date)
{
    list ($a, $m, $j) = preg_split("/-/", $date);
    return "$j/$m/$a";
}

/**
 * Retourne une liste sous forme d'un tableau
 *
 * @Parametres $liste
 *
 * @return la tableau
 */
function liste_tab($liste)
{
    $t = preg_split("/\|/", $liste);
    for ($i = 0; $i < count($t); $i = $i + 2) {
        $cle = $t[$i];
        $val = $t[$i + 1];
        $tab[$cle] = $val;
    }
    return $tab;
}

/**
 * Retourne la classe de l'eleve a partir de son uid
 *
 * @Parametres $login
 *
 * @return la classe
 */
function classe_eleve($login)
{
    list ($user, $groups) = search_user($login, true);
    $nb_groupes = count($groups);
    for ($g = 0; $g < $nb_groupes; $g ++) {
        if (preg_match("/^Classe/", $groups[$g]["cn"])) {
            $classe = $groups[$g]["cn"];
            return $classe;
            break;
        }
    }
    if (isset($classe)) {
        return $classe;
    }
}

/**
 * a partir du tab d'uid fournit un tableau associatif uid-eleve => classe
 *
 * @Parametres $tab tableau
 *
 * @return tabeau uid eleve => classe
 */
function classe_eleves($tab)
{
    $nb = sizeof($tab);
    $tab_eleves_classe = array();
    for ($p = 0; $p < $nb; $p ++) {
        $uid = $tab[$p];
        $tab_eleves_classe[$uid] = classe_eleve($uid);
    }
    return $tab_eleves_classe;
}

/**
 *
 * fournit la classe, le fullname et le sexe d'un eleve a partir de son uid (tableau)
 *
 * @Parametres $login de l'eleve
 *
 * @return Retourne la classe, le fullname et le sexe
 *        
 */
function params_eleve($login)
{
    list ($user, $groups) = search_user($login, true);
    $nb_groupes = count($groups);
    // oblige de faire une boucle parmi tous les groupes !!
    for ($g = 0; $g < $nb_groupes; $g ++) {
        if (preg_match("/^Classe/", $groups[$g]["cn"])) {
            $classe = $groups[$g]["cn"];
            break;
        }
    }
    return array(
        'classe' => $classe,
        'sexe' => $user["sexe"],
        'nom' => $user["fullname"]
    );
}

/**
 * Retourne un select pour choisir la date
 *
 * @Parametres date et param
 * @return un select au sens HTML
 *        
 */
function choix_date($date, $param)
{
    $tab_mois = array(
        9 => gettext("Septembre"),
        10 => gettext("Octobre"),
        11 => gettext("Novembre"),
        12 => gettext("D&#233;cembre"),
        1 => gettext("Janvier"),
        2 => gettext("F&#233;vrier"),
        3 => gettext("Mars"),
        4 => gettext("Avril"),
        5 => gettext("Mai"),
        6 => gettext("Juin"),
        7 => gettext("Juillet"),
        8 => gettext("Ao&#251;t")
    );
    list ($an, $mois, $jour) = preg_split("/-/", $date);
    echo "<select name=\"jour_$param\">";
    // <option value=\"jour\"> jour</option>";
    for ($i = 1; $i <= 31; $i ++) {
        $ii = ($i < 10 ? "0$i" : $i);
        echo "<option value=\"$ii\" " . ($ii == $jour ? "selected" : "") . "> $ii</option>";
    }
    echo "</select>";
    echo "<select name=\"mois_$param\"> ";
    // <option value=\"mois\"> mois</option>";
    for ($i = 1; $i <= 12; $i ++) {
        $ii = ($i < 10 ? "0$i" : $i);
        echo "<option value=\"$ii\" " . ($ii == $mois ? "selected" : "") . "> $tab_mois[$i]</option>";
    }
    echo "</select>";
    echo "<select name=\"an_$param\"> ";
    // <option value=\"an\"> ann&#233;e</option>";
    echo "<option value=\"$an\" selected> $an</option>";
    $an_suivant = $an + 1;
    echo "<option value=\"$an_suivant\" > $an_suivant</option>";
    echo "</select>";
}

/**
 * Test si uid est prof
 *
 * @Parametres uid
 * @return 1 su oui 0 si non
 */
function est_prof($uid)
{
    $groupes = filter_group("memberUid=$uid");
    $prof = 0;
    $n = count($groupes);
    if ($n > 0)
        for ($i = 0; $i < $n; $i ++)
            if ($groupes[$i]["cn"] == "Profs") {
                $prof = 1;
                break;
            }
    return $prof;
}

/**
 *
 * fournit la liste des classes d'un prof
 * modif : inclut aussi les groupes auxquels le prof appartient
 * prevoir un tableau gid => cn
 *
 * @Parametres login du prof
 * @return un tableau avec les classes
 */
function classes_prof($login)
{
    $classes = array();
    list ($user, $groups) = search_user($login, true);
    $nb_groupes = count($groups);
    for ($g = 0; $g < $nb_groupes; $g ++) {
        if (preg_match("/^Equipe/", $groups[$g]["cn"]))
            $classes[] = preg_replace("/^Equipe/", "Classe", $groups[$g]["cn"]);
        elseif (preg_match("/^Matiere/", $groups[$g]["cn"]))
            continue;
        elseif (preg_match("/^Cours/", $groups[$g]["cn"]))
            continue;
        elseif (preg_match("/^Profs/", $groups[$g]["cn"]))
            continue;
        elseif (preg_match("/^admins/", $groups[$g]["cn"]))
            continue;
        else
            $classes[] = $groups[$g]["cn"];
    }
    return $classes;
}

/**
 *
 * retourne une liste a partir d'un tableau
 *
 * @Parametres tab le tableau a transformer
 * @return une liste
 *        
 */
function tab_liste($tab)
{
    $liste = "";
    foreach ($tab as $cle => $val) {
        if ($val != "" and $val != "#")
            $liste .= $cle . '|' . $val . '|';
    }
    $liste = preg_replace("/|$/", "", $liste);
    return $liste;
}

/**
 * a partir d'un uid d'élève fournit le nom inversé de son répertoire classe
 * en cas de login prenom.nom, renvoie nom.prenom, sinon renvoie le login
 *
 * @Parametres $uid
 *
 * @return $rep
 */
function inverse_login($login)
{
    $tab = preg_split('/\./', $login);
    if (count($tab) == 2) {
        $rep = $tab[1] . "." . $tab[0];
    } else {
        $rep = $login;
    }
    return $rep;
}

function affiche_all_groups($config, $align, $afftype)
{
    // // Etablissement des listes des groupes disponibles
    $list_groups = search_ad($config, "*", "group");
    // Etablissement des sous listes de groupes :
    $j = 0;
    $k = 0;
    $m = 0;
    $l = 0;
    $i = 0;
    $cours = $matiere = $equipe = $autres = array();
    for ($loop = 0; $loop < count($list_groups); $loop ++) {
        // Cours
        if (preg_match("/Cours_/", $list_groups[$loop]["cn"])) {
            $cours[$i]["cn"] = $list_groups[$loop]["cn"];
            $cours[$i]["description"] = $list_groups[$loop]["description"] ?? "";
            $i ++;
        }
        // // Classe
        if (preg_match("/Classe_/", $list_groups[$loop]["cn"])) {
            $classe[$j]["cn"] = $list_groups[$loop]["cn"];
            $classe[$j]["description"] = $list_groups[$loop]["description"] ?? "";
            $j ++;
        } // Matiere
        elseif (preg_match("/Matiere_/", $list_groups[$loop]["cn"])) {
            $matiere[$l]["cn"] = $list_groups[$loop]["cn"];
            $matiere[$l]["description"] = $list_groups[$loop]["description"] ?? "";
            $l ++;
        } // Equipe
        elseif (preg_match("/Equipe_/", $list_groups[$loop]["cn"])) {
            $equipe[$k]["cn"] = $list_groups[$loop]["cn"];
            $equipe[$k]["description"] = $list_groups[$loop]["description"] ?? "";
            $k ++;
        } // Autres
          // elseif (! preg_match("/^overfill/", $list_groups[$loop]["cn"]) && ! preg_match("/^lcs-users/", $list_groups[$loop]["cn"]) && ! preg_match("/nogroup/", $list_groups[$loop]["cn"]) && ! preg_match("/Cours_/", $list_groups[$loop]["cn"]) && ! preg_match("/Matiere_/", $list_groups[$loop]["cn"]) && ! preg_match("/root/", $list_groups[$loop]["cn"]) && ! preg_match("/invites/", $list_groups[$loop]["cn"]) && ! preg_match("/^machines/", $list_groups[$loop]["cn"])) {
          // $autres[$m]["cn"] = $list_groups[$loop]["cn"];
          // $autres[$m]["description"] = $list_groups[$loop]["description"] ?? "";
          // $m ++;
          // }
    }

    // Affichage des boites de s&#233;lection des groupes sur lesquels fixer les quotas + choix d'un user sp&#233;cifique
    echo "
		<table align=\"$align\" border=\"0\" cellspacing=\"10\" style='float: none' >
		<tr>
		<td>" . gettext("Classes") . "</td>";
    echo "<td>" . gettext("Mati&#232res") . "</td>";
    if ($afftype == "cours")
        echo "<td>" . gettext("Cours") . "</td>";
    echo "<td>" . gettext("Equipes") . "</td>
		<td>" . gettext("Autres") . "</td>";
    if ($afftype == "user")
        echo "<td>" . gettext("Utilisateur sp&#233;cifique") . "</td>";
    echo "</tr>
		<tr>
		<td valign=\"top\">
        
		<select name= \"classe_gr[]\" size=\"8\" multiple=\"multiple\">\n";
    for ($loop = 0; $loop < count($classe); $loop ++) {
        echo "<option value=" . $classe[$loop]["cn"] . ">" . $classe[$loop]["cn"];
    }
    echo "</select>";
    echo "</td>\n";

    echo "<td valign=\"top\">\n";
    echo "<select name= \"matiere_gr[]\" size=\"8\" multiple=\"multiple\">\n";
    for ($loop = 0; $loop < count($matiere); $loop ++) {
        echo "<option value=" . $matiere[$loop]["cn"] . ">" . $matiere[$loop]["cn"];
    }
    echo "</select></td>\n";
    // echo "<td valign=\"top\">";

    if ($afftype == "cours") {
        echo "<td valign=\"top\">\n";
        echo "<select name= \"cours_gr[]\" value=\"$cours_gr\" size=\"10\" multiple=\"multiple\">";
        for ($loop = 0; $loop < count($cours); $loop ++) {
            echo "<option value=" . $cours[$loop]["cn"] . ">" . $cours[$loop]["cn"];
        }
        echo "</select>";
        echo "</td>";
    }

    echo "<td valign=\"top\">\n";
    echo "<select name= \"equipe_gr[]\" size=\"8\" multiple=\"multiple\">\n";
    for ($loop = 0; $loop < count($equipe); $loop ++) {
        echo "<option value=" . $equipe[$loop]["cn"] . ">" . $equipe[$loop]["cn"];
    }
    echo "</select></td>\n";
    echo "<td valign=\"top\">";
    $autres = filter_group_autres($config);
    $autres_projets = filter_group_projets($config);
    echo "<select name=\"autres_gr[]\" size=\"8\" multiple=\"multiple\">";
    echo "<option value=administratifs>administratifs";
    echo "<option value=eleves>eleves";
    echo "<option value=profs>profs";
    for ($loop = 0; $loop < count($autres); $loop ++) {
        echo "<option value=" . $autres[$loop]["cn"] . ">" . $autres[$loop]["cn"];
    }
    for ($loop = 0; $loop < count($autres_projets); $loop ++) {
        echo "<option value=" . $autres_projets[$loop]["cn"] . ">" . $autres_projets[$loop]["cn"];
    }
    echo "</select></td>\n";
    if ($afftype == "user")
        echo "<td valign=\"top\"><INPUT TYPE=\"TEXT\" NAME=\"user\" size=15></td>\n";
    echo "</tr><tr></tr></table>\n\n";
}

?>
