<?php

/*
 * fonctions de manipultion des GPO pour sambaedu
 * @Projet SambaEdu
 *
 * @Note: Ce fichier de fonction doit etre appele par un include
 *
 * @Licence Distribue sous la licence GPL
 *
 * file: gpo.inc.php
 *
 * @Repertoire: includes/
 */
define('REG_NONE', 0);
define('REG_SZ', 1);
define('REG_EXPAND_SZ', 2);
define('REG_BINARY', 3);
define('REG_DWORD', 4);
define('REG_DWORD_BIG_ENDIAN', 5);
define('REG_LINK', 6);
define('REG_MULTI_SZ', 7);
define('REG_RESOURCE_LIST', 8);
define('REG_FULL_RESOURCE_DESCRIPTOR', 9);
define('REG_RESOURCE_REQUIREMENTS_LIST', 10);
define('REG_QWORD', 11);

define('REGFILE_SIGNATURE', "PReg");
define('REGFILE_VERSION', "\1\0\0\0");
define('UTF32_BIG_ENDIAN_BOM', chr(0x00) . chr(0x00) . chr(0xFE) . chr(0xFF));
define('UTF32_LITTLE_ENDIAN_BOM', chr(0xFF) . chr(0xFE) . chr(0x00) . chr(0x00));
define('UTF16_BIG_ENDIAN_BOM', chr(0xFE) . chr(0xFF));
define('UTF16_LITTLE_ENDIAN_BOM', chr(0xFF) . chr(0xFE));
define('UTF8_BOM', chr(0xEF) . chr(0xBB) . chr(0xBF));

function detect_utf_encoding($text)
{
    $first2 = substr($text, 0, 2);
    $first3 = substr($text, 0, 3);
    $first4 = substr($text, 0, 3);

    if ($first3 == UTF8_BOM)
        return 'UTF-8';
    elseif ($first4 == UTF32_BIG_ENDIAN_BOM)
        return 'UTF-32BE';
    elseif ($first4 == UTF32_LITTLE_ENDIAN_BOM)
        return 'UTF-32LE';
    elseif ($first2 == UTF16_BIG_ENDIAN_BOM)
        return 'UTF-16BE';
    elseif ($first2 == UTF16_LITTLE_ENDIAN_BOM)
        return 'UTF-16LE';
}

/*
 * Fonction de comparaison pour trier les listes des GPO
 * par la clé displayname
 *
 *
 */
function compare_list_gpo_by_name($gpo1, $gpo2)
{
    return strcasecmp($gpo1['displayname'], $gpo2['displayname']);
}

function dword2int($str)
{
    $bytes = unpack("nlow/nhigh", $str);
    if (count($bytes) == 2) {
        return $bytes['low'] + 65536 * $bytes['high'];
    } else {
        return NULL;
    }
}

function int2dword(int $int)
{
    $l = $int % 65536;
    $h = intdiv($int, 65536);
    return pack("v2", $l, $h);
}

function dstr2str($str)
{
    return iconv("UTF-16LE", "UTF-8", substr($str, 1, strlen($str) - 1));
}

function str2dstr($str)
{
    return iconv("UTF-8", "UTF-16LE", $str); // . "\0";
}

function char2dchar($str)
{
    return iconv("UTF-8", "UTF-16LE", $str);
}

function get_domain_sid($config)
{
    $out = exec("wbinfo --name-to-sid " . $config['ldap_admin_name']);
    return preg_replace("/-[0-9]+ .*$/", "", $out);
}

/**
 * lit un fichier gpo .pol, le résultat est rangé dans un tableau, les chaines de caractere sont en utf-8
 * le tableau peut directement être utilié pour recontruire un fichier .pol
 * seuls les valeurs REG_DWORD et REG_SZ sont réellement parsées
 * Les données non manipulables simplement sont laissées telles quelles.
 *
 * @param unknown $file
 * @return array
 */
function read_pol($file)
{
    $pol = fopen($file, 'rb');
    $info = fread($pol, filesize($file));
    fclose($pol);
    $gpo = array();
    mb_regex_encoding("UTF-8");
    mb_internal_encoding("UTF-16LE");
    $table = mb_split("\[", $info);
    array_shift($table);
    foreach ($table as $ligne) {
        $ligne = mb_split("\]", $ligne)[0];
        $pack = mb_split(";", $ligne);
        $cle = array(
            'key' => dstr2str($pack[0]),
            'value' => dstr2str($pack[1]),
            'type' => dword2int($pack[2]),
            'size' => dword2int($pack[3])
        );
        if (($cle['type'] == REG_SZ) || ($cle['type'] == REG_EXPAND_SZ) || ($cle['type'] == REG_MULTI_SZ)) {
            $cle['data'] = dstr2str($pack[4]);
        } elseif ($cle['type'] == REG_DWORD) {
            $cle['data'] = dword2int($pack[4]);
        } else {
            $cle['data'] = $pack[4];
        }
        $gpo[] = $cle;
    }
    return $gpo;
}

/**
 * ecrit un fichier gpo .pol, à partir d'un tableau
 * seuls les valeurs REG_DWORD et REG_SZ sont réellement parsées
 * Les données non manipulables simplement sont laissées telles quelles.
 *
 * @param unknown $file
 * @return array
 */
function write_pol($file, $gpo)
{
    $info = REGFILE_SIGNATURE . REGFILE_VERSION;
    foreach ($gpo as $cle) {
        if (($cle['type'] == REG_SZ) || ($cle['type'] == REG_EXPAND_SZ) || ($cle['type'] == REG_MULTI_SZ)) {
            $data = str2dstr($cle['data']);
        } elseif ($cle['type'] == REG_DWORD) {
            $data = int2dword($cle['data']);
        } else {
            $data = $cle['data'];
        }
        $size = strlen($data);

        $info .= char2dchar("[");
        $info .= str2dstr($cle['key']);
        $info .= char2dchar(";");
        $info .= str2dstr($cle['value']);
        $info .= char2dchar(";");
        $info .= int2dword($cle['type']);
        $info .= char2dchar(";");
        $info .= int2dword($size);
        $info .= char2dchar(";");
        $info .= $data;
        $info .= char2dchar("]");
    }
    $pol = fopen($file, 'wb');
    $ret = fwrite($pol, $info);
    fclose($pol);
    return $ret;
}

function read_pref($file)
{}

function form_pol($config, $gpo)
{}

/**
 * modification du parametrage du proxy à partir des parametres de conf
 * Le parametrage est réécrit complètement, on ne remplace pas les valeurs existantes
 * parametrage proxy windows (IE8 et 10) et mozilla
 *
 * @param unknown $config
 */
function change_proxy_file($config, $line)
{
    if ($config['proxy_type'] != "automatique" && ! empty($config['proxy_address']) && ! empty($config['proxy_port'])) {
        $config['proxy_url'] = "http://" . $config['proxy_address'] . ":" . $config['proxy_port'];
    }
    if (! empty($config['proxy_url']) && empty($config['proxy_type'])) {
        $config['proxy_type'] = "manuel";
    } else {
        $config['proxy_type'] = "";
    }
    switch ($config['proxy_type']) {
        case "automatique":
            $config['proxy_autoconfigurl'] = $config['proxy_url'];
            $config['proxy_server'] = "http://" . @$config['proxy_address'] . ":" . @$config['proxy_port'];
            $config['proxy_auto'] = 1;
            $config['proxy_enable'] = 1;
            break;
        case "manuel":
            $config['proxy_autoconfigurl'] = "";
            $config['proxy_auto'] = 0;
            $config['proxy_server'] = $config['proxy_url'];
            $config['proxy_enable'] = 1;
            break;
        default:
            $config['proxy_autoconfigurl'] = "";
            $config['proxy_auto'] = 0;
            $config['proxy_server'] = "";
            $config['proxy_enable'] = 0;
            $config['proxy_url'] = "";
    }
    $xml = new DOMDocument();
    $xml->formatOutput = false;
    $xml->preserveWhiteSpace = false;
    $xml->loadXML($line);
    $xpath = new DOMXPath($xml);

    // Vintage

    if ($key = $xpath->query("/InternetSettings/IE8/Properties/Reg[@name='ProxyOverride']"))
        $key[0]->setAttribute('value', $config['proxy_url'] . ";<local>");
    if ($key = $xpath->query("/InternetSettings/IE8/Properties/Reg[@name='AutoConfigURL']"))
        $key[0]->setAttribute('value', $config['proxy_autoconfigurl']);
    if ($key = $xpath->query("/InternetSettings/IE8/Properties/Reg[@name='ProxyEnable']"))
        $key[0]->setAttribute('value', $config['proxy_enable']);
    if ($key = $xpath->query("/InternetSettings/IE8/Properties/Reg[@name='ProxyServer']"))
        $key[0]->setAttribute('value', $config['proxy_server']);
    if ($key = $xpath->query("/InternetSettings/IE8/Properties/Reg[@name='AutoDetect']"))
        $key[0]->setAttribute('value', $config['proxy_auto']);

    if ($key = $xpath->query("/InternetSettings/IE10/Properties/Reg[@name='ProxyOverride']"))
        $key[0]->setAttribute('value', $config['proxy_url'] . ";<local>");
    if ($key = $xpath->query("/InternetSettings/IE10/Properties/Reg[@name='AutoConfigURL']"))
        $key[0]->setAttribute('value', $config['proxy_autoconfigurl']);
    if ($key = $xpath->query("/InternetSettings/IE10/Properties/Reg[@name='ProxyEnable']"))
        $key[0]->setAttribute('value', $config['proxy_enable']);
    if ($key = $xpath->query("/InternetSettings/IE10/Properties/Reg[@name='ProxyServer']"))
        $key[0]->setAttribute('value', $config['proxy_server']);
    if ($key = $xpath->query("/InternetSettings/IE10/Properties/Reg[@name='AutoDetect']"))
        $key[0]->setAttribute('value', $config['proxy_auto']);

    return $xml->saveXML();
}

function change_proxy_pol($config)
{}

function generalise_gpo($config, $source)
{
    $pol_list = [
        $source . "/User/Registry.pol",
        $source . "/Machine/Registry.pol"
    ];
    $params = [
        "domain",
        "samba_domain",
        "se4fs_name",
        "se4ad_name",
        "domain_sid",
        "se4install_name"
    ];

    $config['domain_sid'] = get_domain_sid($config);

    foreach ($pol_list as $pol) {
        if (file_exists($pol)) {
            $gpo = read_pol($pol);
            foreach ($gpo as $i => $cle) {
                foreach ($params as $param) {
                    $gpo[$i]['key'] = preg_replace("/" . $config[$param] . "/", "###_" . strtoupper($param) . "_###", $gpo[$i]['key']);
                    $gpo[$i]['value'] = preg_replace("/" . $config[$param] . "/", "###_" . strtoupper($param) . "_###", $gpo[$i]['value']);
                    $gpo[$i]['data'] = preg_replace("/" . $config[$param] . "/", "###_" . strtoupper($param) . "_###", $gpo[$i]['data']);
                }
            }
            write_pol($pol, $gpo);
        }
    }

    if (is_dir($source) === true) {
        $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);

        foreach ($files as $file) {

            // Ignore "." and ".." folders
            if (in_array(substr($file, strrpos($file, '/') + 1), array(
                '.',
                '..',
                '.pol'
            )))
                continue;

            $file = realpath($file);

            if (is_file($file) === true) {
                $proxy_xml = preg_match("/InternetSettings\.xml/", $file);
                if ($proxy_xml != 1) {
                    $content = file_get_contents($file);
                    $enc = detect_utf_encoding($content);

                    if ($enc == 'UTF-16LE') {
                        $content = mb_convert_encoding($content, "UTF-8", "UTF-16LE");
                    }
                    foreach ($params as $param) {

                        $content_chunks = explode($config[$param], $content);
                        $content = implode("###_" . strtoupper($param) . "_###", $content_chunks);
                    }
                    $content_chunks = explode(">" . strtoupper($config['samba_domain']) . "\\", $content);
                    $content = implode(">###_SAMBA_DOMAIN_###\\", $content_chunks);
                    if ($enc == 'UTF-16LE') {
                        $content = mb_convert_encoding($content, "UTF-16LE", "UTF-8");
                    }
                    file_put_contents($file, $content);
                }
            }
        }
    }
}

function specialise_gpo($config, $source)
{
    $pol_list = [
        $source . "/User/Registry.pol",
        $source . "/Machine/Registry.pol"
    ];
    $params = [
        "domain",
        "samba_domain",
        "se4fs_name",
        "se4ad_name",
        "domain_sid",
        "se4install_name"
    ];
    $config['domain_sid'] = get_domain_sid($config);

    foreach ($pol_list as $pol) {
        if (file_exists($pol)) {
            $gpo = read_pol($pol);
            foreach ($gpo as $i => $cle) {
                foreach ($params as $param) {
                    $gpo[$i]['key'] = preg_replace("/###_" . strtoupper($param) . "_###/", $config[$param], $gpo[$i]['key']);
                    $gpo[$i]['value'] = preg_replace("/###_" . strtoupper($param) . "_###/", $config[$param], $gpo[$i]['value']);
                    $gpo[$i]['data'] = preg_replace("/###_" . strtoupper($param) . "_###/", $config[$param], $gpo[$i]['data']);
                }
                change_proxy_pol($config);
            }
            write_pol($pol, $gpo);
        }
    }
    if (is_dir($source) === true) {
        $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);

        foreach ($files as $file) {

            // Ignore "." and ".." folders
            if (in_array(substr($file, strrpos($file, '/') + 1), array(
                '.',
                '..',
                '.pol'
            )))
                continue;

            $file = realpath($file);

            if (is_file($file) === true) {
                $proxy_xml = preg_match("/InternetSettings\.xml/", $file);
                $content = file_get_contents($file);
                $enc = detect_utf_encoding($content);
                if ($enc == 'UTF-16LE') {
                    $content = mb_convert_encoding($content, "UTF-8", "UTF-16LE");
                }
                foreach ($params as $param) {
                    $content_chunks = explode("###_" . strtoupper($param) . "_###", $content);
                    $content = implode($config[$param], $content_chunks);
                }
                if ($proxy_xml == 1) {
                    $content = change_proxy_file($config, $content);
                }
                if ($enc == 'UTF-16LE') {
                    $content = mb_convert_encoding($content, "UTF-16LE", "UTF-8");
                }
                file_put_contents($file, $content);
            }
        }
    }
}

function zip_gpo($source, $destination)
{
    if (! extension_loaded('zip') || ! file_exists($source)) {
        return false;
    }

    $zip = new ZipArchive();
    if (! $zip->open($destination, ZIPARCHIVE::CREATE)) {
        return false;
    }

    if (is_dir($source) === true) {
        $files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);

        foreach ($files as $file) {

            // Ignore "." and ".." folders
            if (in_array(substr($file, strrpos($file, '/') + 1), array(
                '.',
                '..'
            )))
                continue;

            $file = realpath($file);

            if (is_dir($file) === true) {
                $zip->addEmptyDir(str_replace($source . '/', '', $file . '/'));
            } else if (is_file($file) === true) {
                $zip->addFromString(str_replace($source . '/', '', $file), file_get_contents($file));
            }
        }
    } else if (is_file($source) === true) {
        $zip->addFromString(basename($source), file_get_contents($source));
    }

    return $zip->close();
}

function unzip_gpo($gpo_zip, $name)
{
    $gpo_zip = "/var/www/sambaedu/gpo/templates/" . $gpo_zip;
    if (! extension_loaded('zip') || ! file_exists($gpo_zip)) {
        return false;
    }

    $zip = new ZipArchive();
    if ($zip->open($gpo_zip) === TRUE) {
        $tmppath = sys_get_temp_dir() . "/" . $name;
        if (! is_dir($tmppath)) {
            mkdir($tmppath);
        }
        $zip->extractTo($tmppath);
    } else {
        return false;
    }
    return $zip->close();
}

/**
 * recupère les informations d'une gpo à importer
 *
 * @param string $dir
 *            dossier de la gpo
 * @return array|boolean
 */
function get_gpo_template_info(string $gpo)
{
    $path = "/var/www/sambaedu/gpo/templates/";
    $zip = new ZipArchive();
    if ($zip->open($path . $gpo) === TRUE) {
        $gptini = parse_ini_string($zip->getFromName('GPT.INI'));
        $zip->close();
        if ($gptini && (isset($gptini['gpcmachineextensionnames']) || isset($gptini['gpcuserextensionnames'])))
            return $gptini;
    }
    return false;
}

/**
 * liste les templates de GPO disponibles pour l'import
 * qui contiennent un prefixe se4_ (issues du paquet)
 *
 * @return array
 */
function list_gpo_templates()
{
    $path = "/var/www/sambaedu/gpo/templates/";
    // $dir = opendir($path);
    $gpos = scandir($path);
    $templates = array();
    foreach ($gpos as $gpo) {
        $match = array();
        if (preg_match("/^(se4_.*\.zip)$/", $gpo, $match)) {
            if ($info = get_gpo_template_info($match[1])) {
                $templates[] = array(
                    'displayname' => $info['displayName'],
                    'version' => $info['Version']
                );
            }
        }
    }
    usort($templates, 'compare_list_gpo_by_name');
    return $templates;
}

/**
 * liste les templates de GPO disponibles pour l'import
 * qui contiennent un prefixe etab_ (issues d'un export ...)
 *
 * @return array
 */
function list_gpo_templates_etab()
{
    $path = "/var/www/sambaedu/gpo/templates/";
    // $dir = opendir($path);
    $gpos = scandir($path);
    $templates = array();
    foreach ($gpos as $gpo) {
        $match = array();
        if (preg_match("/^(etab_.*\.zip)$/", $gpo, $match)) {
            if ($info = get_gpo_template_info($match[1])) {
                $templates[] = array(
                    'displayname' => $info['displayName'],
                    'version' => $info['Version']
                );
            }
        }
    }
    usort($templates, 'compare_list_gpo_by_name');
    return $templates;
}

/**
 * met à jour une gpo sur l'AD à partir de l'archive $gpo_zip.zip dans /var/www/sambaaedu/gpo/templates.
 * Si la GPO existe déjà sur le serveur AD, incrémente le numéro de version.
 * utilise smbclient pour copier les fichiers.
 *
 * @param array $config
 * @param string $displayname
 *            : nom de la gpo
 * @param string $gpo_zip
 *            : nom de l'archive ($gpo_zip.zip) normalement le même que $displayname
 * @return boolean
 */
function import_gpo(array $config, string $displayname, string $gpo_zip)
{
    $message = "";
    $ret = 0;
    $gpo = search_ad($config, $displayname, "gpo");
    if (count($gpo) > 0) {

        $tmppath = sys_get_temp_dir() . "/" . $displayname;
        $info = get_gpo_template_info($gpo_zip);
        $increment = 0;
        if (isset($info['gpcuserextensionnames']))
            $increment = 0x10000;
        if (isset($info['gpcmachineextensionnames']))
            $increment = $increment + 1;
        $version = max([
            (int) $info['Version'],
            (int) $gpo[0]['versionnumber']
        ]) + $increment;
        $content = "[General]\r\nVersion=" . $version . "\r\ndisplayName=" . $displayname . "\r\n";
        if (unzip_gpo($gpo_zip, $displayname)) {
            $handle = fopen($tmppath . "/GPT.INI", "w");
            fwrite($handle, $content);
            fclose($handle);
            specialise_gpo($config, $tmppath);
            $command = "cd \"" . $config['domain'] . "/Policies/" . $gpo[0]['cn'] . "\";lcd \"" . $tmppath . "\";prompt OFF;recurse ON;mput *";
            exec('smbclient //se4ad/sysvol -k -c ' . escapeshellarg($command), $message, $ret);
            // exec("rm -fr " . $tmppath, $message, $ret);
            if ($ret == 0) {
                $attrs = array();
                $attrs['versionnumber'] = $version;
                if (isset($info['gpcuserextensionnames']))
                    $attrs['gpcuserextensionnames'] = $info['gpcuserextensionnames'];
                if (isset($info['gpcmachineextensionnames']))
                    $attrs['gpcmachineextensionnames'] = $info['gpcmachineextensionnames'];
                if (isset($info['gpcfunctionalityversion']))
                    $attrs['gpcfunctionalityversion'] = $info['gpcfunctionalityversion'];

                return modify_ad($config, $gpo[0]['cn'], "gpo", $attrs);
            } else
                return false;
        } else
            return false;
    } else {
        $uuid = gpocreate($config, $displayname);
        if ($uuid) {
            if (import_gpo($config, $displayname, $gpo_zip))
                return gposetlink($config, $config['ldap_base_dn'], $uuid);
            else
                return false;
        } else
            return false;
    }
}

/**
 * Exporte vers le se4fs une GPO présente dans L'AD sous la forme d'une archive zip
 * en la prefixant d'un superbe "etab_"
 *
 * @param array $config
 * @param string $displayname
 * @return boolean
 */
function export_gpo(array $config, string $displayname)
{
    $gpo = search_ad($config, $displayname, "gpo");
    if (count($gpo) > 0) {
        $message = array();
        $res = 1;
        $tmppath = sys_get_temp_dir() . "/" . $displayname;
        exec("rm -fr " . escapeshellarg($tmppath), $message, $res);
        exec("mkdir -p " . escapeshellarg($tmppath), $message, $res);
        $command = "cd \"" . $config['domain'] . "/Policies/" . $gpo[0]['cn'] . "\";lcd \"" . $tmppath . "\";prompt OFF;recurse ON;mget *";
        exec('smbclient //se4ad/sysvol -k -c ' . escapeshellarg($command), $message, $res);
        $content = "[General]\r\nVersion=" . $gpo[0]['versionnumber'] . "\r\ndisplayName=" . $displayname . "\r\n";
        $content .= "[CSE]\r\n";
        if (isset($gpo[0]['gpcuserextensionnames']))
            $content .= "gpcuserextensionnames = \"" . $gpo[0]['gpcuserextensionnames'] . "\"\r\n";
        if (isset($gpo[0]['gpcmachineextensionnames']))
            $content .= "gpcmachineextensionnames = \"" . $gpo[0]['gpcmachineextensionnames'] . "\"\r\n";
        if (isset($gpo[0]['gpcfunctionalityversion']))
            $content .= "gpcfunctionalityversion = \"" . $gpo[0]['gpcfunctionalityversion'] . "\"\r\n";
        $handle = fopen($tmppath . "/GPT.INI", "w");
        fwrite($handle, $content);
        fclose($handle);
        generalise_gpo($config, $tmppath);
        // creation archive zip
        $zip = "/var/www/sambaedu/gpo/templates/etab_" . $displayname . ".zip";
        exec("rm -fr " . escapeshellarg($zip), $message, $res);
        if (zip_gpo($tmppath, $zip)) {
            exec("rm -fr " . escapeshellarg($tmppath), $message, $res);
            return "etab_" . $displayname . ".zip";
        }
    }
    return false;
}

function increment_gpo(array $config, string $gpo)
{
    $gpo = search_ad($config, $gpo, "gpo");
    $zip = export_gpo($config, $gpo[0]['displayname']);
    return import_gpo($config, $gpo[0]['displayname'], $zip);
}

function set_proxy_gpo(array $config)
{
    $html = "";
    if (increment_gpo($config, "proxy"))
        $html .= "gpo proxy modifiée<br>\n";
    if (increment_gpo($config, "mozilla")) {
        $html .= "gpo mozilla modifiée<br>\n";
    }
    return $html;
}
?>
