<?php

/**
 * Fonctions LDAP

 * @Projet LCS / SambaEdu

 * @Auteurs Equipe Sambaedu

 * @Version $Id: ldap.inc.php  09-11-2018 mrfi $

 * @Note: Ce fichier de fonction doit etre appele par un include

 * @Licence Distribue sous la licence GPL
 */
/**
 *
 * file: ldap.inc.php
 *
 * @Repertoire: includes/
 */
require_once ("lang.inc.php");
bindtextdomain('sambaedu-core', "/var/www/sambaedu/locale");
textdomain('sambaedu-core');

// pour utiliser bind_ad_gssapi
include_once "functions.inc.php";
require_once "samba-tool.inc.php";
require_once "dhcpd.inc.php";

// Pour activer/desactiver la modification du givenname (Prenom) lors de la modification dans annu/mod_user_entry.php
$corriger_givenname_si_diff = "n";

// fonctions validées se4
function array_unique_multi($input)
{
    $serialized = array_map('serialize', $input);
    $unique = array_unique($serialized);
    return array_intersect_key($input, $unique);
}

/**
 * Comparaison sur fullname
 *
 * @description Fonctions de comparaison utilisees dans la fonction usort, pour trier le fullname
 * @type
 * @Parametres $a - La premiere entree 	$b - La deuxieme entree a comparer
 * @returns < 0 - Si $a est plus petit a $b > 0 - Si $a est plus grand que $b
 */
function cmp_fullname($a, $b)
{
    return strcmp($a["fullname"], $b["fullname"]);
}

/**
 * Comparaison sur name
 *
 * @description Fonctions de comparaison utilisees dans la fonction usort, pour trier le name
 * @type
 * @Parametres $a - La premiere entree 	$b - La deuxieme entree a comparer
 * @returns < 0 - Si $a est plus petit a $b > 0 - Si $a est plus grand que $b
 *
 */
function cmp_nom($a, $b)
{
    $a['nom'] = $a['nom'] ?? "";
    $b['nom'] = $b['nom'] ?? "";
    return strcmp($a["nom"], $b["nom"]);
}

/**
 * Comparaison sur cn
 *
 * @description Fonctions de comparaison utilisees dans la fonction usort, pour trier le cn (common name)
 * @Parametres  $a - La premiere entree  $b - La deuxieme entree a comparer
 * @type
 * @returns < 0 - Si $a est plus petit a $b > 0 - Si $a est plus grand que $b
 */
function cmp_cn($a, $b)
{
    return strcmp($a["cn"], $b["cn"]);
}

/**
 * Comparaison sur groupe
 *
 * @description Fonctions de comparaison utilisees dans la fonction usort, pour trier les groupes
 * @type
 * @Parametres  $a - La premiere entree 	$b - La deuxieme entree a comparer
 * @returns < 0 - Si $a est plus petit a $b > 0 - Si $a est plus grand que $b
 *
 */
function cmp_group($a, $b)
{
    return strcmp($a["group"], $b["group"]);
}

/**
 * Comparaison sur categories
 *
 * @description Fonctions de comparaison utilisees dans la fonction usort, pour trier les categories
 * @type
 * @Parametres  $a - La premiere entree  $b - La deuxieme entree a comparer
 * @returns < 0 - Si $a est plus petit a $b > 0 - Si $a est plus grand que $b
 */
function cmp_cat($a, $b)
{
    return strcmp($a["cat"], $b["cat"]);
}

/**
 * commparaison sur printer-name
 *
 * @description Fonctions de comparaison utilisees dans la fonction usort, pour trier le printer-name, insensible a la case
 * @type
 * @Parametres  $a - La premiere entree  $b - La deuxieme entree a comparer
 * @returns < 0 - Si $a est plus petit a $b > 0 - Si $a est plus grand que $b
 */
function cmp_printer($a, $b)
{
    return strcasecmp($a["name"], $b["name"]);
}

/**
 * commparaison sur printer-location
 *
 * @description Fonctions de comparaison utilisees dans la fonction usort, pour trier le printer-location, insensible a la case
 * @type
 * @Parametres  $a - La premiere entree  $b - La deuxieme entree a comparer
 * @returns < 0 - Si $a est plus petit a $b > 0 - Si $a est plus grand que $b
 */
function cmp_location($a, $b)
{
    return strcasecmp($a["location"], $b["location"]);
}

function sort_by_key($key, &$array)
{
    array_multisort(array_column($array, $key), SORT_ASC, $array);
}

/**
 * dn vers cn
 *
 * @description permet de passer du format dn complet au format cn
 * @type
 * @Parametres string $dn
 * @returns string $cn
 */
function ldap_dn2cn_case(string $dn)
{
    $rdns = explode(",", $dn);
    $cn = explode("=", $rdns[0]);
    return $cn[1];
}

/**
 * dn vers cn
 *
 * @description permet de passer du format dn complet au format cn
 * @type
 * @Parametres string $dn
 * @returns string $cn
 */
function ldap_dn2cn(string $dn)
{
    $rdns = explode(",", ldap_dn2ufn($dn));
    return $rdns[0];
}

/**
 * dn vers ou
 *
 * @description permet de passer du format dn complet au format ou=truc
 * @type
 * @Parametres string $dn
 * @returns string $cn
 */
function ldap_dn2ou(string $dn)
{
    $rdns = explode(",", $dn, 2);
    $ou = $rdns[0];
    return $ou;
}

/**
 * retourne le dn de l'ou parente
 *
 * @param string $dn
 * @return mixed
 */
function ldap_dn2oudn(string $dn)
{
    $rdns = explode(",", $dn, 2);
    return $rdns[1];
}

/**
 * formate le samaccountname pour une machine en le mettant en minuscule et sans le $ final
 *
 * @param array $machine
 * @return mixed
 */
function ldap_hostname(array $machine)
{
    $hostname = $machine['samaccountname'] ?? $machine['cn'];
    return str_replace("$", "", strtolower($hostname));
}

function to_guid($guid)
{
    $hex_guid = unpack("H*hex", $guid);
    $hex = $hex_guid["hex"];

    $hex1 = substr($hex, - 26, 2) . substr($hex, - 28, 2) . substr($hex, - 30, 2) . substr($hex, - 32, 2);
    $hex2 = substr($hex, - 22, 2) . substr($hex, - 24, 2);
    $hex3 = substr($hex, - 18, 2) . substr($hex, - 20, 2);
    $hex4 = substr($hex, - 16, 4);
    $hex5 = substr($hex, - 12, 12);

    $guid = $hex1 . "-" . $hex2 . "-" . $hex3 . "-" . $hex4 . "-" . $hex5;

    return $guid;
}

/**
 *
 * Fonction de generation de mot de passe recuperee sur TotallyPHP
 * Aucune mention de licence pour ce script...
 *
 * @Parametres
 * @return string $password
 *        
 *         The letter l (lowercase L) and the number 1
 *         have been removed, as they can be mistaken
 *         for each other.
 */
function create_random_password(int $nb_chars, bool $complex = false)
{
    if ($complex) {
        $chars = 'ABCDEFGHIJKLMN_PQRSTUVWXYZ&*$abcdefghijkmnopqrstuvwxyz023456789';
        $nchars = 62;
    } else {
        $chars = "abcdefghijkmnopqrstuvwxyz023456789";
        $nchars = 33;
    }
    srand((double) microtime() * 1000000);
    $i = 0;
    $pass = '';

    while ($i <= $nb_chars) {
        $num = rand() % $nchars;
        $tmp = substr($chars, $num, 1);
        $pass = $pass . $tmp;
        $i ++;
    }

    return $pass;
}

/**
 * A définir !
 *
 * @description todo
 * @type
 * @Parametres
 * @returns
 */
function remove_count($arr)
{
    foreach ($arr as $key => $val) {
        if ($key === "count")
            unset($arr[$key]);
        elseif (is_array($val))
            $arr[$key] = remove_count($arr[$key]);
    }
    return $arr;
}

/**
 * Connexion avec l'AD en GSSAPI
 *
 * @description Connexion GSSAPI utilisée par les fonctions de plus haut niveau search_ad, modify_ad, move_ad, etc...
 */
function bind_ad_gssapi($config, &$error = "")
{
    $url = "ldap://" . $config['se4ad_name'] . "." . $config['domain'];
    $ds = ldap_connect($url, '389');
    if ($ds) {
        ldap_set_option($ds, LDAP_OPT_PROTOCOL_VERSION, 3);
        ldap_set_option($ds, LDAP_OPT_REFERRALS, 0);
        $r = ldap_sasl_bind($ds, null, null, 'GSSAPI');
        if (! $r) {
            $error = gettext("Echec de l'Authentification.");
            $ds = false;
        }
    } else {
        $error = gettext("Erreur de connection kerberos au serveur AD");
    }
    return $ds;
}

function check_ad($config)
{
    // verification de l'arborescence AD
    if (empty($config['ad_check'])) {
        foreach ($config['dn'] as $branche) {
            ouadd($config, $branche, "", "dn");
        }
        groupadd($config, "Eleves", $config['groups_rdn'], "groupe des eleves");
        groupadd($config, "Profs", $config['groups_rdn'], "groupe des profs");
        groupadd($config, "Administratifs", $config['groups_rdn'], "groupe des administratifs");
        groupadd($config, "computer_is_admin", $config['rights_rdn'], "administration des ordinateurs");
        if (! have_right($config, "se3_is_admin", "admin")) {
            add_right($config, "admin", "se3_is_admin");
        }
        $admin = search_user($config, "admin");
        if (ldap_dn2ou($admin['dn']) == $config['admin_rdn']) {
            move_ad($config, "admin", "cn=admin," . $config['dn']['people'], "user");
        }
        set_config('ad_check', 1, 'sambaedu');
    }
}

/**
 * Recherche des objets dans l'AD
 *
 * @description Recherche des objets dans AD en fonction du type demandé
 * @type tableau
 * @Parametres $name - le nom de l'objet (cn|ou|dn) "*" pour tout
 * $type - le type d'objets à chercher : user, employeenumber, filter, member, memberof, pp, dn (default), machine, salle, parc, group, classe, equipe, matiere, projet, cours, autre, imprimante, delegation, rights, type, gpo
 * $branch - la branche de recherche "all" pour tout (sans effet pour dn)
 * @returns Retourne un tableau avec les objets et leurs attributs utiles
 */
function search_ad($config, $name, $type = "dn", $branch = "all", $attrs = array(), $scope = "subtree")
{
    // Initialisation
    $info = array();
    // correspondance des attributs: ldapAD => php uniquement pour les utilisateurs !
    $map = array(
        "sn" => "nom",
        "displayname" => "fullname",
        "givenname" => "prenom",
        "initials" => "pseudo",
        "mail" => "email",
        "telephonenumber" => "tel",
        "title" => "employeenumber"
    );

    // LDAP attributs
    switch ($type) {
        case "user":
            $filter = "(&(objectclass=user)(cn=" . $name . "))";
            $ldap_attrs = array(
                "cn", // login
                "displayname", // Prenom Nom
                "sn", // Nom
                "givenname", // Pseudo -> Prenom
                "mail", // Mail
                "telephonenumber", // Num telephone
                "description",
                "physicaldeliveryofficename", // Date de naissance,Sexe (F/M)
                "title", // numero unique siecle
                "initials", // pseudo
                "useraccountcontrol", // état du compte actif : 512, desactivé 514
                "memberof", // groupes
                "userprincipalname" // pseudo-adresse mail correspondant au login ENT
            );
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;
        case "employeenumber": // recherche par le n° Siecle/STS
            $filter = "(&(objectclass=user)(|(title=" . $name . ")(title=" . sprintf("%05d", $name) . ")(title=" . preg_replace("/^0*/", "", $name) . ")))";
            $ldap_attrs = array(
                "cn" // login
            );
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;
        case "userprincipalname": // recherche par l'identifiant ENT
            $filter = "(&(objectclass=user)(|(cn=" . $name . ")(userprincipalname=" . $name . "@" . $config['ent_domain'] . ")(userprincipalname=" . $name . "@" . $config['domain'] . ")))";
            $ldap_attrs = array(
                "cn" // login
            );
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;

        case "filter": // user ou group
            $filter = $name;
            $ldap_attrs = array(
                "cn",
                "displayname",
                "description" //
            );
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;
        case "member": // cherche si user ou group est dans la ou $branch (profs eleves administratifs)
            $filter = "(&(cn=" . $name . ")(objectclass=user))";
            $ldap_attrs = array(
                "cn" // login
            );
            $branch = "ou=" . $branch . "," . $config['dn']['people'];
            break;
        case "memberof": // cherche si user ou group est membre du groupe branch
            $filter = "(&(cn=" . $name . ")(objectclass=user)(memberof=cn=" . $branch . "*))";
            $ldap_attrs = array(
                "cn", // login
                "displayname", // Prenom Nom
                "givenname", // prenm
                "sn", // Nom
                "physicaldeliveryofficename",
                "mail"
            );
            $branch = $config['dn']['people'];
            break;
        case "pp": // cherche si user ou group est pp de la classe $branch
            if ($name == "*")
                $filter = "(&(objectclass=group)(cn=PP_*))";
            else
                $filter = "(&(objectclass=group)(cn=PP_*)(|(cn=PP_" . $name . ")(cn=" . $name . ")(dn=" . $name . ")(member=cn=" . $name . "*)))";
            $ldap_attrs = array(
                "cn", // login
                "member"
            );
            $branch = $config['dn']['groups'];
            break;

        case "dn":
            $ldap_attrs = array(
                "cn"
            );
            $filter = "(cn=*)";
            $branch = $name;
            break;
        case "machine":
            $leases = import_dhcp_leases($config);
        case "machine_fast":
            $ldap_attrs = array(
                "cn", // nom d'origine
                "samaccountname", // Nom netbios type NOM$
                "dnsHostname", // FDQN
                "location", // Emplacement
                "lastlogon", // derniere connexion
                "description", // Description de la machine
                "iphostnumber", // adresse ip reservée
                "networkaddress", // adresse mac
                "memberof", // appartenance aux groupes
                "netbootinitialization", // action IPXE (id)
                "netbootmachinefilepath", // action programmée (token)
                "operatingsystem", // Windows
                "operatingsystemversion", // build version
                "objectguid"
            );
            $filter = "(&(objectclass=computer)(|(cn=" . $name . ")(samaccountname=" . $name . ")(samaccountname=" . strtoupper($name) . "$)(dnshostname=" . $name . "." . $config['domain'] . ")(dnshostname=" . $name . ")(iphostnumber=" . $name . ")(networkaddress=" . $name . ")(netbootmachinefilepath=" . $name . ")(dn=" . $name . ")))";
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;
        case "machine_parc":
            $leases = import_dhcp_leases($config);
            $type = "machine";
            $ldap_attrs = array(
                "cn", // nom d'origine
                "samaccountname", // Nom netbios type NOM$
                "dnsHostname", // FDQN
                "location", // Emplacement
                "lastlogon", // derniere connexion
                "description", // Description de la machine
                "iphostnumber", // adresse ip reservée
                "networkaddress", // adresse mac
                "memberof", // appartenance aux groupes
                "netbootinitialization", // action IPXE (id)
                "netbootmachinefilepath", // action programmée (token)
                "operatingsystem", // Windows
                "operatingsystemversion", // build version
                "objectguid"
            );
            $filter = "(&(objectclass=computer)(|(memberof=" . $name . ")))";
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;
        case "remote":
            $ldap_attrs = array(
                "cn", // nom d'origine
                "member", // appartenance aux groupes
                "guacConfigProtocol", // protocole
                "guacConfigParameter" // Parametres
            );
            $filter = "(&(objectclass=guacConfigGroup)(|(cn=" . $name . ")(cn=remote_" . $name . ")))";
            $branch = "ou=rdp," . $config['ldap_base_dn'];
            break;

        case "ou":
            $filter = "(&(objectclass=organizationalunit)(ou=" . $name . "))";
            $branch = $config['dn']['people'];
            $ldap_attrs = array(
                "ou",
                "description"
            );
            break;
        case "salle":
            $filter = "(&(objectclass=organizationalunit)(ou=" . $name . "))";
            $branch = $config['dn']['computers'];
            $ldap_attrs = array(
                "ou",
                "description",
                "objectguid"
            );
            break;
        case "parc":
            $filter = "(&(objectclass=group)(cn=" . $name . "))";
            $branch = $config['dn']['parcs'];
            $ldap_attrs = array(
                "cn",
                "description",
                "member",
                "samaccountname",
                "objectguid"
            );
            break;
        case "group":
            if ($name == "*")
                $filter = "(objectclass=group)";
            else
                $filter = "(&(objectclass=group)(|(cn=" . $name . ")(dn=" . $name . ")))"; // (member=cn=" . $name . "*)))";
            $ldap_attrs = array(
                "cn",
                "description",
                "samaccountname",
                "member",
                "memberof"
            );
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;
        case "classe":
            if ($name == "*")
                $filter = "(&(objectclass=group)(cn=Classe_*))";
            else
                $filter = "(&(objectclass=group)(|(cn=Classe_" . $name . ")(cn=" . $name . ")(dn=" . $name . ")(member=cn=" . $name . ",ou=*)))";
            $ldap_attrs = array(
                "cn",
                "description",
                "member"
            );
            $branch = $config['classes_rdn'] . "," . $config['dn']['groups'];
            break;
        case "equipe":
            if ($name == "*")
                $filter = "(&(objectclass=group)(cn=Equipe_*))";
            else
                $filter = "(&(objectclass=group)(cn=Equipe_*)(|(cn=Equipe_" . $name . ")(cn=" . $name . ")(dn=" . $name . ")(member=cn=" . $name . ",ou=*)))";
            $ldap_attrs = array(
                "cn",
                "description",
                "member"
            );
            $branch = $config['equipes_rdn'] . "," . $config['dn']['groups'];
            break;
        case "matiere":
            if ($name == "*")
                $filter = "(&(objectclass=group)(cn=Matiere_*))";
            else
                $filter = "(&(objectclass=group)(cn=Matiere_*)(|(cn=Matiere_" . $name . ")(cn=" . $name . ")(dn=" . $name . ")(member=cn=" . $name . ",ou=*)))";
            $ldap_attrs = array(
                "cn",
                "description",
                "member"
            );
            $branch = $config['matieres_rdn'] . "," . $config['dn']['groups'];
            break;
        case "cours":
            if ($name == "*")
                $filter = "(&(objectclass=group)(cn=Cours_*))";
            else
                $filter = "(&(objectclass=group)(cn=Matiere_*)(|(cn=Cours_" . $name . ")(cn=" . $name . ")(dn=" . $name . ")(member=cn=" . $name . ",ou=*)))";
            $ldap_attrs = array(
                "cn",
                "description",
                "member"
            );
            $branch = $config['cours_rdn'] . "," . $config['dn']['groups'];
            break;
        case "projet":
            if ($name == "*")
                $filter = "(&(objectclass=group)(cn=Projet*))";
            else
                $filter = "(&(objectclass=group)(cn=Projet_*)(|(cn=Projet_" . $name . ")(cn=" . $name . ")(dn=" . $name . ")(member=cn=" . $name . ",ou=*)))";
            $ldap_attrs = array(
                "cn",
                "description",
                "member"
            );
            $branch = $config['projets_rdn'] . "," . $config['dn']['groups'];
            break;
        case "autre":
            if ($name == "*")
                $filter = "(&(objectclass=group)(cn=*))";
            else
                $filter = "(&(objectclass=group)(|(cn=" . $name . ")(cn=" . $name . ")(dn=" . $name . ")(member=cn=" . $name . ",ou=*)))";
            $ldap_attrs = array(
                "cn",
                "description",
                "member"
            );
            $branch = $config['other_groups_rdn'] . "," . $config['dn']['groups'];
            break;

        case "printer":
            if ($name == "*")
                $filter = "(objectclass=msPrint-ConnectionPolicy)";
            else
                $filter = "(&(objectclass=msPrint-ConnectionPolicy)(|(printername=" . $name . ")(cn=" . $name . ")))";
            $ldap_attrs = array(
                "cn",
                "printername",
                "servername",
                "printattributes",
                "uncname"
            );
            if ($branch == "all")
                $branch = "CN=Policies,CN=System," . $config['ldap_base_dn'];
            else
                $branch = $branch . ",CN=Policies,CN=System," . $config['ldap_base_dn'];
            break;
        case "container":
            if ($name == "*")
                $filter = "(objectclass=container)";
            else
                $filter = "(&(objectclass=container)(cn=" . $name . "))";
            $ldap_attrs = array(
                "cn"
            );
            if ($branch == "all")
                $branch = "CN=Policies,CN=System," . $config['ldap_base_dn'];
            else
                $branch = $branch . ",CN=Policies,CN=System," . $config['ldap_base_dn'];
            break;
        case "delegation":
            if ($name == "*")
                $filter = "(objectclass=group)";
            else
                $filter = "(&(objectclass=group)(|(member=" . $name . ")(member=cn=" . $name . "*)))";
            $ldap_attrs = array(
                "cn"
            );
            $branch = $config['dn']['delegations'];
            break;
        case "right":
            if ($name == "*") {
                $filter = "(objectclass=group)";
            } else {
                if (! is_array($name)) {
                    $filter = "(&(objectclass=group)(|(cn=" . $name . ")(member=cn=" . $name . "*)(member=" . $name . ")))";
                } else {
                    $filter = "(&(objectclass=group)(|";
                    foreach ($name as $value) {
                        $filter .= "(cn=" . $value . ")(member=cn=" . $value . "*)(member=" . $value . ")";
                    }
                    $filter .= "))";
                }
            }
            $branch = $config['dn']['rights'];
            $ldap_attrs = array(
                "cn",
                "description",
                "member"
            );
            break;
        case "type":
            $filter = "(|(objectclass=group)(objectclass=person)(objectclass=organizationalunit))";
            $ldap_attrs = array(
                "objectClass"
            );
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
            break;
        case "gpo":
            /*
             * liste les gpo, directeent en ldap pour eviter d'avoir à parser les resultats de samba-tool
             */
            if ($name == "*")
                $filter = "(objectclass=grouppolicycontainer)";
            else
                $filter = "(&(objectclass=grouppolicycontainer)(|(cn=" . $name . ")(displayname=" . $name . ")))";
            $ldap_attrs = array(
                "cn",
                "displayname",
                "gpcfilesyspath",
                "versionnumber",
                "gpcuserextensionnames",
                "gpcmachineextensionnames",
                "gpcfunctionalityversion",
                "flags"
            );
            $branch = "CN=Policies,CN=System," . $config['ldap_base_dn'];
            break;
        default:
            $filter = "(cn=" . $name . ")";
            $ldap_attrs = array(
                "cn"
            );
            if ($branch == "all") {
                $branch = $config['ldap_base_dn'];
            }
    }
    $ldap_attrs = array_merge($ldap_attrs, $attrs);
    $ret = array();
    $error = "";
    $ds = bind_ad_gssapi($config, $error);
    if ($ds) {
        // var_dump($branch);
        // var_dump($filter);
        // var_dump($ldap_attrs);
        if ($scope == "subtree")
            $result = ldap_search($ds, $branch, $filter, $ldap_attrs);
        elseif ($scope == "base")
            $result = ldap_list($ds, $branch, $filter, $ldap_attrs);

        // $res = var_dump(ldap_get_entries($ds, $result));
        @ldap_sort($ds, $result, "cn");
        if ($result) {
            $info = ldap_get_entries($ds, $result);
            // var_dump($info);
            foreach ($info as $key1 => $entry) {
                if (is_array($entry)) {
                    foreach ($entry as $key2 => $attr) {
                        if ("$key2" == "dn") {
                            $ret[$key1][$key2] = $attr;
                        }

                        if (is_array($attr)) {
                            if ((isset($map[$key2])) && ($type != "gpo")) {
                                $key = $map[$key2];
                            } else {
                                $key = $key2;
                            }
                            if ("$key" == "physicaldeliveryofficename") {
                                if (isset($attr[0])) {
                                    $tmp = preg_split('/,/', $attr[0], 4);
                                    $ret[$key1]['sexe'] = (isset($tmp[1]) ? $tmp[1] : "");
                                    $ret[$key1]['date'] = (isset($tmp[0]) ? $tmp[0] : "");
                                }
                            } elseif ((is_array($attr)) && (("$key" == "member") || ($key == 'memberof') || ($key == 'guacconfigparameter'))) {
                                foreach ($attr as $key3 => $gdn) {
                                    if ("$key3" != 'count') {
                                        $ret[$key1][$key][] = $gdn;
                                    }
                                }
                            } elseif ("$key" == "objectclass") {
                                foreach ($attr as $obj) {
                                    if (("$obj" == "user") || ("$obj" == "computer") || ("$obj" == "organizationalunit")) {
                                        $ret[$key] = $obj;
                                    }
                                }
                            } elseif (isset($attr[0])) {
                                // $ret[$key1][$key] = utf8_decode($attr[0]);
                                $ret[$key1][$key] = $attr[0];
                            }
                        }
                    }
                    if ($type == "machine") {
                        set_machine_attrs($config, $leases, $ret[$key1]);
                    }
                }
            }
            @ldap_free_result($result);
        } else {
            $ret = false;
        }
    } else {
        $ret = false;
        die($error);
    }
    @ldap_close($ds);
    // print "search_ad(" . $name . ", $type):";
    // var_dump($ret);
    return $ret;
}

/**
 * Modification d'un objet dans l'AD
 *
 * @Description Modifie un objet dans l'AD ou un des ses attributs : add / del / replace
 * @type
 * @Parametres array $config, string $name le nom de l'objet (cn), string $type le type d'objet à chercher : user, computer, group, classe, equipe, projet, parc, rights
 * array $attrs tableau associatif des attributs à changer (format ldapmodify), string $mode type d'opération add / del /replace
 * Pour del aussi, il faut fournir la bonne valeur de l'attribut pour que cela fonctionne. On peut ajouter, modifier, supprimer plusieurs attributs a la fois.
 * @returns  true ou false
 *
 */
function modify_ad(array $config, string $name, string $type, array $attrs, string $mode = "replace")
{
    $res = search_ad($config, $name, $type);
    // var_dump($res);
    if (count($res) == 1) {
        $dn = $res[0]['dn'];
        $ds = bind_ad_gssapi($config);
        if ($ds) {
            switch ($mode) {
                case "add":
                    $result = ldap_mod_add($ds, $dn, $attrs);
                    break;
                case "del":
                    $result = ldap_mod_del($ds, $dn, $attrs);
                    break;
                case "replace":
                    $result = ldap_mod_replace($ds, $dn, $attrs);
                    break;
            }
            @ldap_close($ds);
            return $result;
        }
    }
    return false;
}

/**
 * supprime un objet dans l'AD
 *
 * @type
 * @Parametres $name - le nom de l'objet (cn ou ou), $type - type d'objet recherché
 * @returns true ou false
 */
function delete_ad($config, $name, $type)
{
    $res = search_ad($config, $name, $type);
    if ($res) {
        $dn = $res[0]['dn'];

        $ds = bind_ad_gssapi($config);
        if ($ds) {
            $ret = ldap_delete($ds, $dn);
            @ldap_close($ds);
            return $ret;
        }
    }
    return false;
}

function clean_group($config, $group)
{
    $res = search_ad($config, $group, "group");
    if (strcasecmp($group, $res[0]['samaccountname'])) {
        $attrs = array();
        $attrs['samaccountname'] = $group;
        modify_ad($config, $group, "group", $attrs);
    }
}

/**
 * deplace ou renomme un objet dans l'AD
 *
 * @Parametres $name - le nom de l'objet (cn ou ou), $new_dn - nouveau dn complet
 * @type
 * @return true ou false
 */
function move_ad($config, $name, $new_dn, $type)
{
    $res = search_ad($config, $name, $type);
    if ($res) {
        $dn = $res[0]['dn'];
        $new_dn_elements = preg_split('/,/', $new_dn, 2);
        $ds = bind_ad_gssapi($config);
        if ($ds) {
            $ret = ldap_rename($ds, $dn, $new_dn_elements[0], $new_dn_elements[1], true);
            @ldap_close($ds);
            return $ret;
        }
    }
    return false;
}

function trash_user($config, $user)
{
    // $type = type_user($config, $user);
    // if (! empty($type)) {
    // $ou = "ou=" . $type . ",";
    // groupdelmember($config, $user, $type);
    // }
    if (is_array($user)) {
        foreach ($user['memberof'] as $group) {
            groupdelmember($config, $user['cn'], ldap_dn2cn($group));
        }
        $user = $user['cn'];
    }
    $ret = move_ad($config, $user, "cn=" . $user . "," . $config['dn']['trash'], "user");
    if ($ret) {
        $attrs = array();
        $attrs['useraccountcontrol'] = 514;
        modify_ad($config, $user, "user", $attrs);
    }
    return $ret;
}

function recup_user($config, $user, $cat = "")
{
    $categorie = ($cat == "") ? type_user($config, $user) : $cat;
    $ret = move_ad($config, $user, "cn=" . $user . ",ou=" . $categorie . "," . $config['dn']['people'], "user");
    if ($ret) {
        $attrs = array();
        $attrs['useraccountcontrol'] = 512;
        modify_ad($config, $user, "user", $attrs);
        groupaddmember($config, $user, $categorie);
    }
    return $ret;
}

function trash_users($config)
{
    // $users = search_ad($config, "*", "user", $config['dn']['people']);
    // recherche des comptes orphelins (diff entre les OU et les groupes principaux)
    $cns = array();
    $eleves = search_people_group($config, "Eleves");
    foreach ($eleves as $key => $value) {
        $cns[] = $value["cn"];
    }

    $profs = search_people_group($config, "Profs");
    foreach ($profs as $key => $value) {
        $cns[] = $value["cn"];
    }

    $administratifs = search_people_group($config, "Administratifs");
    foreach ($administratifs as $key => $value) {
        $cns[] = $value["cn"];
    }
    $users = array();
    $people = search_ad($config, "*", "user", $config['dn']['people']);
    foreach ($people as $key => $value) {
        if (! in_array($value["cn"], $cns))
            $users[] = $people[$key];
    }
    $ret = true;
    $users_no_trashables = search_ad($config, "*", "memberof", "no_Trash_user");
    $cns_noTrash = array();
    foreach ($users_no_trashables as $value) {
        $cns_noTrash[] = $value["cn"];
    }
    foreach ($users as $user) {
        $trash = true;
        // if (have_right($config, "no_trash_user", $user['cn'])) {
        // have_right consomme trop de resoources dans la boucle
        $trash = (! in_array($user['cn'], $cns_noTrash));
        // }
        // else {
        // foreach ($user['memberof'] as $groupdn) {
        // if (preg_match("/cn=(Classe_|Equipe_|Cours_|Administratifs|Domain Admins).*/i", $groupdn))
        // $trash = false;
        // }
        // }
        if ($trash) {
            print "trash : " . $user['cn'];
            $ret = trash_user($config, $user['cn']);
            if ($ret == 1) {
                echo "<font color='green'> SUCCES</font>" . "<br>\n";
            } else {
                echo "<font color='red'> ECHEC</font>" . "<br>\n";
            }
        }
    }
    return $ret;
}

/*
 * retourne le type d'un objet ad
 *
 */
function type_ad($config, $name)
{
    $res = search_ad($config, $name, "type");
    if ($res) {
        if (in_array("person", $res[0]))
            return "user";
        elseif (in_array("group", $res[0]))
            return "group";
        elseif (in_array("organizationalunit", $res[0]))
            return "ou";
    }
    return false;
}

/**
 * Recherche d'utilisateurs dans la branche people
 *
 * @Parametres $filter - Un filtre de recherche permettant l'extraction de l'annuaire des utilisateurs
 * @return Un tableau contenant les utilisateurs repondant au filtre de recherche ($filter)
 *        
 */
function filter_user($config, $filter)
{
    $ret = search_ad($config, $filter, "filter");

    // Tri du tableau par ordre alphabetique
    if (count($ret)) {
        usort($ret, "cmp_nom");
    }
    return $ret;
}

/**
 * Rechercher des groupes selon un critère
 *
 * @description  Recherche une liste de groupes repondants aux criteres fixes par la variable $filter. Les filtres sont les memes que pour ldapsearch.Par exemple (&(cnMember=wawa)(cnMember=toto)) recherche le groupe contenant les utilisateurs wawa et toto.
 * @type
 * @returns Retourne un tableau $groups avec le cn et la description de chaque groupe
 * @Parametres $config, string $filter - Un filtre de recherche permettant l'extraction de l'annuaire des utilisateurs, string $branch="default"
 *
 *
 *
 */
function filter_group($config, $filter, string $branch = "default")
{
    if ($branch == "default")
        $branch = $config['dn']['groups'];
    $groups = search_ad($config, "(&(objectclass=group)" . $filter . ")", "filter", $branch);
    if (count($groups))
        usort($groups, "cmp_cn");

    return $groups;
}

/**
 * Liste de tous les groupes autres
 *
 * @description  Recherche la liste des groupes autres et retourne un tableau.
 * @type Tableau
 * @returns Retourne un tableau trié $groups avec le cn et la description de chaque groupe
 * @Parametres $config la conf générale
 */
function filter_group_autres($config)
{
    $groups = search_ad($config, "*", "autre");
    if (count($groups))
        usort($groups, "cmp_cn");

    return $groups;
}

/**
 * Liste de tous les groupes projets
 *
 * @description  Recherche la liste des groupes projets et retourne un tableau.
 * @type Tableau
 * @returns Retourne un tableau trié $groups avec le cn et la description de chaque groupe
 * @Parametres $config la conf générale
 */
function filter_group_projets($config)
{
    $groups = search_ad($config, "*", "projet");
    if (count($groups))
        usort($groups, "cmp_cn");

    return $groups;
}

/**
 * Liste de tous les cours
 *
 * @description  Recherche la liste des cours et retourne un tableau.
 * @type Tableau
 * @returns Retourne un tableau trié $groups avec le cn et la description de chaque groupe
 * @Parametres $config la conf générale
 */
function filter_group_cours($config)
{
    $groups = search_ad($config, "*", "cours");
    if (count($groups))
        usort($groups, "cmp_cn");

    return $groups;
}

/**
 * Liste de toutes les matières
 *
 * @description  Recherche la liste des groupes matieres et retourne un tableau.
 * @type Tableau
 * @returns Retourne un tableau trié $groups avec le cn et la description de chaque groupe
 * @Parametres $config la conf générale
 */
function filter_group_matieres($config)
{
    $groups = search_ad($config, "*", "matiere");
    if (count($groups))
        usort($groups, "cmp_cn");

    return $groups;
}

/**
 * Liste de toutes les équipes
 *
 * @description  Recherche la liste des equipes et retourne un tableau.
 * @type Tableau
 * @returns Retourne un tableau trié $groups avec le cn et la description de chaque groupe
 * @Parametres $config la conf générale
 */
function filter_group_equipes($config)
{
    $groups = search_ad($config, "*", "equipe");
    if (count($groups))
        usort($groups, "cmp_cn");

    return $groups;
}

/**
 * Liste de toutes les classes
 *
 * @description  Recherche la liste des classes et retourne un tableau.
 * @type Tableau
 * @returns Retourne un tableau trié $groups avec le cn et la description de chaque groupe
 * @Parametres $config la conf générale
 */
function filter_group_classes($config)
{
    $groups = search_ad($config, "*", "classe");
    if (count($groups))
        usort($groups, "cmp_cn");

    return $groups;
}

/**
 * Retourne un tableau avec les attributs d'un groupe
 *
 * @type
 * @Parametres $cn du groupe
 *
 * @returns Un tableau contenant les informations du groupe $cn
 * ['cn'] : nom court
 * ['member']  : membres
 * ['memberof'] : droits de la branche rights
 * ['dn'] : dn complet
 */
function search_group($config, $cn)
{
    $ret = search_ad($config, $cn, "group", $config['dn']['groups']);
    // var_dump($ret);
    if (count($ret) > 0) {
        return $ret[0];
    } else
        return array();
}

/**
 *
 * @description Retourne un tableau avec les attributs d'un utilisateur (a partir de l'annuaire LDAP)
 * @type
 * @Parametres $cn de l'utilisateur
 * @returns Un tableau contenant les informations sur l'utilisateur (cn)
 *         les groupes sont dans le tableau $res['memberof']
 *
 */
function search_user($config, $cn, $branch = "all")
{
    $ret = search_ad($config, $cn, "user", $branch);
    if (count($ret) > 0) {
        return $ret[0];
    } else
        return array();
}

/**
 * Création d'un utilisateur
 *
 * @description Crée un utilisateur dans l'AD à l'aide des paramètres passés
 * @type
 * @Parametres array $config, string $cn, string $prenom, string $nom, string $userpwd, string $naissance, string $sexe, string $categorie, string $employeenumber
 * @returns True / False
 */
function create_user(array $config, string $cn, string $prenom, string $nom, string $userpwd, string $naissance, string $sexe, string $categorie, string $employeenumber = "")
{
    if (count(verif_employeenumber($config, $employeenumber)) == 0) {
        // Il faut determiner le login (attribut cn : use-username-as-cn) en fonction du nom prenom de l'uidpolicy...
        // Si $cn existe déja dans l'AD (doublon) il faut en fabriquer un autre
        if ($cn == "")
            $cn = creer_cn($config, $nom, $prenom);
        if (! recup_user($config, $cn))
            $res = useradd($config, $cn, $prenom, $nom, $userpwd, $naissance, $sexe, $categorie, $employeenumber);
        return $res;
    }
    return false;
}

/**
 * Recherche d'une machine
 *
 * @description Retourne un tableau avec les attributs d'un ordinateur (a partir de l'annuaire LDAP)
 * @type
 * @Parametres cn,dn ou displayname, recherche de l'ip depuis le dns ou le dhcp en option
 * @returns Un tableau contenant les informations sur la machine (cn ou dn ou dsiplayname)
 *         les dn groupes sont dans le tableau $res['memberof']
 *
 */
function search_machine(array $config, string $cn, bool $ip = false)
{
    if ($ip) {
        $type = "machine";
    } else {
        $type = "machine_fast";
    }
    $liste = search_ad($config, $cn, $type);
    $del = $ret = $m = array();
    // il peut y avoir des doublons de cn, par exemple dans le cas d'une intégration manuelle. On les efface (sauf 1 bien sur...)!
    foreach ($liste as $machine) {
        if ((count($liste) > 1) && (strtolower($machine['cn']) == ldap_hostname($machine)) && empty($machine['netbootmachinefilepath'])) {
            $del[] = $machine;
        } else {
            $ret[] = $machine;
        }
    }
    if (count($ret) > 0) {
        foreach ($del as $d) {
            delete_ad($config, $d['dn'], 'dn');
            dns_delete($config, $d, true);
        }
        if (count($ret) > 1) {
            foreach ($ret as $key => $r) {
                if (strcasecmp(ldap_dn2oudn($r['dn']), "cn=computers," . $config['ldap_base_dn']) == 0) {
                    delete_ad($config, $r['dn'], 'dn');
                    dns_delete($config, $r, true);
                    unset($ret[$key]);
                }
            }
        }
        $m = $ret[0];
        if (isset($m['attrs'])) {
            modify_ad($config, $m['cn'], "machine", $m['attrs'], "add");
        }
        dns_delete($config, $m, false);
    } elseif (count($del) > 0) {
        $m = $del[0];
        array_shift($del);
        foreach ($del as $d) {
            delete_ad($config, $d['dn'], 'dn');
            dns_delete($config, $m, true);
        }
    }
    return $m;
}

function set_machine_attrs($config, array $leases, array &$ret, $ip = true)
{
    putenv('RES_OPTIONS=retrans:1 retry:1 timeout:1 attempts:1');
    $ret['cn'] = strtolower($ret['cn']);
    if ($ip) {
        if (! empty($ret['iphostnumber']) && ! empty($ret['networkaddress'])) {
            $ret['state'] = "reservation";
        } else {
            $lease = get_dhcp_lease($config, $leases, $ret['cn']);
            if (! $lease && isset($ret['networkaddress']))
                $lease = get_dhcp_lease($config, $leases, $ret['networkaddress']);
            if ($lease) {
                if (! isset($ret['networkaddress'])) {
                    $ret['networkaddress'] = $lease['mac'];
                    $ret['attrs'] = [
                        'networkaddress' => $lease['mac']
                    ];
                }
                $ret['iphostnumber'] = $lease['ip'];
                $ret['state'] = $lease['state'];
            } elseif (isset($ret['dnshostname'])) {
                if ($ip = gethostbynamel($ret['dnshostname'])) {
                    $ret['iphostnumber'] = $ip[0];
                    $ret['state'] = "dns";
                } else {
                    $ret['state'] = "unknown";
                    $ret['iphostnumber'] = "";
                }
                // pas de dns, on tente le dhcp
            }
        }
    }

    return true;
}

/**
 * Ajout d'une machine
 *
 * @description Ajoute une machine dans l'AD si besoin, sinon renvoie true.
 * @type
 * @Parametres cn,dn ou displayname, recherche de l'ip depuis le dns ou le dhcp en option
 * @returns boolean
 *
 */
function create_machine($config, $name, $ou_rdn = "", $description = "reservation dhcp uniquement")
{
    if (count(search_ad($config, $name, "machine")) == 0) {
        // Prépare les données
        if ($ou_rdn == "")
            $ou_rdn = $config['computers_rdn'];
        $info = array();
        $info["cn"] = "$name";
        $info["objectclass"] = array(
            "top",
            "computer"
        );
        $info["samaccountname"] = $name . "$";
        // $info["samaccounttype"] = 0x30000001;
        $info["description"] = $description;
        $info["useraccountcontrol"] = 0x1000;

        // Ajout
        $ds = bind_ad_gssapi($config);
        $ret = ldap_add($ds, "cn=" . $name . "," . $ou_rdn . "," . $config['ldap_base_dn'], $info);
        // $err = ldap_error($ds);
        ldap_close($ds);

        // ajout dans les parcs hérités de la salle
        if ($ret) {
            $parcs = explode(",", ldap_dn2ufn($ou_rdn));
            array_pop($parcs);
            foreach ($parcs as $parc) {
                groupaddmember($config, $name, $parc);
            }
        }
        return $ret;
    } else
        return true;
}

/**
 * Recherche catégorie utilisateur
 *
 * @description retourne le groupe principal d'un utilisateur
 * @type
 * @Parametres array $config, string $user
 * @returns string
 */
function type_user(array $config, string $user)
{
    $res = search_user($config, $user);
    if (count($res) > 0) {
        $match = array();
        $groupsdn = (isset($res['memberof'])) ? $res['memberof'] : array();
        foreach ($groupsdn as $groupdn) {
            if (preg_match("/cn=(Eleves|Profs|Administratifs).*/i", $groupdn, $match)) {
                return $match[1];
            }
        }
        if (preg_match("/ou=(Eleves|Profs|Administratifs).*/i", $res['dn'], $match)) {
            return $match[1];
        }
    }
    return "";
}

/**
 *
 * Vérification droit ldap pour utilisateur
 *
 * @description Recherche si l'utilisateur courant a au moins un des droits type1|type2..
 * vrai dans tous les cas si l'utilisateur a le droit se3_is_admin
 * @type
 * @Parametres right
 * @returns true ou false
 */
function have_right($config, $type, $user = "login")
{
    if ($user == "login")
        $user = $config['login'];
    $rights = list_rights($config, $user);
    $typearr = explode("|", "$type");
    $typearr[] = "se3_is_admin";
    foreach ($typearr as $right) {
        if (in_array($right, $rights)) {
            return true;
        }
    }
    return false;
}

/**
 * Lister les droits d'un utilisateur
 *
 * @description retourne tous les droits dont dispose $name (user ou groupe)
 * recherche recursive pour les groupes d'appartenance
 * @type
 * @Parametres : $config, $name
 * @Parametres : name - utilisateur ou groupe
 * @returns array des droits
 */
function list_rights($config, $name)
{
    $rights = array();
    if ("$name" == "all") {
        $res = search_ad($config, "*", "right");
        foreach ($res as $right) {
            $rights[] = $right['cn'];
        }
        return $rights;
    }
    $user = search_user($config, $name);
    if (count($user) == 0)
        $user = search_group($config, $name);
    $ret = search_ad($config, $name, "right");
    $groups = (isset($user['memberof'])) ? $user['memberof'] : array();
    // foreach ($groups as $groupdn) {
    $ret1 = search_ad($config, $groups, "right");
    $ret = array_merge($ret1, $ret);
    if ($ret) {
        foreach ($ret as $group) {
            $right = $group['cn'];
            if (! in_array($right, $rights)) {
                $rights[] = $right;
            }
        }
    }

    // if ($inverse) {
    // $not_rights = array();
    // foreach (list_rights($config, "all") as $r) {
    // if (! in_array($r, $rights))
    // $not_rights[] = $r;
    // }
    // return $not_rights;
    // }
    // }
    if (count($rights) == 0) {
        return array();
    } else {
        return $rights;
    }
    // // si user n'est membre d'aucun groupe Rights
    // if ($inverse) {
    // return list_rights($config, "all");
    // } else {
    // return array();
    // }
}

/**
 * Lister les membres d'un groupe
 *
 * @description retourne tous les membres d'un groupe donné
 * @type
 * @Parametres : config, $group
 * @returns array des membres
 */
function list_members_group($config, string $group)
{
    $res = search_people_group($config, $group);
    return ($res);
}

/**
 * Lister les membres disposant d'un droit
 *
 * @description retourne tous les membres disposant d'un droit donné
 * recherche recursive pour les groupes d'appartenance
 * @type
 * @Parametres : config, $right
 * @returns array des membres
 */
function list_members_rights($config, $right)
{
    // $res = grouplistmembers($config, $right);
    $groupe = search_ad($config, $right, "group");
    $res = (isset($groupe[0]['member'])) ? $groupe[0]['member'] : array();
    return $res;
}

/**
 * Délégue un droit ldap
 *
 * @description donne le droit à l'utilisateur ou a un groupe
 * @type
 * si il l'a dejà par héritage d'un groupe on ne fait rien
 * @Parametres array $config, string $name, string $right
 * @returns true ou false
 */
function add_right($config, $name, $right)
{
    $arr = array(
        "$name"
    );
    $add = array();
    $add['member'] = $arr[0];
    // return groupaddmember($config, $name, $right);
    return modify_ad($config, $right, "rights", $add, "add");
    return true;
}

/**
 * Supprime un droit à l'utilisateur
 *
 * @description non récursif : si le droit est hérité il ne sera pas retiré.
 * @type
 * @parametres array $config, string $name, string $right
 * @returns true ou false
 */
function remove_right($config, $name, $right)
{
    $arr = array(
        "$name"
    );
    $del = array();
    $del['member'] = $arr[0];
    // var_dump($del);
    return modify_ad($config, $right, "rights", $del, "del");
}

// Parcs
// Un parc est composé d'un groupe (pour wpkg et les délégations), et éventuellement d'une OU pour les GPO
// Les OU peuvent être hierarchisées, dans ce cas les groupes parents contiennent aussi les machines des OU enfants

/**
 * recherche les attributs d'un parc (OU + groupe) et les combine
 *
 * @description indique s'il s'agit d'un parc ou d'une salle
 * @type
 * @Parametres array $config, string $search (* pour tous)
 * @returns string salle ou parc
 */
function search_parcs(array $config, string $search, string $type = "all")
{
    $parcs = $salles = array();
    if (($type == "all") || ($type == "salle"))
        $salles = search_ad($config, $search, "salle");
    if (! is_array($salles)) {
        return false;
    }
    if (($type == "all") || ($type == "parc"))
        $parcs = search_ad($config, $search, "parc");
    if (! is_array($parcs)) {
        return false;
    }
    if (count($salles) + count($parcs) == 0)
        return array();
    else {
        $res = array();
        $i = - 1;
        if (count($salles) > 0) {
            foreach ($salles as $salle) {
                $found = false;
                if (count($res) > 0) {
                    foreach ($res as $re) {
                        if (strtolower($re['name']) == strtolower($salle['ou'])) {
                            $found = true;
                            break;
                        }
                    }
                }

                if (! $found) {
                    $i ++;
                    $res[$i]['name'] = $salle['ou'];
                    $res[$i]['parent'] = ldap_dn2cn(ldap_dn2oudn($salle['dn']));
                    $res[$i]['type'] = "salle";
                    $res[$i]['dn'] = $salle['dn'];
                    $res[$i]['uuid'] = to_guid($salle['objectguid']);
                    if (isset($res[$i]['description']))
                        $res[$i]['description'] = $salle['description'];
                }
            }
        }
        if (count($parcs) > 0) {
            foreach ($parcs as $parc) {
                if ($parc['cn'] != $parc['samaccountname']) {
                    $attrs = array();
                    $attrs['samaccountname'] = $parc['cn'];
                    modify_ad($config, $parc['cn'], "parc", $attrs);
                }
                $found = false;
                if (count($res) > 0) {
                    foreach ($res as $key => $re) {
                        if (strtolower($re['name']) == strtolower($parc['cn'])) {
                            $found = true;
                            break;
                        }
                    }
                }
                if (! $found) {
                    $i ++;
                    $res[$i]['name'] = $parc['cn'];
                    $res[$i]['type'] = "parc";
                    $res[$i]['dn'] = $parc['dn'];
                    $res[$i]['description'] = $parc['description'] ?? "";
                    $res[$i]['uuid'] = to_guid($parc['objectguid']);
                } else {
                    $res[$key]['description'] = $res[$i]['description'] ?? $parc['description'] ?? "";
                }
            }
        }
        sort_by_key("name", $res);
        return $res;
    }
}

function type_parc(array $config, string $parc)
{
    $parcs = search_parcs($config, $parc);
    return $parcs[0]['type'];
}

/**
 * recherche les parcs pour une machine
 * une recherche "salle" ne renvoie que l'OU contenant la machine
 * une recherche "parc" renvoie tous les parcs
 *
 * @type
 * @parametres : machine a rechercher"
 * @parametres : type - "salle", "all" (defaut)"
 * @returns : array(array parcs))
 */
function list_parcs($config, $machine, $type = "all")
{
    $parcs = array();
    if ($machine != "*") {
        if (count($info = search_machine($config, $machine, false)) > 0) {
            $salle = ldap_dn2cn(ldap_dn2oudn($info['dn']));
            $parcs = search_parcs($config, $salle);
            $salle = $parcs;
            if ($type == "salle")
                return $salle[0];
            foreach ($info['memberof'] as $parc) {
                $p = search_parcs($config, ldap_dn2cn($parc));
                if ($salle != $p) {
                    if (! preg_match("/" . $p[0]['dn'] . "/", $info['dn'])) {
                        $p = search_parcs($config, ldap_dn2cn($parc), "parc");
                    }
                    $parcs[] = $p[0];
                }
            }
        }
    }
    // sort_by_key("name", $parcs);
    return $parcs;
}

/**
 * recherche les parcs pour une machine
 * une recherche "salle" ne renvoie que l'OU contenant la machine
 * une recherche "parc" renvoie tous les parcs
 *
 * @type
 * @parametres : machine a rechercher"
 * @parametres : type - "salle", "all" (defaut)"
 * @returns : array(array parcs))
 */
function list_machines_parcs($config)
{
    $machines = search_ad($config, "*", "machine_fast", $config['dn']['computers']);
    if (is_array($machines)) {
        $ret = [];
        foreach ($machines as $machine) {
            $parc = [];
            if (isset($machine['memberof'])) {
                foreach ($machine['memberof'] as $parc_dn) {
                    $parc[] = ldap_dn2cn_case($parc_dn);
                }
            }
            $dn = ldap_dn2oudn($machine['dn']);
            while (strcasecmp($dn, $config['dn']['computers']) != 0) {
                $ou = ldap_dn2cn_case($dn);
                if (! in_array($ou, $parc)) {
                    $parc[] = $ou;
                }
                $dn = ldap_dn2oudn($dn);
            }
            $ret[$machine['cn']] = $parc;
        }
    } else {
        $ret = false;
    }
    return $ret;
}

/**
 * creation de parc : crée un groupe, et éventuellement une ou si le type est "salle"
 * le type "salle" permet d'appliquer des GPO, le type "parc" sert pour les delegations et wpkg.
 * Les salles peuvent être imbriquées ( parentou ), afin de gérer des GPO de façon globale.
 * Dans tous les cas un groupe du même nom est créé
 *
 * @parametres
 * @type
 * @returns
 */
function create_parc(array $config, string $parc, string $description = "", string $type = "salle", string $parentou = "")
{
    $res = groupadd($config, $parc, $config['parcs_rdn'], $description);
    if ($type == "salle") {
        if (($parentou == "") || (strtolower($parentou) == "computers"))
            $ou = $config['computers_rdn'];
        else
            $ou = "ou=" . $parentou . "," . $config['computers_rdn'];
        $res = ouadd($config, $parc, $ou);
    }
    return $res;
}

/**
 * suppression de parc
 *
 * @description Si il s'agit d'une salle il faut commencer par deplacer les machines vers l'ou parent, Normalement
 * @parametres
 * @type
 * @returns
 */
function delete_parc($config, $parc)
{
    $info = search_parcs($config, $parc);
    if ($info[0]['type'] == "salle") {
        $machines = list_members_parc($config, $parc, true);
        foreach ($machines as $machine) {
            move_member_parc($config, $info[0]['parent'], $machine['cn']);
        }
        delete_ad($config, $parc, "salle");
    }
    $ret = delete_ad($config, $parc, "parc");
    if ($ret) {
        $res = search_delegations($config, $parc);
        if ($res) {
            foreach ($res as $delegation) {
                groupdel($config, $delegation['cn']);
            }
        }
    }
    return $ret;
}

/**
 *
 * @Description deplace une salle
 * Cela implique de sortir les machines
 * de tous les parcs anciens parents, et de les remettre dans tous les nouveaux parents
 *
 * @param array $config
 * @param string $newparc
 * @param string $member
 * @return true
 */
function move_parc(array $config, string $new_parent, string $parc)
{
    $parc = search_parcs($config, $parc, "salle")[0];
    $old_parents = explode(",", ldap_dn2oudn($parc['dn']));
    $old_parcs = array();
    foreach ($old_parents as $old) {
        if (strcasecmp(ldap_dn2cn($old), ldap_dn2cn($config['computers_rdn'])) == 0) {
            break;
        } else {
            $old_parcs[] = ldap_dn2cn($old);
        }
    }
    $new_parent = search_parcs($config, $new_parent, "salle")[0];
    $new_parcs = array();
    if (count($new_parent) > 0) {
        $new_parents = explode(",", $new_parent['dn']);
        foreach ($new_parents as $new) {
            if (strcasecmp(ldap_dn2cn($new), ldap_dn2cn($config['computers_rdn'])) == 0) {
                break;
            } else {
                $new_parcs[] = ldap_dn2cn($new);
            }
        }
    }
    $remove = array_diff($old_parcs, $new_parcs);
    $add = array_diff($new_parcs, $old_parcs);
    $machines = list_members_parc($config, $parc['name'], false, true);
    $res = move_ad($config, $parc['name'], "cn=" . $parc['name'] . "," . $new_parent['dn'], "salle");
    if ($res) {
        foreach ($machines as $machine) {
            foreach ($remove as $r) {
                groupdelmember($config, $machine . "$", $r);
            }
            foreach ($add as $a) {
                groupaddmember($config, $machine . "$", $a);
            }
        }
    }
    return $res;
}

/**
 *
 * @Description renomme un parc
 * Cela implique de :
 *  - récuperer les applis wpkg si besoin,
 *  - renommer le cn et le ou si salle
 *  - synchroniser la base wpkg
 *  - reinjecter les applis wpkg
 *
 * @param array $config
 * @param string $newparc
 * @param string $member
 * @return true
 */
function rename_parc(array $config, string $new_name, string $parc)
{
    $info = search_parcs($config, $parc)[0];
    if ($info['type'] == "salle") {
        move_ad($config, $info['dn'], "ou=" . $new_name, ldap_dn2oudn($info));
    }
    $res = move_ad($config, "cn=" . $parc . "," . $config['parcs_dn'], "cn=" . $new_name, $config['parcs_dn']);
    return $res;
}

/**
 * liste le contenu d'un parc
 *
 * @description
 * @parametres : parc à lister
 * @parametre : attrs - booleen enregistrements complets ou liste
 * @returns : liste des machines, enregistrements complets des machines, false
 */
function list_members_parc($config, $parc, $attrs = false, $recurse = false, $type = "all")
{
    if ($recurse) {
        $scope = "subtree";
    } else {
        $scope = "base";
    }
    $ret = array();

    if ($type == "all" || $type == "salle") {
        $info = search_parcs($config, $parc);
        if (count($info) > 0) {
            if ($info[0]['type'] == "salle") {
                $ret = search_ad($config, "*", "machine", $info[0]['dn'], array(), $scope);
                if (! $attrs)
                    $ret = array_column($ret, "cn");
            }

            /*
             * if ($attrs) {
             * $ret = $res;
             * } else {
             * foreach ($res as $machine) {
             * $ret[] = $machine['cn'];
             * }
             * }
             */
        }
    }
    // elseif ($info[0]['type'] = "parc") {

    $machines = search_ad($config, "*", "machine");
    $res = search_ad($config, $parc, "parc");
    if (isset($res[0]['member'])) {
        if ($attrs) {
            foreach ($ret as $ms) {
                foreach ($res[0]['member'] as $key => $m) {
                    if ($m == $ms['dn']) {
                        unset($res[0]['member'][$key]);
                        break;
                    }
                }
            }
            foreach ($machines as $machine) {

                foreach ($res[0]['member'] as $m) {
                    if ($machine['dn'] == $m) {
                        $ret[] = $machine;
                        break;
                    }
                }
            }
        } else {
            foreach ($res[0]['member'] as $machine) {
                $cn = ldap_dn2cn($machine);

                $ret[] = $cn;
                $ret = array_unique($ret);
            }
        }
    }

    if ($attrs) {
        // $ret = array_unique(array_merge($ret, search_ad($config, $parc['dn'], "machine")));
        sort_by_key("cn", $ret);
    } else {
        // $ret = array_unique(array_merge($ret, array_column(search_ad($config, $parc['dn'], "machine"), "cn")));
        sort($ret);
    }
    return $ret;
}

/**
 * liste le contenu d'un parc
 *
 * @description
 * @parametres : parc à lister
 * @parametre : attrs - booleen enregistrements complets ou liste
 * @returns : liste des machines, enregistrements complets des machines, false
 */
function list_members_parc_fast($config, $parc, $attrs = false)
{
    $ret = array();
    if ($attrs) {
        $ret = search_ad($config, $parc, "machine_parc");
        sort_by_key("cn", $ret);
    } else {
        $ret = array_column(search_ad($config, $parc, "machine_parc"), "cn");
        sort($ret);
    }
    return $ret;
}

/**
 *
 * @Description deplace un membre d'un  parc
 * pour les salles il faut déplacer les machines depuis leur ou d'origine. Cela implique de les sortir aussi du groupe associé.
 * Pour les parcs on ajoute juste au groupe.
 *
 * @param array $config
 * @param string $newparc
 * @param string $member
 * @return true
 */
function move_member_parc(array $config, string $newparc, string $member)
{
    $salle = list_parcs($config, $member, "salle");
    $newparcs = search_parcs($config, $newparc);
    if ($newparcs[0]['type'] == "salle" || strcasecmp($newparc, ldap_dn2cn($config['computers_rdn'])) == 0) {
        $newsalle = $newparcs[0];
        // on retire la machine de tous les anciens parcs
        while (strcasecmp($salle['name'], ldap_dn2cn($config['computers_rdn']))) {
            groupdelmember($config, $member . "$", $salle['name']);
            $parentparc = search_parcs($config, $salle['parent']);
            $salle = $parentparc[0];
        }
        // on ajoute la machine à tous les parcs parents
        while (strcasecmp($newsalle['name'], ldap_dn2cn($config['computers_rdn']))) {
            groupaddmember($config, $member . "$", $newsalle['name']);
            $newparentparc = search_parcs($config, $newsalle['parent']);
            $newsalle = $newparentparc[0];
        }
        return move_ad($config, $member, "cn=" . $member . "," . $newparcs[0]['dn'], "machine");
    } else
        return groupaddmember($config, $member . "$", $newsalle['name']);
}

/**
 *
 * @Description ajoute un membre au parc
 * pour les salles il faut déplacer les machines depuis leur ou d'origine. Cela implique de les sortir aussi du groupe associé.
 * Pour les parcs on ajoute juste au groupe.
 * @type
 * @returns true ou false
 */
function add_member_parc($config, $parc, $member)
{
    return groupaddmember($config, $member . "$", $parc);
}

/**
 *
 * @Description enlève un membre du parc
 * @parametres
 * @type
 * @returns true ou false
 */
function remove_member_parc($config, $parc, $member)
{
    $parcs = search_parcs($config, $parc, "salle");
    $machine = search_machine($config, $member);

    if (type_parc($config, $parc) == "salle" && ldap_dn2oudn($machine['dn']) == $parcs[0]['dn']) {
        return move_member_parc($config, $parcs[0]['parent'], $member);
    } else {
        return groupdelmember($config, $member . "$", $parc);
    }
}

/**
 *
 * @Description teste l'appartenance au parc
 * @parametres $config, $parc, $member
 * @type
 * @returns true ou false
 */
function is_member_parc($config, $parc, $member)
{
    $res = list_parcs($config, $member, "all");
    if ($res) {
        foreach ($res as $value) {
            if (strtolower($value['name']) == strtolower($parc))
                return true;
        }
        return false;
    }
}

// Delegations -------------------------------------------------------------------
/**
 *
 * @Description teste si l'utilisateur (loggé) a les droits sur le parc
 * @parametres $config, $parc, $right
 * @type
 * @returns true ou false
 */
function have_delegation($config, $parc, $right, $name = "login")
{
    if (type_delegation($config, $parc, $name) == $right)
        return true;
    return false;
}

/**
 *
 * @Description recherche les parcs délégués à l'utilisateur ou groupe, recursive sur les groupes d'appartenance
 * @parametres config, $name = "login", $recurse = true
 * @type
 * @returns array(user, parc, level) ou false
 */
function list_delegations($config, $name = "login", $recurse = true, $level = "")
{
    $parc = array();
    if ($name == "login")
        $name = $config['login'] ?? "";
    if (! empty($name)) {
        $user = search_user($config, $name);
        if (empty($user)) {
            $user = search_group($config, $name);
        }
        $cn = $user['cn'] ?? "";
        if (! empty($cn)) {
            if ($recurse) {
                $ret = list_delegations($config, $name, false, $level);
                $user = search_user($config, $name);
                foreach ($user['memberof'] as $groupdn) {
                    $ret1 = list_delegations($config, ldap_dn2cn($groupdn), false, $level);
                    $ret = array_merge($ret1, $ret);
                }
                return $ret;
            } else {

                $res = search_ad($config, $name, "delegation");
                if ($res) {
                    foreach ($res as $key => $group) {
                        $delegation = explode('_', $group['cn'], 2);
                        if ((! empty($level) && $delegation[0] == $level) || empty($level)) {
                            $parc[$key]['user'] = $cn;
                            $parc[$key]['cn'] = $delegation[1];
                            $parc[$key]['level'] = $delegation[0];
                        }
                    }
                    return $parc;
                }
            }
        }
    }
    return array();
}

/**
 * niveau de Délégation de parc pour un user
 *
 * @Description retourne le niveau de délégation de l'utlisateur courant pour une machine ou un parc
 * @parametre : $config, $name nom du parc ou de la machine, $user = "login"
 * @returns : string "manage", "view", "rdp", false
 */
function type_delegation($config, $name, $user = "login")
{
    $parcs = list_delegations($config, $user);
    $ret = false;
    if ($parcs) {
        foreach ($parcs as $parc) {
            if ($parc['cn'] == $name)
                return $parc['level'];
            if (is_member_parc($config, $parc['cn'], $name)) {
                if ($parc['level'] == "manage") {
                    return "manage";
                } elseif ($parc['level'] == "rdp") {
                    return "rdp";
                } elseif ($parc['level'] == "vnc") {
                    return "vnc";
                } else {
                    $ret = "view";
                }
            }
        }
    }
    return $ret;
}

/**
 * delegations en cours pour un parc
 *
 * @Description cherche les delegations pour un parc
 * @parametres : array $config, string $parc
 * @returns : array
 *
 */
function search_delegations(array $config, string $parc = "")
{
    if (empty($parc)) {
        $filter = "*";
    } else {
        $filter = "*_" . $parc;
    }
    $res = search_ad($config, $filter, "group", $config['dn']['delegations']);
    foreach ($res as $key => $delegation) {
        $del = explode("_", $delegation['cn'], 2);
        $res[$key]['level'] = $del[0];
        $res[$key]['parc'] = $del[1];
    }
    return $res;
}

/**
 * Mise en place délégation
 *
 * @Description met en place une delegation pour un parc
 * @parametres : $config, $parc, $name, $level)
 * @returns : array
 *
 */
function create_delegation($config, $parc, $name, $level)
{
    $delegation = $level . "_" . $parc;
    $res = search_ad($config, $delegation, "group", $config['dn']['delegations']);
    $ret = true;
    if (! $res) {
        $ret = groupadd($config, $delegation, $config['delegations_rdn'], "délégation du droit" . $level);
    }
    if ($ret)
        $ret = groupaddmember($config, $name, $delegation);
    return $ret;
}

/**
 *
 * @description supprime une delegation pour un utilisateur ou groupe
 *
 */
function delete_delegation($config, $parc, $name, $level)
{
    $delegation = $level . "_" . $parc;
    $res = search_ad($config, $delegation, "group", $config['dn']['delegations']);
    $ret = groupdelmember($config, $name, $delegation);
    if (count($res[0]['member']) == 0) {
        $ret = groupdel($config, $delegation);
    }
    return $ret;
}

// ----------------------------------------------------
// dhcp
/**
 * retourne la reservation, false dans les autres cas
 *
 * @param unknown $config
 * @param unknown $machine
 *            nom, ip ou mac
 * @return unknown[]|boolean
 */
function get_dhcp_reservation($config, $machine)
{
    $res = search_ad($config, $machine, "machine");
    if (isset($res[0]['iphostnumber']) && isset($res[0]['networkaddress']) && $res[0]['state'] == "reservation") { // != "dns" && $res[0]['state'] != "reservation") {
        return array(
            'cn' => $res[0]['cn'],
            'iphostnumber' => $res[0]['iphostnumber'],
            'networkaddress' => $res[0]['networkaddress']
            // 'state' => $res[0]['state']
        );
    } else {
        return false;
    }
}

/**
 * renvoie l'ip si elle est libre ou une ip fixe libre dans le meme vlan
 *
 * @Parametres $ip : Adresse IP a tester
 *
 * @return adresse ip libre
 *        
 */
function get_free_ip($config, $ip)
{
    $reseau = get_vlan($config, $ip);
    $calcul_ip = floatval(sprintf("%u", ip2long($ip)));
    $ipmin = floatval(sprintf("%u", $reseau['ip_min']));
    $ipmax = floatval(sprintf("%u", $reseau['ip_max']));
    $gateway = floatval(sprintf("%u", $reseau['gateway']));
    $begin_range = floatval(sprintf("%u", $reseau['begin_range']));
    $end_range = floatval(sprintf("%u", $reseau['end_range']));

    if ((isset(get_dhcp_reservation($config, $ip)['iphostnumber'])) or ($ip == long2ip($reseau['gateway'])) or ($calcul_ip < $ipmin) or (($calcul_ip >= $begin_range) and ($calcul_ip <= $end_range))) {
        $calcul_free = $reseau['ip_min'];
        while (floatval(sprintf("%u", $calcul_free)) <= $ipmax) {
            if ((fping(long2ip($calcul_free)) == 1) or (isset(get_dhcp_reservation($config, long2ip($calcul_free))['iphostnumber']) or ($calcul_free == $reseau['gateway']))) {
                // echo long2ip($calcul_free) . " prise<br>";
                $calcul_free ++;
            } elseif ($calcul_free == $reseau['begin_range']) {
                $calcul_free = $reseau['end_range'] + 1;
            } else {
                print "l'adresse <b>" . long2ip($calcul_free) . "</b> est disponible<br>";
                return long2ip($calcul_free);
            }
        }
        return false;
    } else {
        echo "l'adresse <b>" . $ip . " </b>saisie est libre<br>";
        return $ip;
    }
}

/**
 *
 * @description enregistre une reservation dhcp dans AD
 *  l'ip est basée sur le bail actuel de la machine
 *  la machine doit déjà avoir un enregistrement ad
 *  si une ip est passée on tente de la reserver.
 * @parametres : $config, $cn
 * @returns : true si ok
 */
function set_dhcp_reservation($config, $cn, $ip = "", $mac = "")
{
    $mode = false;
    // search_machine a besoin de l'argument N3 true pour fournir l'index state
    $machine = search_machine($config, $cn, true);
    $state = $machine['state'] ?? "";
    // if (! empty($machine['networkaddress']) && (! empty($machine['iphostnumber']) || $ip)) {
    if ($state == "reservation") {
        // if (! empty($ip) && $ip != $machine['iphostnumber']) {
        if (! empty($ip) && $ip != $machine['iphostnumber'] || ! empty($ip) && $ip == $machine['iphostnumber']) {
            $mode = "replace";
        }
    } elseif ((! empty($mac) || ! empty($machine['networkaddress'])) && (! empty($machine['iphostnumber']) || ! empty($ip))) {
        $reservation = get_dhcp_reservation($config, $machine['networkaddress'] ?? $mac);
        if (isset($reservation['iphostnumber'])) {
            if ($machine['iphostnumber'] != $reservation['iphostnumber']) {

                // un enregistrement existe sous un autre nom pour cette ip
                // @TODO DANGER !!!
                // on ne supprime les machines que si elle ont une reservation identique
                $old = search_machine($config, $reservation['iphostnumber']);
                if (count($old) > 0) {
                    if ($old['cn'] != $machine['cn'] && @$old['state'] == 'reservation') {
                        delete_ad($config, $old['cn'], 'machine');
                        $mode = "add";
                    }
                }
            }
        }
        $mode = "add";
    }
    if (empty($ip)) {
        $ip = $machine['iphostnumber'] ?? "";
    }
    $newip = get_free_ip($config, $ip);

    $attrs = array(
        'networkaddress' => $machine['networkaddress'] ?? $mac
    );
    modify_ad($config, $machine['cn'], "machine", $attrs, $mode);
    unset($attrs);
    $attrs = array(
        'iphostnumber' => $newip
    );
    if (! $mode) {
        return true;
    } else {
        return modify_ad($config, $machine['cn'], "machine", $attrs, $mode);
        // @TODO
        // ajouter la mise à jour du reservations.inc et restart dhcpd
    }
}

/**
 * enregistre une reservation dhcp dans AD
 *
 * @parametres : $machine
 * @parametres : $reservation - tableau {ip, mac}
 * @returns : true si ok
 */
function delete_dhcp_reservation($config, $machine)
{
    $reservation = get_dhcp_reservation($config, $machine);
    if (isset($reservation['iphostnumber'])) {
        $del['iphostnumber'] = $reservation['iphostnumber'];
        return modify_ad($config, $machine, "machine", $del, "del");
    } else {
        return false;
    }
}

function delete_all_dhcp_reservations($config)
{
    foreach (search_ad($config, "*", "machine") as $machine) {
        if (isset($machine['iphostnumber']) && $machine['state'] == "reservation") {
            modify_ad($config, $machine['cn'], "machine", [
                'iphostnumber' => $machine['iphostnumber']
            ], "del");
        }
    }
}

/**
 * Importe le fichier des reservations /etc/sambaedu/reservations.inc vers l'AD
 *
 * @returns : array machine, ip, mac
 */
function import_dhcp_reservations($config)
{
    if (file_exists("/etc/sambaedu/reservations.inc.se3")) {
        $contents = file_get_contents("/etc/sambaedu/reservations.inc.se3");
        $contents = explode("\n", $contents);
        $index = 0;
        $data = array();
        $m = array();
        $html = "";
        $record = false;
        foreach ($contents as $line) {
            if (preg_match("/^\s*(|#.*)$/", $line, $m)) {
                // on saute les commentaires
            } elseif (preg_match("/^host (.*)$/", $line, $m) && (! $record)) {
                $index ++;
                $data[$index]['cn'] = strtolower($m[1]);
            } elseif (preg_match("/^{/", $line)) {
                $record = true;
            } elseif (preg_match("/^\s*hardware ethernet\s*([0-9a-fA-F:]*)\s*;$/", $line, $m)) {
                $data[$index]['networkaddress'] = strtolower($m[1]);
            } elseif (preg_match("/^\s*fixed-address\s*([0-9\.]*)\s*;$/", $line, $m)) {
                $data[$index]['iphostnumber'] = $m[1];
            } elseif (preg_match("/}*/", $line, $m)) {
                $record = false;
            } else {
                $html .= "Erreur ligne '$line'\n";
                break;
            }
        }
        foreach ($data as $reservation) {
            if (! search_ad($config, $host['cn'], "machine")) {
                create_machine($config, $reservation['cn'], $config['equipements_rdn']);
            }
            $res = set_dhcp_reservation($config, $reservation['cn'], $reservation['iphostnumber'], $reservation['networkaddress']);
            if ($res) {
                $html .= "machine " . $reservation['cn'] . ": ip " . $reservation['iphostnumber'] . " OK\n";
            } else {
                $html .= "machine " . $reservation['cn'] . ": ip " . $reservation['iphostnumber'] . " ERREUR\n";
            }
        }
    }
    return $html;
}

function export_dhcp_reservations($config)
{
    $reservations = "/etc/sambaedu/reservations.inc";
    $content = "# reservations exportees automatiquement de l'annuaire AD\n";
    $content .= "# le " . date(DATE_RSS) . "\n";
    $machines = search_ad($config, "*", "machine_fast", "all");
    foreach ($machines as $machine) {
        if (! empty($machine['networkaddress']) && ! valid_mac($machine['networkaddress'])) {
            delete_dhcp_reservation($config, $machine['cn']);
            unset($machine['networkaddress']);
        }
        $res = (! empty($machine['networkaddress']) && ! empty($machine['iphostnumber'])); // && preg_match("/^(25[0–5]|2[0–4][0–9]|[01]?[0–9][0–9]?).(25[0–5]|2[0–4][0–9]|[01]?[0–9][0–9]?).(25[0–5]|2[0–4][0–9]|[01]?[0–9][0–9]?).(25[0–5]|2[0–4][0–9]|[01]?[0–9][0–9]?)$/", $machine['iphostnumber'])));
        if ($res) {
            $content .= "host " . preg_replace("/ /", "-", $machine['cn']) . "\n";
            $content .= "{\n";
            $content .= "hardware ethernet " . $machine['networkaddress'] . ";\n";
            $content .= "fixed-address " . $machine['iphostnumber'] . ";\n";
            $content .= "}\n";
        }
    }
    if (! $handle = fopen($reservations, "w")) {
        die("Erreur d'ecriture des reservtions se4 : $reservations");
    }
    $res = fwrite($handle, $content);
    fclose($handle);
    return true;
}

/**
 * importe les leases dhcp.
 *
 * @Description Seul le dernier enregistrement actif d'une ip est conservé.
 * @type
 * @parametres array $config
 * @returns array(array(ip, name, mac, state))
 */
function import_dhcp_leases($config)
{
    // $ret = apcu_fetch('dhcp_leases');
    if (empty($ret)) {
        $contents = file_get_contents("/var/lib/dhcp/dhcpd.leases");
        $contents = explode("\n", $contents);
        $current = 0;
        $data = $m = array();
        foreach ($contents as $line) {
            switch ($current) {
                case 0:
                    if (preg_match("/^\s*(|autho.*|#.*)$/", $line, $m)) {} else if (preg_match("/^lease (.*) {/", $line, $m)) {
                        $current = $m[1];
                    } else if (preg_match("/^server-duid/", $line)) {
                        // ignore
                    } else {
                        print "Failed parsing '$line'\n";
                    }
                    break;
                default:
                    if (preg_match('/^\s*(binding state|hardware ethernet|client-hostname|uid|set vendor-class-identifier =) "?(.*?)"?;$/', $line, $m)) {
                        $data[$current][$m[1]] = $m[2];
                    } elseif (preg_match("/}/", $line, $m)) {
                        $current = 0;
                    } else {
                        // print "Failed parsing '$line'\n";
                    }
            }
        }
        $ret = array();
        foreach ($data as $ip => $d) {
            if ((($d['binding state'] == "active") || ($d['binding state'] == "free")) && ((isset($d['hardware ethernet']) && isset($d['client-hostname']) && (! empty($d['client-hostname']))) || (isset($d['uid'])))) {
                $ret[] = array(
                    'name' => strtolower($d['client-hostname'] ?? "ipxe"),
                    'ip' => $ip,
                    'mac' => strtolower($d['hardware ethernet']),
                    'state' => $d['binding state']
                );
            }
        }
        // apcu_store('dhcp_leases', $ret, 300);
    }
    return $ret;
}

/**
 * liste les baux dhcp actifs
 *
 * @param unknown $config
 * @return unknown|unknown[][]|NULL[][]|mixed[][]
 */
function list_dhcp_leases($config)
{
    $sup = 0;
    $parser = import_dhcp_leases($config);
    $machines = search_ad($config, "*", "machine");
    foreach ($parser as $key => $lease) {
        foreach ($machines as $machine) {
            if (isset($machine['state']) && (isset($machine['networkaddress']) && $machine['networkaddress'] == $lease['mac'])) {
                if (($machine['state'] == "reservation") && ($machine['networkaddress'] == $lease['mac'])) {
                    unset($parser[$key]);
                    $sup ++;
                    break;
                } else {
                    $parser[$key]['state'] = $machine['state'];
                }
            }
        }
    }
    return $parser;
}

function search_dhcp_leases($config, $ip)
{
    $res = [];
    $machines = search_ad($config, "*", "machine");
    foreach ($machines as $machine) {
        if (preg_match("/". $ip. "/", @$machine['iphostnumber']) == 1 || preg_match("/". $ip. "/", @$machine['cn']) == 1 || preg_match("/". $ip. "/", @$machine['networkaddress']) == 1) {
            $res[] = $machine;
        }
    }
    return $res;
}

function get_dhcp_lease($config, $leases, $name)
{
    foreach ($leases as $data) {
        if (($data['ip'] == $name) || ($data['name'] == strtolower($name)) || ($data['mac'] == $name))
            return $data;
    }
    return false;
}

// ----------- gestion des classes------------------------------
function is_eleve($config, $name)
{
    if (count(search_ad($config, $name, "member", "Eleves")) > 0)
        return true;
    else
        return false;
}

function is_prof($config, $name)
{
    if (count(search_ad($config, $name, "member", "Profs")) > 0)
        return true;
    else
        return false;
}

function is_pp(array $config, string $name)
{}

function list_groups($config, $user)
{
    $groups = [];
    foreach (search_ad($config, $user, "user", $config['dn']["people"])['memberof'] as $dn) {
        $groups[] = search_group($config, $dn);
    }
    return $groups;
}

/**
 *
 * @description liste les classes pour une classe, un eleve, un prof, ou tous
 * @parametres array $config, string $name : eleve, prof, "*"
 * @return array $classe : le nom court des classes
 */
function list_classes(array $config, $name)
{
    $ret = array();
    if (is_array($name)) {
        foreach ($name['memberof'] as $group) {
            if (preg_match("/CN=(Classe|Equipe)_(.*)," . $config['dn']['classes'] . "/i", $group, $m)) {
                $ret[] = $m[2];
            }
        }
    } else {

        $classes = search_ad($config, $name, "classe");
        if (count($classes) == 0) {
            $classes = search_ad($config, $name, "equipe");
        }
        if (is_array($classes)) {
            /*
             * foreach ($classes as $val) {
             * $res = type_group($config, $val['cn']);
             * $ret[] = $res['nom'];
             * }
             */
            foreach ($classes as $classe) {
                $ret[] = preg_split("/_/", $classe['cn'], 2)[1];
            }
        }
    }
    return $ret;
}

/**
 *
 * @description liste les profs pour une classe, un eleve ou tous
 * @type
 * @parametres array $config, string $name : classe ou eleve ou "*"
 * @returns array [profs]
 */
function list_profs($config, $name)
{
    $classes = list_classes($config, $name);
    $profs = array();
    foreach ($classes as $classe) {
        $equipe = search_ad($config, $classe, "equipe");
        // une seule equipe ?
        foreach ($equipe[0]['member'] as $prof) {
            $profs[] = $prof;
        }
    }
    return $profs;
}

/**
 *
 * @description retourne la liste des eleves du prof, ou de la classe ou tous
 * @type
 * @parametres array $config, string $name : prof, classe, "*"
 * @returns array [eleves]
 */
function list_eleves($config, $name)
{
    $classes = list_classes($config, $name);
    $eleves = array();
    foreach ($classes as $classe) {
        $res = search_ad($config, $classe, "classe");
        // une seul resultat ?
        foreach ($res[0]['member'] as $eleve) {
            $eleves[] = $eleve;
        }
    }
    return $eleves;
}

/**
 *
 * @description retourne la liste des profs principaux de l'eleve, de la classe ou tous
 * @type
 * @parametres array $config, string $name : eleve, classe, "*"
 * @returns array(pp)
 */
function list_pp($config, $name)
{
    $classes = list_classes($config, $name);
    $pps = array();
    foreach ($classes as $classe) {
        $res = search_ad($config, $classe, "pp");
        // une seule equipe ?
        if (isset($res[0]['member'])) {
            foreach ($res[0]['member'] as $pp) {
                $pps[] = $pp;
            }
        }
    }
    return $pps;
}

/**
 * prof de l'élève ?
 *
 * @description teste si $prof est prof pour $name (eleve ou classe)
 * @type
 * @parametres array $config, string $name eleve ou classe, string $prof : prof
 * @returns boolean
 */
function is_my_prof($config, $name, $prof)
{
    $profs = list_profs($config, $name);
    $dn = search_user($config, $prof);
    return in_array($dn['dn'], $profs);
}

/**
 * est-il mon élève ?
 *
 * @description teste si $eleve est eleve pour $name (prof ou classe)
 * @type
 * @parametres array $config, string $name: prof ou classe, string $eleve: eleve
 * @returns boolean
 */
function is_my_eleve($config, $name, $eleve)
{
    $eleves = list_eleves($config, $name);
    $dn = search_user($config, $eleve);
    return (in_array($dn['dn'], $eleves) && ($name != $eleve));
}

/**
 * Test Prof principal
 *
 * @description teste si $prof est prof principal pour $name (eleve ou classe)
 *
 * @param array $config
 * @param string $name
 *            eleve, classe
 * @param string $prof
 *            prof
 * @return boolean
 */
function is_my_pp($config, $eleve, $prof)
{
    $pps = list_pp($config, $eleve);
    $dn = search_user($config, $prof);
    if (isset($dn['dn']))
        return in_array($dn['dn'], $pps);
    else
        return false;
}

/**
 * Type de groupe
 *
 * @Description retourne le type de groupe
 * @parametres array $config, string $name : nom complet du groupe (Classe_TRUC) ou court (TRUC)
 * @returns array nom et type du groupe. si les conventions de nommage et de placement ne sont pas vérifiées , retourne type=>"groupe"
 */
function type_group(array $config, string $name)
{
    $groups = search_ad($config, "*" . $name, "group");
    if (count($groups) > 0) {
        foreach ($groups as $group) {
            if (strcasecmp("Classe_" . $name, $group['cn']) == 0 || strcasecmp($name, $group['cn']) == 0) {
                $m = array();
                $cn = explode("_", $group['cn'], 2);
                if (preg_match("/ou=(.*),ou=.*/i", $group['dn'], $m)) {
                    if (strcasecmp($m[1], $cn[0]) == 0 || strcasecmp($m[1], $cn[0] . "s") == 0) {
                        return array(
                            'nom' => $cn[1],
                            'type' => strtolower($cn[0])
                        );
                    } else {
                        return array(
                            'nom' => $name,
                            'type' => "groupe"
                        );
                    }
                }
            }
        }
    }
    return false;
}

/**
 * Ajout d'un groupe dans l'AD
 *
 * @Description crée le groupe du type donné
 * @parametres array $config, string $name : nom court sans le prefixe (Classe_, Equipe_) : nom du groupe
 * string $description, string $type : classe (classe+equipe), matiere, cours, projet, groupe
 * @returns boolean
 */
function create_group(array $config, string $name, string $description, string $type = "groupe")
{
    $res = type_group($config, $name);
    if (is_array($res) && ($type != $res['type'])) {
        // on ne cree pas de groupe du meme nom qu'une classe !
        return false;
    }
    if (($type == "classe") || ($type == "equipe")) {
        $classe = "Classe_" . $name;
        $ou = $config['classes_rdn'] . "," . $config['groups_rdn'];
        $res = groupadd($config, $classe, $ou, $description);
        $equipe = "Equipe_" . $name;
        $ou = $config['equipes_rdn'] . "," . $config['groups_rdn'];
        $res = groupadd($config, $equipe, $ou, "Equipe pédagogique de " . $description);
        $pp = "PP_" . $name;
        $ou = $config['equipes_rdn'] . "," . $config['groups_rdn'];
        $res = groupadd($config, $pp, $ou, "Profs principaux de " . $description);
    } elseif ($type == "cours") {
        $cours = "Cours_" . $name;
        $ou = $config['cours_rdn'] . "," . $config['groups_rdn'];
        $res = groupadd($config, $cours, $ou, "Cours de " . $description);
        // if (! ouexist($config, $ouName[1], "Groups")) {
        // ouadd($config, $ouName[1], "Groups");
        // }
    } elseif ($type == "projet") {
        $projet = "Projet_" . $name;
        $ou = $config['projets_rdn'] . "," . $config['groups_rdn'];
        $res = groupadd($config, $projet, $ou, "Projet " . $description);
    } elseif ($type == "other_group") {
        $classe = ucfirst($name);
        $ou = $config[$type . "s_rdn"] . "," . $config['groups_rdn'];
        // On crée l'OU si elle n'existe pas
        $ouName = explode("=", $config[$type . "s_rdn"]);
        if (! ouexist($config, $ouName[1], $config['groups_rdn'])) {
            ouadd($config, $ouName[1], $config['groups_rdn']);
        }
        $res = groupadd($config, $classe, $ou, $description);
    } else {
        $classe = ucfirst($type) . "_" . $name;
        $ou = $config[$type . "s_rdn"] . "," . $config['groups_rdn'];
        $res = groupadd($config, $classe, $ou, $description);
    }
    return $res;
}

/**
 * ajoute un eleve au groupe
 *
 * @param array $config
 * @param string $classe
 *            nom du groupe (forme courte admise)
 * @param string $user
 * @param string $type
 *            classe, cours, projet
 * @param bool $update
 *            : mettre à jour les partages (eleve) ou prof principal (prof)
 * @return boolean
 */
function add_user_group(array $config, string $classe, string $user, bool $update = false, bool $pp = false)
{
    $res = type_group($config, $classe);
    $type = $res['type'];
    $nom = $res['nom'];
    if ($type == "classe") {
        if (is_eleve($config, $user)) {
            $oldclasses = list_classes($config, $user);
            $res = groupaddmember($config, $user, "Classe_" . $nom);
            // un élève ne peut être que dans une classe !
            if ($res)
                foreach ($oldclasses as $oldclasse) {
                    groupdelmember($config, $user, $oldclasse);
                }
            if ($update) {
                $res = update_eleve($config, $user);
            }
        } elseif (is_prof($config, $user)) {
            $res = groupaddmember($config, $user, "Equipe_" . $nom);
            if ($res) {
                $my_pp = is_my_pp($config, $classe, $user);
                if ((! $my_pp) && $pp) {
                    $res = groupaddmember($config, $user, "PP_" . $nom);
                    $res = groupaddmember($config, $user, "Profs_Principaux");
                    print "ajout";
                } elseif ($my_pp && (! $pp)) {
                    $res = groupdelmember($config, $user, "PP_" . $nom);
                    print "suppression";
                    // si il ne reste plus aucune classe ou il est pp
                    if (count(list_pp($config, $user)) == 0)
                        $res = groupdelmember($config, $user, "Profs_Principaux");
                }
            }
        } else {
            // CPE, AED...
            $res = groupaddmember($config, $user, "Equipe_" . $nom);
        }
    } else {
        $classe = ucfirst($type) . "_" . $nom;
        $res = groupaddmember($config, $user, $classe);
    }
    return $res;
}

/**
 * enlève un utilisateur d'un groupe
 *
 * @param array $config
 * @param string $classe
 * @param string $eleve
 * @param bool $update
 * @return boolean
 */
function remove_user_group(array $config, string $classe, string $user, bool $update = false)
{
    $res = type_group($config, $classe);
    $type = $res['type'];
    $nom = $res['nom'];
    if ($type == "classe") {
        if (is_eleve($config, $user)) {
            $res = groupdelmember($config, $user, "Classe_" . $nom);
        } elseif (is_prof($config, $user)) {
            $res = groupdelmember($config, $user, "Equipe_" . $nom);
            $res = groupdelmember($config, $user, "PP_" . $nom);
        } else {
            $res = groupdelmember($config, $user, "Equipe_" . $nom);
        }
    } else {
        $classe = ucfirst($type) . "_" . $nom;
        $res = groupdelmember($config, $user, $classe);
    }
    return $res;
}

function delete_group($config, $name)
{
    $res = type_group($config, $name);

    if (is_array($res)) {
        $type = $res['type'];
        $classe = $res['nom'];
    }

    if (($type == "classe") || ($type == "equipe")) {
        $name = "Classe_" . $classe;
        $res = groupdel($config, $name);
        $equipe = "Equipe_" . $classe;
        $res = groupdel($config, $equipe);
        $pp = "PP_" . $classe;
        $res = groupdel($config, $pp);
    } else {
        $res = groupdel($config, $name);
    }
    return $res;
}

/*
 * test si $name (un cn) est prof principal de $classe
 * retourne un boolean
 */
function is_pp_this_classe($config, string $name, string $classe)
{
    $lespp = array();
    $lespp = list_pp($config, $classe);
    return preg_grep("/$name/i", $lespp);
}

/*
 * retourne un tableau [cn,Prenom Nom,Nom,sexe,Ddn, dn],
 * trié par Nom croissant
 * des membres du groupe $groupe
 */
function search_people_group($config, string $groupe)
{
    $res = search_ad($config, "*", "memberof", $groupe);
    usort($res, "cmp_nom");
    return $res;
}

/*
 * Retourne le tableau de la liste des classes
 * sous la forme de leur nom courtavec un cache
 * de 30 minutes
 * @return array
 */
function list_classes_etab(array $config)
{
    $retour = apcu_fetch('list_classes_etab');
    if ($retour === false) {
        $retour = [];
        $classes = search_ad($config, "Classe*", "group");
        foreach ($classes as $name) {
            $retour[] = type_group($config, $name['cn'])['nom'];
        }
        sort($retour);
        apcu_add('list_classes_etab', $retour, 18000);
    }
    return $retour;
}

?>
