<?php

function log_connexion($config, $user, $machine, $os, $action)
{
    // statut de la machine dans la base sql.
    $status = get_machine_status($config, $machine);

    $db_link = db_connect($config);

    switch ($action) {
        case 'startup':
            if ($status['state'] != $os) {
                // on considère qu'au bout de 3 minutes, l'action aurait dû se terminer.
                // le score permet de classer les machines qui bootent vite !
                $score = 300 - (time() - strtotime($status['starttime']));
                if ($score <= 0) {
                    $score = 0;
                }
                if ($status['state'] == "off") {
                    // nouveau boot
                    $query = mysqli_prepare($db_link, "insert into `machines` (name, os, starttime) values (?, ?, now())");
                    mysqli_stmt_bind_param($query, "ss", $machine, $os);
                    mysqli_stmt_execute($query);
                    mysqli_stmt_close($query);
                } elseif ($status['state'] == "wol") {
                    $query = mysqli_prepare($db_link, "update `machines` set wol=?, os=?, starttime=now()  where name=? and stoptime=0");
                    mysqli_stmt_bind_param($query, "iss", $score, $os, $machine);
                    mysqli_stmt_execute($query);
                    mysqli_stmt_close($query);
                } elseif ($status['state'] == "ipxe") {
                    $query = mysqli_prepare($db_link, "update `machines` set ipxe=?, starttime=now(), os=?  where name=? and stoptime=0");
                    mysqli_stmt_bind_param($query, "iss", $score, $os, $machine);
                    mysqli_stmt_execute($query);
                    mysqli_stmt_close($query);
                } else {
                    $query = mysqli_prepare($db_link, "update `machines` set starttime=now(), os=? where name=? and stoptime=0");
                    mysqli_stmt_bind_param($query, "ss", $os, $machine);
                }
                $query = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where netbios_name=? and logouttime=0");
                mysqli_stmt_bind_param($query, "s", $machine);
            }
            break;
        case 'shutdown':
            $query = mysqli_prepare($db_link, "update `machines` set stoptime=now() where name=? and stoptime=0");
            mysqli_stmt_bind_param($query, "s", $machine);
            mysqli_stmt_execute($query);
            mysqli_stmt_close($query);
            $query = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where netbios_name=? and logouttime=0");
            mysqli_stmt_bind_param($query, "s", $machine);
            break;
        case 'logon':
            if (! empty($user) && $user != "root" && $user != "Debian-gdm"){
                if ($status['state'] != $os) {
                    $query = mysqli_prepare($db_link, "update `machines` set os=? where name=? and stoptime=0");
                    mysqli_stmt_bind_param($query, "ss", $os, $machine);
                    mysqli_stmt_execute($query);
                    mysqli_stmt_close($query);
                }
                $query = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where username=? and netbios_name=? and logouttime=0");
                mysqli_stmt_bind_param($query, "ss", $user, $machine);
                mysqli_stmt_execute($query);
                mysqli_stmt_close($query);
                $query = mysqli_prepare($db_link, "insert into `connexions` (username, netbios_name, logintime) values (?, ?, now())");
                mysqli_stmt_bind_param($query, "ss", $user, $machine);
            }
            break;
        case 'logoff':
            if (! empty($user) && $user != "root" && $user != "Debian-gdm") {
                $query = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where username=? and netbios_name=? and logouttime=0");
                mysqli_stmt_bind_param($query, "ss", $user, $machine);
            }
            break;
        case 'blacklist':
            $query = mysqli_prepare($db_link, "update `machines` set wol=0, ipxe=0, stoptime=now(), os=?  where name=? and stoptime=0");
            mysqli_stmt_bind_param($query, "ss", $os, $machine);
            break;
    }
    if (isset($query) && $query) {
        mysqli_stmt_execute($query);
        mysqli_stmt_close($query);
        mysqli_close($db_link);
    }
}

function get_machine_status($config, $machine)
{
    $db_link = db_connect($config);
    // $query = mysqli_prepare($db_link, "select starttime, os from `machines` where name = ? and stoptime=0 order by id desc limit 1");
    $query = mysqli_prepare($db_link, "select starttime, stoptime, os from `machines` where name = ? order by id desc limit 1");
    mysqli_stmt_bind_param($query, "s", $machine);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_starttime, $res_stoptime, $res_os);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    $query2 = mysqli_prepare($db_link, "select username, logintime from `connexions` where netbios_name = ? and logouttime=0");
    mysqli_stmt_bind_param($query2, "s", $machine);
    mysqli_stmt_execute($query2);
    mysqli_stmt_bind_result($query2, $res_username, $res_logintime);
    mysqli_stmt_store_result($query2);
    $num_rows2 = mysqli_stmt_num_rows($query2);
    if ($num_rows2 > 0) {
        while (mysqli_stmt_fetch($query2)) {
            $tab['user'][] = array(
                'name' => $res_username,
                'logintime' => $res_logintime
            );
        }
    }
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab['state'] = $res_os;
            if ($res_stoptime == 0) {
                $tab['starttime'] = $res_starttime;
            } elseif (isset($tab['user']) && count($tab['user']) > 0) {
                // on ne devrait jamais arriver ici...
                $query3 = mysqli_prepare($db_link, "insert into `machines` (name, os, starttime) values (?, ?, now())");
                mysqli_stmt_bind_param($query3, "ss", $machine, $res_os);
                mysqli_stmt_execute($query3);
                mysqli_stmt_close($query3);
                $tab['starttime'] = time();
            } else {
                $tab['state'] = "off";
                $tab['starttime'] = 0;
            }
        }
    }
    mysqli_stmt_close($query2);
    mysqli_stmt_close($query);

    $query2 = mysqli_prepare($db_link, "select wol, ipxe from `machines` where name = ? and stoptime!=0 order by id desc limit 1");
    mysqli_stmt_bind_param($query2, "s", $machine);
    mysqli_stmt_execute($query2);
    mysqli_stmt_bind_result($query2, $res_wol, $res_ipxe);
    mysqli_stmt_store_result($query2);
    $num_rows2 = mysqli_stmt_num_rows($query2);
    if ($num_rows2 > 0) {
        while (mysqli_stmt_fetch($query2)) {
            $tab['score'] = $res_wol + $res_ipxe;
        }
    } else
        $tab['score'] = 0;

    mysqli_close($db_link);
    return $tab;
}

function get_user_status($config, $user)
{
    $db_link = db_connect($config);
    $query = mysqli_prepare($db_link, "select netbios_name, logintime from `connexions` where username=? and logouttime=0 order by logintime desc");
    mysqli_stmt_bind_param($query, "s", $user);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_machine, $res_logintime);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'machine' => strtolower($res_machine),
                'logintime' => $res_logintime
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function list_user_connexions($config, $user, $start = 0, $nb = 10)
{
    $db_link = db_connect($config);
    $query = mysqli_prepare($db_link, "select netbios_name, logintime, logouttime from `connexions` where username=? order by id desc limit " . $start . "," . $nb);
    mysqli_stmt_bind_param($query, "s", $user);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_machine, $res_logintime, $res_logouttime);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'machine' => strtolower($res_machine),
                'logintime' => $res_logintime,
                'logouttime' => $res_logouttime
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function list_machine_connexions($config, $machine, $start = 0, $nb = 10)
{
    $db_link = db_connect($config);
    $query = mysqli_prepare($db_link, "select username, logintime, logouttime from `connexions` where netbios_name=? order by id desc limit " . $start . "," . $nb);
    mysqli_stmt_bind_param($query, "s", $machine);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_user, $res_logintime, $res_logouttime);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'user' => $res_user,
                'logintime' => $res_logintime,
                'logouttime' => $res_logouttime
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function list_machine_startups($config, $machine, $start = 0, $nb = 10)
{
    $db_link = db_connect($config);
    $query = mysqli_prepare($db_link, "select os, starttime, stoptime from `machines` where name=? order by starttime desc limit " . $start . "," . $nb);
    mysqli_stmt_bind_param($query, "s", $machine);
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_os, $res_starttime, $res_stoptime);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'os' => $res_os,
                'starttime' => $res_starttime,
                'stoptime' => $res_stoptime
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function open_sessions($config)
{
    $db_link = db_connect($config);
    $query = mysqli_prepare($db_link, "select username, netbios_name, logintime from `connexions` where logouttime=0 order by username");
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_username, $res_machine, $res_logintime);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    if ($num_rows > 0) {
        while (mysqli_stmt_fetch($query)) {
            $tab[] = array(
                'machine' => strtolower($res_machine),
                'user' => $res_username,
                'logintime' => $res_logintime
            );
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $tab;
}

function clean_machines_status($config)
{
    // on cherche les machines demarrées win ou linux ou depuis plus de 2 minutes
    $db_link = db_connect($config);
    $query = mysqli_prepare($db_link, "select name, os from `machines` where stoptime=0 and
                    (DATE_SUB(now(),INTERVAL 2 MINUTE) >= starttime) order by id desc");
    // (os='windows' or os='linux' or DATE_SUB(now(),INTERVAL 2 MINUTE) >= starttime) order by id desc");
    mysqli_stmt_execute($query);
    mysqli_stmt_bind_result($query, $res_name, $res_os);
    mysqli_stmt_store_result($query);
    $num_rows = mysqli_stmt_num_rows($query);
    $tab = array();
    $out = "";
    if ($num_rows > 0) {
        // $out = "nettoyage des connexions des machines plantees :<br>\n";
        while (mysqli_stmt_fetch($query)) {
            if (! fping($res_name)) {
                // pas de ping, on ferme les cnx
                $query2 = mysqli_prepare($db_link, "update `machines` set stoptime=now() where name='" . $res_name . "' and stoptime=0");
                mysqli_stmt_execute($query2);
                mysqli_stmt_close($query2);
                $query2 = mysqli_prepare($db_link, "update `connexions` set logouttime=now() where netbios_name='" . $res_name . "' and logouttime=0");
                mysqli_stmt_execute($query2);
                mysqli_stmt_close($query2);
                if ($res_os == "wol") {
                    $out .= strtolower($res_name) . " n'a pas été allumée, soit le WOL ne fonctionne pas, soit la machine est débranchée ou n'existe plus<br>\n";
                } elseif ($res_os == "ipxe") {
                    $out .= strtolower($res_name) . " a booté en ipxe, mais n'a pas démarré correctement de système d'exploitation ensuite<br>\n";
                } else {
                    $out .= "connexion fermee pour " . strtolower($res_name) . " os : " . $res_os . "<br>\n";
                }
            } else {
                // $out .= "test connexion pour " . strtolower($res_name) . " : " . $res_os . "\n";
                /*
                 * if ($res_os != "windows" && $res_os != "linux") {
                 * $query2 = mysqli_prepare($db_link, "update `machines` set os='windows' where name='" . $res_name . "' and stoptime=0");
                 * mysqli_stmt_execute($query2);
                 * mysqli_stmt_close($query2);
                 * $out .= "connexion actualisée pour " . strtolower($res_name) . "<br>\n";
                 * }
                 */
                $smb_status = machine_smbstatus($config, $res_name);
                if (! empty($smb_status && $smb_status != "wpkg")) {
                    $sql_status = get_machine_status($config, $res_name);
                    if (! @in_array($smb_status, array_column($sql_status['user'], 'name'))) {
                        log_connexion($config, $smb_status, $res_name, $res_os, "logon");
                        $out .= "connexion actualisée pour " . strtolower($res_name) . " utilisateur " . $smb_status . "<br>\n";
                    }
                }
            }
        }
    }
    mysqli_stmt_close($query);
    mysqli_close($db_link);
    return $out;
}
?>