<?php

/**
 * api de base pour pronote
 * action permet de récupérer les infos en temps réél: 
 * - groupes : listes des groupes
 * - classe : listes des eleves et profs pour une classe ou groupe
 * - salles : emploi du temps du jour courant pour les salles
 *  
 * @param unknown $config
 * @param string $action : groupes|salles|classe
 * @param string $nom : nom de la classe
 * @return array()
 */
function pronote_api($config, $action, $nom = "", $periode = 0)
{
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['pronote_api_url'],
        'cookies' => true
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    try {
        $response = $client->request('POST', '/', [
            'json' => [
                "type" => $action,
                "username" => $config['pronote_api_user'],
                "password" => $config['pronote_api_passwd'],
                "url" => $config['pronote_server_url'],
                "typecon" => "viescolaire.html",
                "cas" => $config['pronote_cas'],
                "nom" => $nom,
                "periode" => $periode
            ]
        ]);
        (string) $data = $response->getBody();
        return json_decode($data, true);
    } catch (\GuzzleHttp\Exception\ConnectException $e) {
        return false;
    }
}

function get_salle_state($config, $salle)
{
    $action = false;
    $timers = wol_table($config)[$salle] ?? [];
    foreach ($timers as $timer) {
        if (time() > $timer['start'] && time() < $timer['stop']) {
            $action = 'wol';
            break;
        } else {
            $action = 'stop';
        }
    }
    return $action;
}

function wol_table($config)
{
    $wol = apcu_fetch('wol', $success);
    if (! $success) {
        if ($salles = pronote_api($config, "salles")) {
            foreach ($salles as $bloc) {
                $heure = [
                    'start' => $bloc['date'] - 600,
                    'stop' => $bloc['date'] + $bloc['duree'] * $config['pronote_cours_unit'] + 600
                ];
                $wol[$bloc['nom']][] = $heure;
            }

            apcu_store('wol', $wol, 7200);
        }
    }
    return $wol;
}

function check_pronote_password($config, $user, $password)
{
    $type_user = type_user($config, $user);
    $map = [
        'Eleves' => "eleve.html",
        'Profs' => "professeur.html",
        'Administratifs' => "viescolaire.html"
    ];
    require_once (dirname(__FILE__) . '/../vendor/autoload.php');

    $opt = [
        'base_uri' => $config['pronote_api_url'],
        'cookies' => true
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    $response = $client->request('POST', '/', [
        'json' => [
            "type" => "login",
            "username" => $user,
            "password" => $password,
            "url" => $config['pronote_server_url'],
            "typecon" => $map[$type_user],
            "cas" => $config['pronote_cas']
        ]
    ]);
    if ($response) {
        (string) $data = $response->getBody();
        return json_decode($data, true);
    } else {
        return false;
    }
}

function sync_pronote($config, $merge = false)
{
    $users = apcu_fetch('pronote');
    if (empty($users)) {
        $groups = pronote_api($config, "groups");
        $users = [];
        foreach ($groups as $group) {
            $key = $group['nom'];
            $type = $group['type'];
            if ($type == 1) {
                $type = "allClasses";
                $classe = pronote_api($config, "classe", $key);
            } else {
                $classe = [
                    'equipe' => [],
                    'eleves' => []
                ];
                $classe = pronote_api($config, "classe", $key);
                $type = "allGroups";
            }
            foreach ($classe['equipe'] as $u) {
                $user = [
                    'lastName' => $u['nom'],
                    'firstName' => $u['prenom'],
                    'birthDate' => $u['naissance'] ?? "",
                    'sex' => $u['civ'] ?? "",
                    'type' => 'Teacher'
                ];
                $id = hash('md5', $user['lastName'] . $user['firstName'] . $user['birthDate'] . $user['sex']);
                $index = array_search($id, array_column($users, 'pronote_id'));
                if ($index) {
                    $users[$index][$type][] = [
                        'name' => $key
                    ];
                } else {
                    $user['pronote_id'] = $id;
                    $user[$type][] = [
                        'name' => $key
                    ];
                    $users[] = $user;
                }
            }
            foreach ($classe['eleves'] as $u) {
                $user = [
                    'lastName' => $u['nom'],
                    'firstName' => $u['prenom'],
                    'birthDate' => $u['naissance'] ?? "",
                    'sex' => $u['sexe'] ?? "",
                    'type' => 'Student'
                ];
                $id = hash('md5', $user['lastName'] . $user['firstName'] . $user['birthDate'] . $user['sex']);
                $index = array_search($id, array_column($users, 'pronote_id'));
                if ($index) {
                    $users[$index][$type][] = [
                        'name' => $key
                    ];
                } else {
                    $user['pronote_id'] = $id;
                    $user[$type][] = [
                        'name' => $key
                    ];
                    $users[] = $user;
                }
            }
        }
        if (! $merge) {
            apcu_store('ent', $users, 3600);
        }
        apcu_store('pronote', $users, 3600);
    }
    $html = count($users) . " utilisateurs synchronisés<br>";
    return $html;
}
?>