<?php

function test_remote($machine, $type = "all")
{
    if (empty($type))
        $type = "all";

    if ($type == "all") {
        if (ts($machine, .5)) {
            if (vnc($machine, .5)) {
                return "all";
            } else {
                return "rdp";
            }
        } elseif (vnc($machine, .5)) {
            return "vnc";
        } else {
            return false;
        }
    } elseif ($type == "rdp" && ts($machine, .5)) {
        return "rdp";
    } elseif ($type == "vnc" && vnc($machine, .5)) {
        return "vnc";
    } else {
        return false;
    }
}

function add_unique($nom, &$liste)
{
    $found = false;
    $ret = array();
    if (empty($liste))
        $ret[] = $nom;
    else {
        foreach ($liste as $poste) {
            if ($nom['cn'] != $poste['cn']) {
                $ret[] = $poste;
            } else {
                $ret[] = array_replace($poste, $nom);
                $found = true;
            }
        }
        if (! $found) {
            $ret[] = $nom;
        }
    }
    $liste = $ret;
}

function remove_unique($nom, &$liste)
{
    $ret = array();
    foreach ($liste as $poste) {
        if ($nom['cn'] != $poste['cn'])
            $ret[] = $poste;
    }
    $liste = $ret;
}

function remote_status($config, $login, $poste, $type = "", $action = "")
{
    if (empty($action)) {
        // log de connexion
        $status = get_machine_status($config, $poste['cn']);
        $poste['status'] = $status['state'];
        $poste['score'] = $status['score'];
    } else {
        $poste['status'] = $action;
    }
    // test ports rdp/vnc
    if ($type = test_remote($poste['cn'], $type)) {
        // if ($type = test_remote($poste['iphostnumber'], $type)) {
        $poste['type'] = $type;
        // $poste['timer'] = 0;
        $poste['remote'] = list_remote_connexion($config, $poste['cn'], $login);
        // poste allumé et opérationnel
        if (@count($status['user']) > 0) {
            foreach ($status['user'] as $user) {

                if ($user['name'] == $login) {
                    // session ouverte
                    $poste['status'] = 'login';
                    break;
                } elseif ($user['name'] != "wpkg" && $status['state'] != "linux") {
                    // session ouverte par un autre utilisateur
                    $poste['status'] = 'no';
                } else {
                    // poste dispo
                    $poste['status'] = 'on';
                }
            }
        } else {
            // poste dispo
            $poste['status'] = 'on';
        }
    } elseif ($action == "wait") {
        // le poste n'est pas prêt
        // if (empty($poste['timer'])){
        // $poste['timer'] = time();
        // }
        // if (time() - $poste['timer'] > 120) {
        // le poste ne s'est pas allumé ou ports fermés
        // $poste['status'] = 'no';
        // }
    } else {
        $poste['status'] = 'off';
    }
    return $poste;
}

/**
 * Recherche une machine libre dans les parcs pour lesquels l'utilisateur a le droit de faire du rdp
 * - 1 renvoie la machine à laquelle l'utilisateur est connecté
 * - 2 renvoie une machine allumée du parc
 * - 3 liste les machines éteintes
 *
 *
 *
 * @param unknown $config
 * @param unknown $login
 * @return unknown
 */
function find_remote_list($config, $login, $force = false, $machine = "", $action = "")
{
    $time = time();
    $liste = [];
    $remote = apcu_fetch('remote');
    if (! empty($remote[$login]) && ! $force) {
        $liste = $remote[$login]['liste'];
        $postes = $remote[$login]['postes'];
    }
    if (count($liste) == 0 || $force) {
        // recherche des postes autorisés
        $postes = array();
        $parcs = list_delegations($config, $login, true, "rdp");
        $parcs = array_merge($parcs, list_delegations($config, $login, true, "vnc"));
        foreach ($parcs as $parc) {
            $postes = array_merge($postes, array_slice(list_members_parc_fast($config, "cn=" . $parc['cn'] . "," . $config['dn']['parcs'], true), 0, 50));
        }
        $postes = array_unique_multi($postes);
    }
    if (empty($action)) {
        // on traite le statut des postes pas encore traités
        if (count($postes) > 0) {
            foreach ($postes as $k => $poste) {
                add_unique(remote_status($config, $login, $poste), $liste);
                unset($postes[$k]);
                if (time() - $time > 10) {
                    break;
                }
            }
        }

        // on cherche les postes ou l'utilisateur est déjà connecté, même en en dehors des délégations
        foreach (get_user_status($config, $login) as $m) {
            $poste = search_machine($config, $m['machine']);
            add_unique(remote_status($config, $login, $poste), $liste);
        }
    }
    // on ajoute le poste demandé
    if (! empty($machine)) {
        $poste = search_machine($config, $machine);
        add_unique(remote_status($config, $login, $poste, "", $action), $liste);
    }
    $etats = [
        'login',
        'wait',
        'on',
        'off',
        'no'
    ];
    $liste_triee = [];

    if (isset($liste[0]['score'])) {
        array_multisort(array_column($liste, "score"), SORT_DESC, $liste);
        foreach ($etats as $etat) {
            foreach ($liste as $poste)
                if ($poste['status'] == $etat) {
                    if ($poste['cn'] == $machine)
                        array_unshift($liste_triee, $poste);
                    else
                        $liste_triee[] = $poste;
                }
        }
    }
    $remote[$login] = [
        'liste' => $liste_triee,
        'postes' => $postes
    ];
    apcu_store('remote', $remote, 600);
    return $liste_triee;
}

function list_remote_connexion($config, $machine, $name)
{
    $type = "";
    if (! empty($config['guacamole_url'])) {
        $user = search_user($config, $name);
        $remote = search_ad($config, $machine, "remote");
        if (count($remote) > 0) {
            if (in_array($user['dn'], $remote[0]['member'])) {
                $type = $remote[0]['guacconfigprotocol'];
            }
        }
    }
    return $type;
}

/**
 * crée une connexion rdp ou vnc pour l'utilisateur sur la machine.
 * pour le moment les 2 sont exclusifs, on
 * supprime le profil existant pour créer le nouveau.
 *
 * @param unknown $config
 * @param unknown $machine
 * @param unknown $name
 * @param string $type
 * @return boolean
 */
function create_remote_connexion($config, $machine, $name, $type = "rdp")
{
    $ret = ouadd($config, "ou=rdp," . $config['ldap_base_dn'], "", "dn");
    dsacl_set($config, "ou=rdp," . $config['ldap_base_dn'], "(A;CI;RPLCRC;;;AU)");
    if (! empty($config['guacamole_url'])) {
        $user = search_user($config, $name);
        $remote = search_ad($config, $machine, "remote");
        if (count($remote) > 0) {
            // if ($type != $remote[0]['guacconfigprotocol'] && in_array($user['dn'], $remote[0]['member'])) {
            delete_remote_connexion($config, $machine);
            // }
            /*
             * if (! in_array($user['dn'], $remote[0]['member'])) {
             * // $ret = groupaddmember($config, $user['cn'], $remote[0]['cn']);
             * $attrs = [
             * 'member' => $user['dn']
             * ];
             * modify_ad($config, $remote[0]['cn'], $type, $attrs, "add");
             * } else {
             * $ret = true;
             * }
             * } else {
             */
        }
        $dn = "cn=remote_" . $machine . ",ou=rdp," . $config['ldap_base_dn'];
        if ($type == "rdp") {
            $attrs = array(
                'objectclass' => array(
                    'top',
                    'group',
                    'guacConfigGroup'
                ),
                'cn' => "remote_" . $machine,
                'samAccountName' => "remote_" . $machine,
                'guacConfigParameter' => array(
                    "hostname=" . $machine,
                    "port=3389",
                    "password=\${GUAC_PASSWORD}",
                    "username=\${GUAC_USERNAME}",
                    "domain=" . $config['samba_domain'],
                    "server-layout=fr-fr-azerty",
                    "ignore-cert=true",
                    "resize-method=display-update"
                ),
                'guacConfigProtocol' => "rdp",
                'member' => $user['dn']
            );
        } else {
            $attrs = array(
                'objectclass' => array(
                    'top',
                    'group',
                    'guacConfigGroup'
                ),
                'cn' => "remote_" . $machine,
                'samAccountName' => "remote_" . $machine,
                'guacConfigParameter' => array(
                    "hostname=" . $machine,
                    "port=5900",
                    "password=" . $config['vnc_password']
                ),
                'guacConfigProtocol' => "vnc",
                'member' => $user['dn']
            );
        }
        $ds = bind_ad_gssapi($config);
        $ret = ldap_add($ds, $dn, $attrs);
        ldap_close($ds);
        // }
    } else {
        // $ret = "<input type=\"hidden\" name=\"refresh\" value=\"1\">";
        $ret = true;
    }
    return $ret;
}

function delete_remote_connexion($config, $machine)
{
    if (! empty($config['guacamole_url'])) {
        return groupdel($config, "remote_" . $machine);
    } else {
        return true;
    }
}

function display_remote_list($config, $login)
{
    $type = "all";
    $html = $message = $script = "";
    $force = $res = false;
    $action = $_POST['action'] ?? "";
    $time = $_POST['time'] ?? time();
    $machine = $_POST['machine'] ?? $_GET['machine'] ?? "";
    $open = $_POST['open'] ?? 0;
    $end = $_POST['end'] ?? 1;
    $post_type = $_POST['type'] ?? "rdp";
    if (empty($machine) && $end) {
        $force = true;
    }
    $liste = find_remote_list($config, $login, $force, $machine, $action);
    // $html .= "<pre>" . print_r($liste[0], true) . "</pre>";
    // si on a selectionné une machine
    $remote = apcu_fetch('remote');
    $end = (count($remote[$login]['postes']) == 0);
    $refresh = ! $end;

    if (! empty($machine)) {
        $p = array_search($machine, array_column($liste, 'cn'));
        $poste = $liste[$p];
        // si on a selectionné une machine

        switch ($poste['status']) {
            case 'login':
            case 'on':
                if ($poste['remote'] == $post_type) {
                    $res = true;
                } else {
                    $res = create_remote_connexion($config, $machine, $login, $post_type);
                    $open = 0;
                }
                if ($res && $open == 0) {
                    $url = $config['guacamole_url'];
                    $script = "<script type=\"text/javascript\">
        window.open('" . $url . "', 'guacamole', 'resizable=yes,height=1080,width=1920');
        </script>";
                    $open = 1;
                    $refresh = true;
                }
                $action = "login";
                break;
            case "wait":
                $diff = time() - $time;
                if ($diff > 120) {
                    $liste[$p]['status'] = "no";
                    $action = "no";
                } else {
                    sleep(5);
                    $message = $diff . "s";
                    $html .= "Allumage en cours... <progress  max=\"120\" value=" . $diff . "> " . $diff . "% </progress>";
                    $refresh = true;
                    $action = "wait";
                }
                break;
            case "off":
                start_poste($config, "wol", $poste['cn']);
                sleep(5);
                $liste[$p]['status'] = "wait";
                $time = time();
                $refresh = true;
                $action = "wait";
                break;
            default:
                $action = "";
        }
        // $html .= "index $p poste : <pre>" . print_r($liste[$p], true) . "</pre>";

        $remote[$login]['liste'] = $liste;
        apcu_store('remote', $remote, 600);
    }
    // $html .= "nb de machines : " . count($liste) . " nb en attente : " . count($remote[$login]['postes']) . " ";
    if (! $end) {
        $progress = 100 * count($liste) / (count($liste) + count($remote[$login]['postes']));
        $html .= "Détection des machines... <progress  max=\"100\" value=" . $progress . "> " . $progress . "% </progress>";
    }
    $categories = [
        'wait' => "attente d'allumage depuis " . $message,
        'login' => "rejoindre la session déjà ouverte",
        'on' => "se connecter sur la machine",
        'off' => "allumer et se connecter à la machine",
        'no' => "non disponible, déjà occupé"
    ];
    $html .= "<FORM action=\"\" id=\"remote\" method=\"post\" name=\"remote\">\n";
    $html .= "<label Choisir le poste pour la connexion à distance>";
    $html .= "<select name=\"machine\" size=6>";
    $i = 0;
    $oldstat = "";
    foreach ($liste as $poste) {
        if ($poste['status'] != $oldstat) {
            if ($i > 0) {
                $html .= "</optgroup>";
            }
            $oldstat = $poste['status'];
            $html .= "<optgroup label=\"" . $categories[$poste['status']] . "\">";
        }
        $remote_msg = "";
        $html .= "<option value=\"" . $poste['cn'] . "\"";
        if ($i ++ == 0 && $end) { // && (empty($machine) || $machine == $poste['cn'])) {
            $html .= " selected";
            $type = $poste['type'] ?? $post_type;
        }
        if (($poste['status'] == "on" || $poste['status'] == "login") && ! empty($poste['remote'])) {
            if ($poste['status'] == "on") {
                $remote_msg = " en attente de connexion " . $poste['remote'];
            } else {
                $remote_msg = " ouvert en " . $poste['remote'];
            }
        }
        $html .= ">" . $poste['cn'] . $remote_msg . "</option>";
    }

    $html .= "</select>";
    $html .= "</label>";
    $html .= "<input type=\"hidden\" name=\"open\" value=\"" . $open . "\">";
    $html .= "<input type=\"hidden\" name=\"end\" value=\"" . $end . "\">";
    $html .= "<input type=\"hidden\" name=\"time\" value=\"" . $time . "\">";

    if ($type == "all") {
        $html .= "<div>
        <input type=\"radio\" id=\"rdp\" name=\"type\" value=\"rdp\"";
        if ($post_type == "rdp") {
            $html .= " checked ";
        }
        $html .= ">";
        $html .= "<label for=\"rdp\">Connexion RDP</label>
            <input type=\"radio\" id=\"vnc\" name=\"type\" value=\"vnc\"";
        if ($post_type == "vnc") {
            $html .= " checked ";
        }
        $html .= ">";
        $html .= "<label for=\"vnc\">Connexion VNC</label>
            </div>";
    } else {
        $html .= "<input type=\"hidden\" name=\"type\" value=\"" . $type . "\">";
    }
    $html .= "<div>";
    $html .= "<input type=\"hidden\" name=\"action\" value=\"" . $action . "\">";
    $html .= "<input type=\"submit\" name=\"valider\" value=\"Valider\">";
    $html .= "</form>";
    /*
     * $html .= "<FORM action=\"\" id=\"actualiser\" method=\"post\" name=\"actualiser\">\n";
     * $html .= "<input type=\"hidden\" name=\"machine\" value=\"\">";
     * $html .= "<input type=\"hidden\" name=\"type\" value=\"" . $post_type . "\">";
     * $html .= "<input type=\"submit\" name=\"action\" value=\"Actualiser\">";
     * $html .= "</form>";
     */
    $html .= "</div>";
    $html .= $script;
    if ($refresh) {
        $html .= "<script type=\"text/javascript\">
    var wait = setTimeout(document.remote.submit(), 10000)
</script>";
    }

    return $html;
}

