<?php
/**
 * Librairie de fonctions utilisees dans l'interface d'administration

 * @Version $Id: samba-tool.inc.php  2018-28-09  jlcf $

 * @Projet  SambaEdu

 * @Note: Ce fichier de fonction doit etre appele par un include

 * @Licence Distribue sous la licence GPL
 */

/**
 * file: samba-tool.inc.php
 *
 * @Repertoire: includes/
 */

// =============================================================
// Ensemble de fonctions destinées à remplacer les scripts sudo perl
// pour les opérations d'écritures dans l'AD SambaEdu

/*
 * function useradd ($prenom, $nom, $userpwd, $naissance, $sexe, $categorie, $employeeNumber) : Return $cn if succes.
 *
 * function userdel ($cn) : Return true if userdel succes false if userdel fail
 *
 * function groupadd ($cn, $inou, $description) : Return true if group is create false in other cases
 *
 * function groupdel ($cn) : Return true if group is delete false in other cases
 *
 * function groupaddmember ( $cn, $ingroup) : Return true if cn is add in ingroup false in other cases
 *
 * function groupdelmember ($cn, $ingroup) : Return true if cn is remove of ingroup false in other cases
 *
 * A faire si nécessaire :
 * function grouplist ($filter)
 * function groupaddlistmembers ( $cnlist, $ingroup)
 *
 */
require_once ("siecle.inc.php");

/*
 * Fonctions de siecle.inc.php utilisées dans samba-tool.inc.php
 * useradd() -> creer_cn($nom,$prenom) //modifiée
 * -> verif_employeeNumber($employeeNumber) // modifiée
 */
function sambatool($config, $command, &$message)
{
    $RET = 0;
    exec("/usr/bin/samba-tool $command -k yes -H ldap://" . $config['se4ad_name'] . " 2>&1", $message, $RET);
    return $RET;
}

function userexist($config, $cn)
{
    /*
     * Return true if user exist false if not exist
     */
    $message = array();
    $command = "user list";
    $RES = sambatool($config, $command, $message);
    if ($RES == 0) {
        if (in_array($cn, $message))
            return true;
        else
            return false;
    } else {
        return false;
    }
}

/**
 * Ajout utilisateur
 *
 * @description : Ajoute un utlisateur dans l'AD
 * @paramètres config, $cn : nom court, $prenom, $nom, $userpwd, $naissance, $sexe  : M ou F, $categorie : Eleves ou Profs ou Administratifs, $employeeNumber
 * @Returns true ou false.
 */
function useradd($config, $cn, $prenom, $nom, $userpwd, $naissance, $sexe, $categorie, $employeeNumber)
{
    $office = $naissance . "," . $sexe;

    if (! isset($userpwd)) {
        $userpwd = $naissance;
    }
    $userpwd = escapeshellarg($userpwd);
    $prenom = escapeshellarg(ucfirst($prenom));
    $nom = escapeshellarg(ucfirst($nom));

    if (empty($employeeNumber)) {
        // Pas de champ job-title pour employeeNumber dans ce cas
        $command = "user create '$cn' $userpwd --use-username-as-cn --given-name=$prenom --surname=$nom --mail-address='$cn@" . $config['domain'] . "' --physical-delivery-office='$office'";
        if ($categorie != '')
            $command = $command . " --userou='ou=$categorie," . $config['people_rdn'] . "'";
    } else {
        $command = "user create '$cn' $userpwd --use-username-as-cn --given-name=$prenom --surname=$nom --mail-address='$cn@" . $config['domain'] . "' --job-title='$employeeNumber' --physical-delivery-office='$office'";
        if ($categorie != '')
            $command = $command . " --userou='ou=$categorie," . $config['people_rdn'] . "'";
    }
    $message = array();
    $RES = sambatool($config, $command, $message);
    if ($RES == 0) {
        $newcn = explode("'", $message[0]);
        // Ajout a un groupe principal
        $RES = groupaddmember($config, $newcn[1], $categorie);
        return true;
    } else
        return false;
}

/**
 * Suppression utilisateur
 *
 * @description : Supprime un utlisateur dans l'AD
 * @paramètres config, $cn
 * @Returns true ou false.
 */
function userdel($config, $cn)
{
    /*
     * Return true if userdel succes false if userdel fail
     */
    if (userexist($config, $cn)) {
        $message = array();
        $command = "user delete " . escapeshellarg($cn);
        $RES = sambatool($config, $command, $message);
        if ($RES == 0)
            return true;
        else
            return false;
    } else
        return false;
}

/**
 * Passwd utilisateur
 *
 * @description : Met en place le mot de passe d'un utilisateur
 * @paramètres $config, $cn, $password, $change = false
 * @Returns true ou false.
 */
function usersetpassword($config, $cn, $password, $change = false)
{
    /*
     * Return true if password succes false if userdel fail
     */
    if (userexist($config, $cn)) {
        $message = array();
        $command = "user setpassword " . escapeshellarg($cn) . " --newpassword=" . escapeshellarg($password);
        if ($change)
            $command .= " --must-change-at-next-login";
        $RES = sambatool($config, $command, $message);
        if ($RES == 0)
            return true;
        else
            return false;
    } else
        return false;
}

/*
 * Gestion des OU
 */

/**
 * Test existence OU
 *
 * @description : vérifie l'existence de  l'ou dans l'AD
 * @paramètres $config, $ou, $ouparent_rdn
 * @Returns true ou false.
 */
function ouexist($config, $ou, $ouparent_rdn = "", $prefix = "ou")
{

    /*
     * Return true if OU exist false in other cases
     */
    $contenu = array(
        "ou"
    );

    $ds = bind_ad_gssapi($config);

    if ($ds) {
        if (empty($ouparent_rdn)) {
            $dn = $ou;
            $ou_parent = ldap_dn2oudn($dn);
            $ou = ldap_dn2ou($dn);
            list ($prefix, $ou) = explode("=", $ou);
        } else {
            $ou_parent = $ouparent_rdn . "," . $config['ldap_base_dn'];
            $dn = $prefix . "=" . $ou . "," . $ou_parent;
        }
        $ret = ldap_search($ds, $ou_parent, "(" . $prefix . "=" . $ou . ")", $contenu);
        $info = ldap_get_entries($ds, $ret);
        if ($info["count"] > 0) {
            return true;
        } else {
            return false;
        }
    } else {
        echo "Echec du bind sasl";
        return false;
    }
}

/**
 * Ajout OU
 *
 * @description : Ajoute l'ou dans l'AD
 * @paramètres $config, $ou, $ouparent_rdn
 * @Returns true ou false.
 */
function ouadd($config, $ou, $ouparent_rdn = "", $prefix = "ou")
{

    /*
     * Return true if OU is create or if there already exists false in other cases
     */
    if (empty($ouparent_rdn)) {
        $dn = $ou;
        $ou_parent = ldap_dn2oudn($dn);
        $ou = ldap_dn2ou($dn);
        list ($prefix, $ou) = explode("=", $ou);
    } else {
        $ou_parent = $ouparent_rdn . "," . $config['ldap_base_dn'];
        $dn = $prefix . "=" . $ou . "," . $ou_parent;
    }

    // Ajoute le OU si il n'existe pas
    if (! ouexist($config, $dn)) {
        // Prépare les données
        $info = array();
        $info[$prefix] = $ou;
        // $info["name"] = "$ou";
        $info["objectClass"] = "organizationalUnit";
        // Ajout
        $ds = bind_ad_gssapi($config);
        ldap_add($ds, $dn, $info);
        $err = ldap_error($ds);
        ldap_close($ds);
        if (ouexist($config, $dn)) {
            return true;
        } else {
            return false;
        }
    } else {
        // le OU existe déja
        return true;
    }
}

/**
 * Suppression OU
 *
 * @description : supprime l'ou dans l'AD
 * @paramètres $config, $ou, $ouparent_rdn, $prefixe = ou, ou alors $ou=dn complet
 * @Returns true ou false.
 */
function oudel($config, $ou, $ouparent_rdn = "", $prefix = "ou")
{
    /*
     * Return true if OU is remove false in other cases
     */
    if (empty($ouparent_rdn)) {
        $dn = $ou;
        $ou_parent = ldap_dn2oudn($dn);
        $ou = ldap_dn2ou($dn);
        list ($prefix, $ou) = explode("=", $ou);
    } else {
        $ou_parent = $ouparent_rdn . "," . $config['ldap_base_dn'];
        $dn = $prefix . "=" . $ou . "," . $ou_parent;
    }

    if (ouexist($config, $dn)) {
        $ds = bind_ad_gssapi();
        // Verifier si le OU est vide !
        if (! ouexist($config, $dn)) {
            // On efface le OU
            ldap_delete($ds, $dn);
            ldap_close($ds);
        }
        if (! ouexist($ou, $dn)) {
            return true;
        } else {
            return false;
        }
    } else {
        return true;
    }
}

/*
 * Samba-Tool
 * Available subcommands:
 * add - Creates a new AD group.
 * addmembers - Add members to an AD group.
 * delete - Deletes an AD group.
 * list - List all groups.
 * listmembers - List all members of an AD group.
 * removemembers - Remove members from an AD group.
 */

/**
 * Listing d'un groupe
 *
 * @description : donne la liste des membres d'un groupe
 * ne fonctionne pas pour les machines !
 * @paramètres $config, $cn
 * @Returns array() ou false.
 */
function grouplistmembers($config, $cn)
{

    /*
     * Return array of member's cn
     */
    $message = array();
    $command = "group listmembers '$cn'";
    $res = sambatool($config, $command, $message);

    if ($res == 0)
        return $message;
    else
        return false;
}

/**
 * test existence groupe
 *
 * @description : vérifie si le groupe existe dans l'AD
 * @paramètres $config, $cn
 * @Returns true ou false.
 */
function groupexist($config, $cn)
{

    /*
     * Return true if cn group exist
     */
    $message = array();
    $command = "group list ";
    $RES = sambatool($config, $command, $message);
    if ($RES == 0) {
        $key = array_search($cn, $message);
        if (! empty($key))
            return true;
        else
            return false;
    } else
        return false;
}

/**
 * Ajout groupe
 *
 * @description : ajoute un groupe dans l'AD
 * @paramètres $config, $cn : cn du groupe, exemple Classe_TARCU, $inou : ou rdn, $description : la description du groupe
 * @Returns true ou false.
 */
function groupadd($config, $cn, $inou, $description)
{
    if (! empty($cn) && ! empty($inou) && ! empty($description)) {

        // creation du ou si il n'existe pas
        // if ( !ouexist($inou,$dn["groups"]) ) {
        // ouadd ($inou, $dn["groups"]);
        // }
        $message = array();
        $command = "group add " . escapeshellarg($cn) . " --groupou=" . escapeshellarg($inou) . " --description=" . escapeshellarg($description);
        $RES = sambatool($config, $command, $message);
        if ($RES == 0)
            return true;
        else
            return false;
    } else {
        return false;
    }
}

/**
 * Suppression d'un groupe
 *
 * @description Principe : supprime un groupe dans l'AD avec samba-tool group delete
 * @exemple groupdel($config, $cn)
 * @parametres $cn : cn du groupe a supprimer
 * @returns true if group is delete false in other cases
 */
function groupdel($config, $cn)
{
    $message = array();
    $command = "group delete " . escapeshellarg($cn);
    $RES = sambatool($config, $command, $message);

    if ($RES == 0) {
        $group = explode(" ", $message[0]);
        if ($group[2] == $cn) {
            return true;
        } else {
            return false;
        }
    } else {
        return false;
    }
}

/**
 * Ajout de membres dans un groupe
 * * attention les machines ont un $ à la fin
 *
 * @description ajoute des membres à un groupe existant dans l'AD
 * @parametres $config, $cn, $ingroup
 * @returns true ou false
 */
function groupaddmember($config, $cn, $ingroup)
{
    // le groupe exist ?
    if (groupexist($config, $ingroup)) {
        // Ajout du cn in group
        $message = array();
        $command = "group addmembers " . escapeshellarg($ingroup) . " " . escapeshellarg($cn);
        $RES = sambatool($config, $command, $message);

        if ($RES == 0) {
            return true;
        } else {
            return false;
        }
    } else {
        return false;
    }
}

/**
 * Suppression de membres dans un groupe
 * attention les machines ont un $ à la fin
 *
 * @description supprime des membres à un groupe existant dans l'AD
 * @parametres $config, $cn, $ingroup
 * @returns true ou false
 */
function groupdelmember($config, $cn, $ingroup)
{
    // le groupe exist ?
    if (groupexist($config, $ingroup)) {
        // Remove du cn in group
        $message = array();
        $command = "group removemembers " . escapeshellarg($ingroup) . " " . escapeshellarg($cn);
        $RES = sambatool($config, $command, $message);

        if ($RES == 0) {
            return true;
        } else {
            return false;
        }
    } else {
        return false;
    }
}

/**
 *
 * @description retourne la liste des gpo active pour l'utilisateur ou la machine
 *
 * @paramètres array $config, string $cn : utilisateur, machine...
 * @returns array liste des gpo [ uuid, displayname ]
 */
function gpolist(array $config, string $cn)
{
    $message = array();
    $command = "gpo list " . escapeshellarg($cn);
    $RES = sambatool($config, $command, $message);
    $gpo = array();
    $match = array();
    if ($RES == 0) {
        foreach ($message as $key => $ligne) {
            preg_match("/^(.*)\s({.*})$/", $ligne, $match);
            if (isset($match[1]) && isset($match[2])) {
                $gpo[$key]['uuid'] = $match[2];
                $gpo[$key]['displayname'] = $match[1];
            }
        }
    }
    return $gpo;
}

function gposetlink(array $config, string $container, string $gpo, bool $enforce = false, bool $disable = false)
{
    $message = array();
    $command = "gpo getlink " . escapeshellarg($container);
    $RES = sambatool($config, $command, $message);
    if ($RES != 0) {
        $attrs = array(
            'gplink' => array()
        );
        $ds = bind_ad_gssapi($config);
        $res = ldap_mod_del($ds, $container, $attrs);
        ldap_close($ds);
    }

    $command = "gpo setlink " . escapeshellarg($container) . " " . escapeshellarg($gpo);
    if ($enforce)
        $command .= " --enforce";
    if ($disable)
        $command .= " --disable";
    $RES = sambatool($config, $command, $message);
    if ($RES == 0) {
        return true;
    }
    return false;
}

function gpolistcontainers(array $config, string $gpo)
{
    $message = array();
    $command = "gpo listcontainers " . escapeshellarg($gpo);
    $RES = sambatool($config, $command, $message);
    $ou = array();
    if ($RES == 0) {
        $key = 0;
        foreach ($message as $ligne) {
            $match = array();
            if (preg_match("/^\s*dn: (.*)$/", strtolower($ligne), $match)) {
                $ou[] = $match[1];
            }
        }
    }
    return $ou;
}

function gpogetlink(array $config, string $container)
{
    $message = array();
    $command = "gpo getlink " . escapeshellarg($container);
    $RES = sambatool($config, $command, $message);
    $gpo = array();
    if ($RES == 0) {
        $key = 0;
        foreach ($message as $ligne) {
            $match = array();
            if (preg_match("/^\s*(.*)\s*\:\s*(.*)$/", $ligne, $match)) {
                if (trim($match[1]) == "GPO") {
                    $gpo[$key]['uuid'] = $match[2];
                } elseif (trim($match[1]) == "Name") {
                    $gpo[$key]['displayname'] = $match[2];
                } elseif (trim($match[1]) == "Options") {
                    $gpo[$key]['options'] = $match[2];
                    $key ++;
                }
            }
        }
    }
    return $gpo;
}

function gpodellink(array $config, string $container, string $gpo)
{
    $message = array();
    $command = "gpo dellink " . escapeshellarg($container) . " " . escapeshellarg($gpo);
    $RES = sambatool($config, $command, $message);
    if (($RES == 0) || ($RES == 255)) {
        return true;
    }
    return false;
}

function gposetinheritance(array $config, string $container, bool $flag = true)
{
    $message = array();
    $command = "gpo setinheritance " . escapeshellarg($container);
    if ($flag)
        $message .= " inherit";
    else
        $message .= " block";
    $RES = sambatool($config, $command, $message);
    if ($RES == 0) {
        return true;
    }
    return false;
}

function gpogetinheritance(array $config, string $container)
{
    $message = array();
    $command = "gpo getinheritance " . escapeshellarg($container);
    $RES = sambatool($config, $command, $message);
    if ($RES == 0) {
        if (preg_match("/GPO_INHERIT/", $message[0]))
            return "inherit";
        else
            return "block";
    }
    return false;
}

function gpofetch(array $config, string $gpo, string $dir = "/var/www/sambaedu/temp/policies")
{
    $message = array();
    $command = "gpo fetch " . escapeshellarg($gpo) . " --tmpdir=" . escapeshellarg($dir);
    $RES = sambatool($config, $command, $message);
    if ($RES == 0) {
        return true;
    }
    return false;
}

function gpocreate(array $config, string $displayname)
{
    $message = $match = array();
    $command = "gpo create " . escapeshellarg($displayname);
    $RES = sambatool($config, $command, $message);
    if ($RES == 0) {
        preg_match("/^.*\s({.*})$/i", $message[0], $match);
        return $match[1];
    }
    return false;
}

function gpodel(array $config, string $gpo)
{
    $message = array();
    $command = "gpo del " . escapeshellarg($gpo);
    $RES = sambatool($config, $command, $message);
    if (($RES == 0) || ($RES == 255)) {
        return true;
    }
    return false;
}

function dsacl_set(array $config, string $dn, string $sddl)
{
    $message = array();
    $command = "dsacl set --objectdn=" . escapeshellarg($dn) . " --sddl=" . escapeshellarg($sddl);
    $RES = sambatool($config, $command, $message);
    if (($RES == 0) || ($RES == 255)) {
        return true;
    }
    return false;
}

function dns_delete($config, $machine, $force = false)
{
    $RET = 0;
    $res = true;
    putenv('RES_OPTIONS=retrans:1 retry:1 timeout:1 attempts:1');
    if (isset($machine['dnshostname'])) {
        if ($As = gethostbynamel($machine['dnshostname'])) {
            foreach ($As as $A) {
                if (isset($machine['dnshostname']) && isset($machine['iphostnumber']) && $A != $machine['dnshostname'] && ($machine['iphostnumber'] != $A || $force)) {
                    $message = array();
                    $command = "dns delete " . $config['se4ad_name'] . " " . $config['domain'] . " " . $machine['cn'] . " A " . $A;
                    exec("/usr/bin/samba-tool " . $command, $message, $RET);
                }
                if ($RET == 0) {
                    $res &= true;
                }
            }
        }
    }
    return $res;
}
function dns_wpad($config)
{
    putenv('RES_OPTIONS=retrans:1 retry:1 timeout:1 attempts:1');
    $message = array();
    if (! empty($config['proxy_url']))
    { 
        $command = "dns add " . $config['se4ad_name'] . " " . $config['domain'] . " " . "wpad CNAME " . explode("/", $config['proxy_url'])[2] . ".";
    }
    else {
        $command = $command = "dns query " . $config['se4ad_name'] . " " . $config['domain'] . " " . "wpad  CNAME";
        $wpad = explode(" ", trim(exec("/usr/bin/samba-tool " . $command)))[1];
        $command = "dns delete " . $config['se4ad_name'] . " " . $config['domain'] . " " . "wpad" . " CNAME " . $wpad;
    }
    exec("/usr/bin/samba-tool " . $command, $message, $RET);
    if ($RET == 0) {
        return true;
    }
    return false;
}
?>
